@@ -12,6 +12,7 @@ module.metadata = {
 };
 
 const { Cc, Ci } = require("chrome");
+const { on } = require("sdk/event/core");
 const { setTimeout } = require("sdk/timers");
 const { LoaderWithHookedConsole } = require("sdk/test/loader");
 const { Worker } = require("sdk/content/worker");
@@ -25,7 +26,13 @@ const DEPRECATE_PREF = "devtools.errorconsole.deprecation_warnings";
 
 const DEFAULT_CONTENT_URL = "data:text/html;charset=utf-8,foo";
 
-function makeWindow(contentURL) {
+const WINDOW_SCRIPT_URL = "data:text/html;charset=utf-8," +
+                          "<script>window.addEventListener('message', function (e) {" +
+                          "  if (e.data === 'from -> content-script')" +
+                          "    window.postMessage('from -> window', '*');" +
+                          "});</script>";
+
+function makeWindow() {
   let content =
     "<?xml version=\"1.0\"?>" +
     "<window " +
@@ -116,6 +123,8 @@ exports["test:sample"] = WorkerTest(
 
     assert.equal(worker.url, window.location.href,
                      "worker.url works");
+    assert.equal(worker.contentURL, window.location.href,
+                     "worker.contentURL works");
     worker.postMessage("hi!");
   }
 );
@@ -226,7 +235,7 @@ exports["test:post-json-values-only"] = WorkerTest(
             self.postMessage([ message.fun === undefined,
                                typeof message.w,
                                message.w && "port" in message.w,
-                               message.w.url,
+                               message.w._url,
                                Array.isArray(message.array),
                                JSON.stringify(message.array)]);
           });
@@ -247,6 +256,10 @@ exports["test:post-json-values-only"] = WorkerTest(
                        "Array is correctly serialized");
       done();
     });
+    // Add a new url property sa the Class function used by
+    // Worker doesn't set enumerables to true for non-functions
+    worker._url = DEFAULT_CONTENT_URL;
+
     worker.postMessage({ fun: function () {}, w: worker, array: array });
   }
 );
@@ -264,7 +277,7 @@ exports["test:emit-json-values-only"] = WorkerTest(
                             fun === null,
                             typeof w,
                             "port" in w,
-                            w.url,
+                            w._url,
                             "fun" in obj,
                             Object.keys(obj.dom).length,
                             Array.isArray(array),
@@ -295,6 +308,9 @@ exports["test:emit-json-values-only"] = WorkerTest(
       fun: function () {},
       dom: browser.contentWindow.document.createElement("div")
     };
+    // Add a new url property sa the Class function used by
+    // Worker doesn't set enumerables to true for non-functions
+    worker._url = DEFAULT_CONTENT_URL;
     worker.port.emit("addon-to-content", function () {}, worker, obj, array);
   }
 );
@@ -372,8 +388,8 @@ exports["test:ensure console.xxx works in cs"] = WorkerTest(
     let calls = [];
     function onMessage(type, msg) {
       assert.equal(type, msg,
-                       "console.xxx(\"xxx\"), i.e. message is equal to the " +
-                       "console method name we are calling");
+        "console.xxx(\"xxx\"), i.e. message is equal to the " +
+        "console method name we are calling");
       calls.push(msg);
     }
 
@@ -381,19 +397,23 @@ exports["test:ensure console.xxx works in cs"] = WorkerTest(
     let worker =  loader.require("sdk/content/worker").Worker({
       window: browser.contentWindow,
       contentScript: "new " + function WorkerScope() {
+        console.time("time");
         console.log("log");
         console.info("info");
         console.warn("warn");
         console.error("error");
         console.debug("debug");
         console.exception("exception");
+        console.timeEnd("timeEnd");
         self.postMessage();
       },
       onMessage: function() {
         // Ensure that console methods are called in the same execution order
+        const EXPECTED_CALLS = ["time", "log", "info", "warn", "error",
+          "debug", "exception", "timeEnd"];
         assert.equal(JSON.stringify(calls),
-                         JSON.stringify(["log", "info", "warn", "error", "debug", "exception"]),
-                         "console has been called successfully, in the expected order");
+          JSON.stringify(EXPECTED_CALLS),
+          "console methods have been called successfully, in expected order");
         done();
       }
     });
@@ -410,12 +430,12 @@ exports["test:setTimeout works with string argument"] = WorkerTest(
         // since we are inside ContentScriptScope function.
         // i'm NOT putting code-in-string inside code-in-string </YO DAWG>
         window.csVal = 13;
-        setTimeout("self.postMessage([" + 
-                      "csVal, " + 
-                      "window.docVal, " + 
-                      "'ContentWorker' in window, " + 
-                      "'UNWRAP_ACCESS_KEY' in window, " + 
-                      "'getProxyForObject' in window, " + 
+        setTimeout("self.postMessage([" +
+                      "csVal, " +
+                      "window.docVal, " +
+                      "'ContentWorker' in window, " +
+                      "'UNWRAP_ACCESS_KEY' in window, " +
+                      "'getProxyForObject' in window, " +
                     "])", 1);
       },
       contentScriptWhen: "ready",
@@ -457,7 +477,7 @@ exports["test:setInterval async Errors passed to .onError"] = WorkerTest(
       contentScriptWhen: "ready",
       onError: function(err) {
         count++;
-        assert.equal(err.message, "ubik", 
+        assert.equal(err.message, "ubik",
             "error (corectly) propagated  " + count + " time(s)");
         if (count >= 3) done();
       }
@@ -473,11 +493,11 @@ exports["test:setTimeout throws array, passed to .onError"] = WorkerTest(
       contentScript: "setTimeout(function() { throw ['array', 42] }, 1)",
       contentScriptWhen: "ready",
       onError: function(arr) {
-        assert.ok(isArray(arr), 
+        assert.ok(isArray(arr),
             "the type of thrown/propagated object is array");
-        assert.ok(arr.length==2, 
+        assert.ok(arr.length==2,
             "the propagated thrown array is the right length");
-        assert.equal(arr[1], 42, 
+        assert.equal(arr[1], 42,
             "element inside the thrown array correctly propagated");
         done();
       }
@@ -493,15 +513,15 @@ exports["test:setTimeout string arg with SyntaxError to .onError"] = WorkerTest(
       contentScript: "setTimeout('syntax 123 error', 1)",
       contentScriptWhen: "ready",
       onError: function(err) {
-        assert.equal(err.name, "SyntaxError", 
+        assert.equal(err.name, "SyntaxError",
             "received SyntaxError thrown from bad code in string argument to setTimeout");
-        assert.ok('fileName' in err, 
+        assert.ok('fileName' in err,
             "propagated SyntaxError contains a fileName property");
-        assert.ok('stack' in err, 
+        assert.ok('stack' in err,
             "propagated SyntaxError contains a stack property");
-        assert.equal(err.message, "missing ; before statement", 
+        assert.equal(err.message, "missing ; before statement",
             "propagated SyntaxError has the correct (helpful) message");
-        assert.equal(err.lineNumber, 1, 
+        assert.equal(err.lineNumber, 1,
             "propagated SyntaxError was thrown on the right lineNumber");
         done();
       }
@@ -768,50 +788,6 @@ exports["test:check worker API with page history"] = WorkerTest(
   }
 );
 
-exports["test:global postMessage"] = WorkerTest(
-  DEFAULT_CONTENT_URL,
-  function(assert, browser, done) {
-    let { loader } = LoaderWithHookedConsole(module, onMessage);
-    setPref(DEPRECATE_PREF, true);
-
-    // Intercept all console method calls
-    let seenMessages = 0;
-    function onMessage(type, message) {
-      seenMessages++;
-      assert.equal(type, "error", "Should be an error");
-      assert.equal(message, "DEPRECATED: The global `postMessage()` function in " +
-                            "content scripts is deprecated in favor of the " +
-                            "`self.postMessage()` function, which works the same. " +
-                            "Replace calls to `postMessage()` with calls to " +
-                            "`self.postMessage()`." +
-                            "For more info on `self.on`, see " +
-                            "<https://addons.mozilla.org/en-US/developers/docs/sdk/latest/dev-guide/addon-development/web-content.html>.",
-                            "Should have seen the deprecation message")
-    }
-
-    assert.notEqual(browser.contentWindow.location.href, "about:blank",
-                        "window is now on the right document");
-
-    let window = browser.contentWindow
-    let worker = loader.require("sdk/content/worker").Worker({
-      window: window,
-      contentScript: "new " + function WorkerScope() {
-        postMessage("success");
-      },
-      contentScriptWhen: "ready",
-      onMessage: function(msg) {
-        assert.equal("success", msg, "Should have seen the right postMessage call");
-        assert.equal(1, seenMessages, "Should have seen the deprecation message");
-        done();
-      }
-    });
-
-    assert.equal(worker.url, window.location.href,
-                     "worker.url works");
-    worker.postMessage("hi!");
-  }
-);
-
 exports['test:conentScriptFile as URL instance'] = WorkerTest(
   DEFAULT_CONTENT_URL,
   function(assert, browser, done) {
@@ -821,7 +797,7 @@ exports['test:conentScriptFile as URL instance'] = WorkerTest(
       window: browser.contentWindow,
       contentScriptFile: url,
       onMessage: function(msg) {
-        assert.equal(msg, "msg from contentScriptFile", 
+        assert.equal(msg, "msg from contentScriptFile",
             "received a wrong message from contentScriptFile");
         done();
       }
@@ -829,4 +805,163 @@ exports['test:conentScriptFile as URL instance'] = WorkerTest(
   }
 );
 
+exports["test:worker events"] = WorkerTest(
+  DEFAULT_CONTENT_URL,
+  function (assert, browser, done) {
+    let window = browser.contentWindow;
+    let events = [];
+    let worker = Worker({
+      window: window,
+      contentScript: 'new ' + function WorkerScope() {
+        self.postMessage('start');
+      },
+      onAttach: win => {
+        events.push('attach');
+        assert.pass('attach event called when attached');
+        assert.equal(window, win, 'attach event passes in attached window');
+      },
+      onError: err => {
+        assert.equal(err.message, 'Custom',
+          'Error passed into error event');
+        worker.detach();
+      },
+      onMessage: msg => {
+        assert.pass('`onMessage` handles postMessage')
+        throw new Error('Custom');
+      },
+      onDetach: _ => {
+        assert.pass('`onDetach` called when worker detached');
+        done();
+      }
+    });
+    // `attach` event is called synchronously during instantiation,
+    // so we can't listen to that, TODO FIX?
+    //  worker.on('attach', obj => console.log('attach', obj));
+  }
+);
+
+exports["test:onDetach in contentScript on destroy"] = WorkerTest(
+  "data:text/html;charset=utf-8,foo#detach",
+  function(assert, browser, done) {
+    let worker = Worker({
+      window: browser.contentWindow,
+      contentScript: 'new ' + function WorkerScope() {
+        self.port.on('detach', function(reason) {
+          window.location.hash += '!' + reason;
+        })
+      },
+    });
+    browser.contentWindow.addEventListener('hashchange', _ => {
+      assert.equal(browser.contentWindow.location.hash, '#detach!',
+                   "location.href is as expected");
+      done();
+    })
+    worker.destroy();
+  }
+);
+
+exports["test:onDetach in contentScript on unload"] = WorkerTest(
+  "data:text/html;charset=utf-8,foo#detach",
+  function(assert, browser, done) {
+    let { loader } = LoaderWithHookedConsole(module);
+    let worker = loader.require("sdk/content/worker").Worker({
+      window: browser.contentWindow,
+      contentScript: 'new ' + function WorkerScope() {
+        self.port.on('detach', function(reason) {
+          window.location.hash += '!' + reason;
+        })
+      },
+    });
+    browser.contentWindow.addEventListener('hashchange', _ => {
+      assert.equal(browser.contentWindow.location.hash, '#detach!shutdown',
+                   "location.href is as expected");
+      done();
+    })
+    loader.unload('shutdown');
+  }
+);
+
+exports["test:console method log functions properly"] = WorkerTest(
+  DEFAULT_CONTENT_URL,
+  function(assert, browser, done) {
+    let logs = [];
+
+    let clean = message =>
+          message.trim().
+          replace(/[\r\n]/g, " ").
+          replace(/ +/g, " ");
+
+    let onMessage = (type, message) => logs.push(clean(message));
+    let { loader } = LoaderWithHookedConsole(module, onMessage);
+
+    let worker =  loader.require("sdk/content/worker").Worker({
+      window: browser.contentWindow,
+      contentScript: "new " + function WorkerScope() {
+        console.log(Function);
+        console.log((foo) => foo * foo);
+        console.log(function foo(bar) { return bar + bar });
+
+        self.postMessage();
+      },
+      onMessage: () => {
+        assert.deepEqual(logs, [
+          "function Function() { [native code] }",
+          "(foo) => foo * foo",
+          "function foo(bar) { \"use strict\"; return bar + bar }"
+        ]);
+
+        done();
+      }
+    });
+  }
+);
+
+exports["test:global postMessage"] = WorkerTest(
+  WINDOW_SCRIPT_URL,
+  function(assert, browser, done) {
+    let contentScript = "window.addEventListener('message', function (e) {" +
+                        "  if (e.data === 'from -> window')" +
+                        "    self.port.emit('response', e.data, e.origin);" +
+                        "});" +
+                        "postMessage('from -> content-script', '*');";
+    let { loader } = LoaderWithHookedConsole(module);
+    let worker =  loader.require("sdk/content/worker").Worker({
+      window: browser.contentWindow,
+      contentScriptWhen: "ready",
+      contentScript: contentScript
+    });
+
+    worker.port.on("response", (data, origin) => {
+      assert.equal(data, "from -> window", "Communication from content-script to window completed");
+      done();
+    });
+});
+
+exports["test:destroy unbinds listeners from port"] = WorkerTest(
+  "data:text/html;charset=utf-8,portdestroyer",
+  function(assert, browser, done) {
+    let destroyed = false;
+    let worker = Worker({
+      window: browser.contentWindow,
+      contentScript: "new " + function WorkerScope() {
+        self.port.emit("destroy");
+        setInterval(self.port.emit, 10, "ping");
+      },
+      onDestroy: done
+    });
+    worker.port.on("ping", () => {
+      if (destroyed) {
+        assert.fail("Should not call events on port after destroy.");
+      }
+    });
+    worker.port.on("destroy", () => {
+      destroyed = true;
+      worker.destroy();
+      assert.pass("Worker destroyed, waiting for no future listeners handling events.");
+      setTimeout(done, 500);
+    });
+  }
+);
+
+
 require("test").run(exports);
