@@ -1,4 +1,4 @@
-/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- /
+/* -*- indent-tabs-mode: nil; js-indent-level: 2 -*- /
 /* vim: set shiftwidth=2 tabstop=2 autoindent cindent expandtab: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this file,
@@ -30,6 +30,10 @@ function LOG(s) {
   dump("== Payment flow == " + s + "\n");
 }
 
+function LOGE(s) {
+  dump("== Payment flow ERROR == " + s + "\n");
+}
+
 if (_debug) {
   LOG("Frame script injected");
 }
@@ -42,7 +46,14 @@ XPCOMUtils.defineLazyServiceGetter(this, "uuidgen",
                                    "@mozilla.org/uuid-generator;1",
                                    "nsIUUIDGenerator");
 
+XPCOMUtils.defineLazyModuleGetter(this, "SystemAppProxy",
+                                  "resource://gre/modules/SystemAppProxy.jsm");
+
 #ifdef MOZ_B2G_RIL
+XPCOMUtils.defineLazyServiceGetter(this, "gRil",
+                                   "@mozilla.org/ril;1",
+                                   "nsIRadioInterfaceLayer");
+
 XPCOMUtils.defineLazyServiceGetter(this, "iccProvider",
                                    "@mozilla.org/ril/content-helper;1",
                                    "nsIIccProvider");
@@ -59,6 +70,7 @@ const kSilentSmsReceivedTopic = "silent-sms-received";
 const kMozSettingsChangedObserverTopic = "mozsettings-changed";
 
 const kRilDefaultDataServiceId = "ril.data.defaultServiceId";
+const kRilDefaultPaymentServiceId = "ril.payment.defaultServiceId";
 
 const MOBILEMESSAGECALLBACK_CID =
   Components.ID("{b484d8c9-6be4-4f94-ab60-c9c7ebcc853d}");
@@ -71,8 +83,8 @@ function SilentSmsRequest() {
 
 SilentSmsRequest.prototype = {
   __exposedProps__: {
-    onsuccess: 'rw',
-    onerror: 'rw'
+    onsuccess: "rw",
+    onerror: "rw"
   },
 
   QueryInterface: XPCOMUtils.generateQI([Ci.nsIMobileMessageCallback]),
@@ -95,33 +107,69 @@ SilentSmsRequest.prototype = {
   },
 
   notifySendMessageFailed: function notifySendMessageFailed(aError) {
-    if (_debug) {
-      LOG("Error sending silent message " + aError);
-    }
+    LOGE("Error sending silent message " + aError);
     this._onerror(aError);
   }
 };
 
 function PaymentSettings() {
-  this.dataServiceId = 0;
   Services.obs.addObserver(this, kMozSettingsChangedObserverTopic, false);
-  gSettingsService.createLock().get(kRilDefaultDataServiceId, this);
+
+  [kRilDefaultDataServiceId, kRilDefaultPaymentServiceId].forEach(setting => {
+    gSettingsService.createLock().get(setting, this);
+  });
 }
 
 PaymentSettings.prototype = {
   QueryInterface: XPCOMUtils.generateQI([Ci.nsISettingsServiceCallback,
                                          Ci.nsIObserver]),
 
+  dataServiceId: 0,
+  _paymentServiceId: 0,
+
+  get paymentServiceId() {
+    return this._paymentServiceId;
+  },
+
+  set paymentServiceId(serviceId) {
+    // We allow the payment provider to set the service ID that will be used
+    // for the payment process.
+    // This service ID will be the one used by the silent SMS flow.
+    // If the payment is done with an external SIM, the service ID must be set
+    // to null.
+    if (serviceId != null && serviceId >= gRil.numRadioInterfaces) {
+      LOGE("Invalid service ID " + serviceId);
+      return;
+    }
+
+    gSettingsService.createLock().set(kRilDefaultPaymentServiceId,
+                                      serviceId, null);
+    this._paymentServiceId = serviceId;
+  },
+
+  setServiceId: function(aName, aValue) {
+    switch (aName) {
+      case kRilDefaultDataServiceId:
+        this.dataServiceId = aValue;
+        if (_debug) {
+          LOG("dataServiceId " + this.dataServiceId);
+        }
+        break;
+      case kRilDefaultPaymentServiceId:
+        this._paymentServiceId = aValue;
+        if (_debug) {
+          LOG("paymentServiceId " + this._paymentServiceId);
+        }
+        break;
+    }
+  },
+
   handle: function(aName, aValue) {
     if (aName != kRilDefaultDataServiceId) {
       return;
     }
 
-    this.dataServiceId = aValue;
-
-    if (_debug) {
-      LOG("dataServiceId " + this.dataServiceId);
-    }
+    this.setServiceId(aName, aValue);
   },
 
   observe: function(aSubject, aTopic, aData) {
@@ -131,21 +179,15 @@ PaymentSettings.prototype = {
 
     try {
       let setting = JSON.parse(aData);
-      if (!setting.key || setting.key !== kRilDefaultDataServiceId) {
+      if (!setting.key ||
+          (setting.key !== kRilDefaultDataServiceId &&
+           setting.key !== kRilDefaultPaymentServiceId)) {
         return;
       }
-
-      this.dataServiceId = setting.value;
-
-      if (_debug) {
-        LOG("dataServiceId " + setting.value);
-      }
+      this.setServiceId(setting.key, setting.value);
     } catch (e) {
-      if (_debug) {
-        LOG(e);
-      }
+      LOGE(e);
     }
-
   },
 
   cleanup: function() {
@@ -158,24 +200,21 @@ const kClosePaymentFlowEvent = "close-payment-flow-dialog";
 
 let gRequestId;
 
-let gBrowser = Services.wm.getMostRecentWindow("navigator:browser");
-
 let PaymentProvider = {
 #ifdef MOZ_B2G_RIL
   __exposedProps__: {
-    paymentSuccess: 'r',
-    paymentFailed: 'r',
-    iccIds: 'r',
-    mcc: 'r',
-    mnc: 'r',
-    sendSilentSms: 'r',
-    observeSilentSms: 'r',
-    removeSilentSmsObserver: 'r'
+    paymentSuccess: "r",
+    paymentFailed: "r",
+    paymentServiceId: "rw",
+    iccInfo: "r",
+    sendSilentSms: "r",
+    observeSilentSms: "r",
+    removeSilentSmsObserver: "r"
   },
 #else
   __exposedProps__: {
-    paymentSuccess: 'r',
-    paymentFailed: 'r'
+    paymentSuccess: "r",
+    paymentFailed: "r"
   },
 #endif
 
@@ -191,11 +230,6 @@ let PaymentProvider = {
     // payment flow dialog and return to the caller application.
     let id = kClosePaymentFlowEvent + "-" + uuidgen.generateUUID().toString();
 
-    let content = gBrowser.getContentWindow();
-    if (!content) {
-      return;
-    }
-
     let detail = {
       type: kClosePaymentFlowEvent,
       id: id,
@@ -206,13 +240,13 @@ let PaymentProvider = {
     // it has successfully closed the payment flow and has recovered the
     // caller app, before notifying the parent process to fire the success
     // or error event over the DOMRequest.
-    content.addEventListener("mozContentEvent",
-                             function closePaymentFlowReturn(evt) {
+    SystemAppProxy.addEventListener("mozContentEvent",
+                               function closePaymentFlowReturn(evt) {
       if (evt.detail.id == id && aCallback) {
         aCallback();
       }
 
-      content.removeEventListener("mozContentEvent",
+      SystemAppProxy.removeEventListener("mozContentEvent",
                                   closePaymentFlowReturn);
 
       let glue = Cc["@mozilla.org/payment/ui-glue;1"]
@@ -220,7 +254,7 @@ let PaymentProvider = {
       glue.cleanup();
     });
 
-    gBrowser.shell.sendChromeEvent(detail);
+    SystemAppProxy.dispatchEvent(detail);
 
 #ifdef MOZ_B2G_RIL
     this._cleanUp();
@@ -242,9 +276,7 @@ let PaymentProvider = {
   },
 
   paymentFailed: function paymentFailed(aErrorMsg) {
-    if (_debug) {
-      LOG("paymentFailed " + aErrorMsg);
-    }
+    LOGE("paymentFailed " + aErrorMsg);
 
     PaymentProvider._closePaymentFlowDialog(function notifyError() {
       if (!gRequestId) {
@@ -256,22 +288,45 @@ let PaymentProvider = {
   },
 
 #ifdef MOZ_B2G_RIL
-  // Bug 938993. Support Multi-SIM for Payments.
+  get paymentServiceId() {
+    return this._settings.paymentServiceId;
+  },
+
+  set paymentServiceId(serviceId) {
+    this._settings.paymentServiceId = serviceId;
+  },
+
+  // We expose to the payment provider the information of all the SIMs
+  // available in the device. iccInfo is an object of this form:
+  //  {
+  //    "serviceId1": {
+  //       mcc: <string>,
+  //       mnc: <string>,
+  //       iccId: <string>,
+  //       dataPrimary: <boolean>
+  //     },
+  //    "serviceIdN": {...}
+  //  }
   get iccInfo() {
-    delete this.iccInfo;
-    return this.iccInfo = iccProvider.getIccInfo(this._settings.dataServiceId);
-  },
+    if (!this._iccInfo) {
+      this._iccInfo = {};
+      for (let i = 0; i < gRil.numRadioInterfaces; i++) {
+        let info = iccProvider.getIccInfo(i);
+        if (!info) {
+          LOGE("Tried to get the ICC info for an invalid service ID " + i);
+          continue;
+        }
 
-  get iccIds() {
-    return [this.iccInfo.iccid];
-  },
+        this._iccInfo[i] = {
+          iccId: info.iccid,
+          mcc: info.mcc,
+          mnc: info.mnc,
+          dataPrimary: i == this._settings.dataServiceId
+        };
+      }
+    }
 
-  get mcc() {
-    return [this.iccInfo.mcc];
-  },
-
-  get mnc() {
-    return [this.iccInfo.mnc];
+    return Cu.cloneInto(this._iccInfo, content);
   },
 
   _silentNumbers: null,
@@ -283,7 +338,21 @@ let PaymentProvider = {
     }
 
     let request = new SilentSmsRequest();
-    smsService.send(aNumber, aMessage, true, request);
+
+    if (this._settings.paymentServiceId === null) {
+      LOGE("No payment service ID set. Cannot send silent SMS");
+      let runnable = {
+        run: function run() {
+          request.notifySendMessageFailed("NO_PAYMENT_SERVICE_ID");
+        }
+      };
+      Services.tm.currentThread.dispatch(runnable,
+                                         Ci.nsIThread.DISPATCH_NORMAL);
+      return request;
+    }
+
+    smsService.send(this._settings.paymentServiceId, aNumber, aMessage, true,
+                    request);
     return request;
   },
 
@@ -349,6 +418,21 @@ let PaymentProvider = {
       return;
     }
 
+    // If the service ID is null it means that the payment provider asked the
+    // user for her MSISDN, so we are in a MT only SMS auth flow. In this case
+    // we manually set the service ID to the one corresponding with the SIM
+    // that received the SMS.
+    if (this._settings.paymentServiceId === null) {
+      let i = 0;
+      while(i < gRil.numRadioInterfaces) {
+        if (this.iccInfo[i].iccId === aSubject.iccId) {
+          this._settings.paymentServiceId = i;
+          break;
+        }
+        i++;
+      }
+    }
+
     this._silentSmsObservers[number].forEach(function(callback) {
       callback(aSubject);
     });
@@ -390,7 +474,7 @@ addEventListener("DOMWindowCreated", function(e) {
 // If the trusted dialog is not closed via paymentSuccess or paymentFailed
 // a mozContentEvent with type 'cancel' is sent from the UI. We need to listen
 // for this event to clean up the silent sms observers if any exists.
-gBrowser.getContentWindow().addEventListener("mozContentEvent", function(e) {
+SystemAppProxy.addEventListener("mozContentEvent", function(e) {
   if (e.detail.type === "cancel") {
     PaymentProvider._cleanUp();
   }
