@@ -5,12 +5,12 @@
 MOZ_APP_BASENAME=B2G
 MOZ_APP_VENDOR=Mozilla
 
-MOZ_APP_VERSION=28.0
+MOZ_APP_VERSION=33.0
 MOZ_APP_UA_NAME=Firefox
 
 MOZ_UA_OS_AGNOSTIC=1
 
-MOZ_B2G_VERSION=1.3.0.0-prerelease
+MOZ_B2G_VERSION=2.1.0.0-prerelease
 MOZ_B2G_OS_NAME=Boot2Gecko
 
 MOZ_BRANDING_DIRECTORY=b2g/branding/unofficial
@@ -26,6 +26,8 @@ MOZ_WEBSMS_BACKEND=1
 MOZ_DISABLE_CRYPTOLEGACY=1
 MOZ_APP_STATIC_INI=1
 NSS_NO_LIBPKIX=1
+NSS_DISABLE_DBM=1
+MOZ_NO_EV_CERTS=1
 MOZ_DISABLE_EXPORT_JS=1
 
 if test "$OS_TARGET" = "Android"; then
@@ -50,13 +52,21 @@ MOZ_EXTENSION_MANAGER=1
 
 MOZ_TIME_MANAGER=1
 
-MOZ_B2G_CERTDATA=1
 MOZ_PAY=1
 MOZ_TOOLKIT_SEARCH=
 MOZ_PLACES=
 MOZ_B2G=1
 
 if test "$OS_TARGET" = "Android"; then
-MOZ_NUWA_PROCESS=
+MOZ_NUWA_PROCESS=1
 fi
 MOZ_FOLD_LIBS=1
+
+MOZ_JSDOWNLOADS=1
+
+MOZ_BUNDLED_FONTS=1
+
+# Enable exact rooting on b2g.
+JSGC_USE_EXACT_ROOTING=1
+
+JS_GC_SMALL_CHUNK_SIZE=1
