@@ -247,7 +247,7 @@ var gPrivacyPane = {
   },
 
   _lastMode: null,
-  _lasCheckState: null,
+  _lastCheckState: null,
   updateAutostart: function PPP_updateAutostart() {
       let mode = document.getElementById("historyMode");
       let autoStart = document.getElementById("privateBrowsingAutoStart");
@@ -284,7 +284,6 @@ var gPrivacyPane = {
 
         if (shouldProceed) {
           pref.value = autoStart.hasAttribute('checked');
-          document.documentElement.acceptDialog();
           let appStartup = Cc["@mozilla.org/toolkit/app-startup;1"]
                              .getService(Ci.nsIAppStartup);
           appStartup.quit(Ci.nsIAppStartup.eAttemptQuit |  Ci.nsIAppStartup.eRestart);
@@ -299,6 +298,7 @@ var gPrivacyPane = {
       } else {
         autoStart.removeAttribute('checked');
       }
+      pref.value = autoStart.hasAttribute('checked');
       mode.selectedIndex = this._lastMode;
       mode.doCommand();
 
@@ -471,7 +471,7 @@ var gPrivacyPane = {
                    introText      : bundlePreferences.getString("cookiepermissionstext") };
     openDialog("chrome://browser/content/preferences/permissions.xul",
                "Browser:Permissions",
-               "model=yes", params);
+               "modal=yes", params);
   },
 
   /**
@@ -481,7 +481,7 @@ var gPrivacyPane = {
   {
     openDialog("chrome://browser/content/preferences/cookies.xul",
                "Browser:Cookies",
-               "model=yes", null);
+               "modal=yes", null);
   },
 
   // CLEAR PRIVATE DATA
@@ -500,7 +500,7 @@ var gPrivacyPane = {
   showClearPrivateDataSettings: function ()
   {
     openDialog("chrome://browser/content/preferences/sanitize.xul",
-               "model=yes", null);
+               "modal=yes", null);
   },
 
 
