@@ -5,13 +5,10 @@
 "use strict";
 
 const {Cu, Cc, Ci} = require("chrome");
-
-let promise = require("sdk/core/promise");
-let EventEmitter = require("devtools/shared/event-emitter");
-
-Cu.import('resource://gre/modules/XPCOMUtils.jsm');
-Cu.import("resource://gre/modules/Services.jsm");
-Cu.import("resource:///modules/devtools/gDevTools.jsm");
+const Services = require("Services");
+const promise = require("promise");
+Cu.import("resource://gre/modules/XPCOMUtils.jsm");
+XPCOMUtils.defineLazyModuleGetter(this, "gDevTools", "resource:///modules/devtools/gDevTools.jsm");
 
 exports.OptionsPanel = OptionsPanel;
 
@@ -31,6 +28,43 @@ XPCOMUtils.defineLazyGetter(this, "l10n", function() {
   return l10n;
 });
 
+function GetPref(name) {
+  let type = Services.prefs.getPrefType(name);
+  switch (type) {
+    case Services.prefs.PREF_STRING:
+      return Services.prefs.getCharPref(name);
+    case Services.prefs.PREF_INT:
+      return Services.prefs.getIntPref(name);
+    case Services.prefs.PREF_BOOL:
+      return Services.prefs.getBoolPref(name);
+    default:
+      throw new Error("Unknown type");
+  }
+}
+
+function SetPref(name, value) {
+  let type = Services.prefs.getPrefType(name);
+  switch (type) {
+    case Services.prefs.PREF_STRING:
+      return Services.prefs.setCharPref(name, value);
+    case Services.prefs.PREF_INT:
+      return Services.prefs.setIntPref(name, value);
+    case Services.prefs.PREF_BOOL:
+      return Services.prefs.setBoolPref(name, value);
+    default:
+      throw new Error("Unknown type");
+  }
+}
+
+function InfallibleGetBoolPref(key) {
+  try {
+    return Services.prefs.getBoolPref(key);
+  } catch (ex) {
+    return true;
+  }
+}
+
+
 /**
  * Represents the Options Panel in the Toolbox.
  */
@@ -40,8 +74,13 @@ function OptionsPanel(iframeWindow, toolbox) {
   this.toolbox = toolbox;
   this.isReady = false;
 
+  this._prefChanged = this._prefChanged.bind(this);
+
+  this._addListeners();
+
+  const EventEmitter = require("devtools/toolkit/event-emitter");
   EventEmitter.decorate(this);
-};
+}
 
 OptionsPanel.prototype = {
 
@@ -50,20 +89,77 @@ OptionsPanel.prototype = {
   },
 
   open: function() {
-    let deferred = promise.defer();
+    let targetPromise;
 
-    this.setupToolsList();
-    this.populatePreferences();
+    // For local debugging we need to make the target remote.
+    if (!this.target.isRemote) {
+      targetPromise = this.target.makeRemote();
+    } else {
+      targetPromise = promise.resolve(this.target);
+    }
 
-    this._disableJSClicked = this._disableJSClicked.bind(this);
+    return targetPromise.then(() => {
+      this.setupToolsList();
+      this.setupToolbarButtonsList();
+      this.populatePreferences();
 
-    let disableJSNode = this.panelDoc.getElementById("devtools-disable-javascript");
-    disableJSNode.addEventListener("click", this._disableJSClicked, false);
+      this._disableJSClicked = this._disableJSClicked.bind(this);
 
-    this.isReady = true;
-    this.emit("ready");
-    deferred.resolve(this);
-    return deferred.promise;
+      let disableJSNode = this.panelDoc.getElementById("devtools-disable-javascript");
+      disableJSNode.addEventListener("click", this._disableJSClicked, false);
+    }).then(() => {
+      this.isReady = true;
+      this.emit("ready");
+      return this;
+    }).then(null, function onError(aReason) {
+      Cu.reportError("OptionsPanel open failed. " +
+                     aReason.error + ": " + aReason.message);
+    });
+  },
+
+  _addListeners: function() {
+    gDevTools.on("pref-changed", this._prefChanged);
+  },
+
+  _removeListeners: function() {
+    gDevTools.off("pref-changed", this._prefChanged);
+  },
+
+  _prefChanged: function(event, data) {
+    if (data.pref === "devtools.cache.disabled") {
+      let cacheDisabled = data.newValue;
+      let cbx = this.panelDoc.getElementById("devtools-disable-cache");
+
+      cbx.checked = cacheDisabled;
+    }
+  },
+
+  setupToolbarButtonsList: function() {
+    let enabledToolbarButtonsBox = this.panelDoc.getElementById("enabled-toolbox-buttons-box");
+    enabledToolbarButtonsBox.textContent = "";
+
+    let toggleableButtons = this.toolbox.toolboxButtons;
+    let setToolboxButtonsVisibility =
+      this.toolbox.setToolboxButtonsVisibility.bind(this.toolbox);
+
+    let onCheckboxClick = (checkbox) => {
+      let toolDefinition = toggleableButtons.filter(tool => tool.id === checkbox.id)[0];
+      SetPref(toolDefinition.visibilityswitch, checkbox.checked);
+      setToolboxButtonsVisibility();
+    };
+
+    let createCommandCheckbox = tool => {
+      let checkbox = this.panelDoc.createElement("checkbox");
+      checkbox.setAttribute("id", tool.id);
+      checkbox.setAttribute("label", tool.label);
+      checkbox.setAttribute("checked", InfallibleGetBoolPref(tool.visibilityswitch));
+      checkbox.addEventListener("command", onCheckboxClick.bind(this, checkbox));
+      return checkbox;
+    };
+
+    for (let tool of toggleableButtons) {
+      enabledToolbarButtonsBox.appendChild(createCommandCheckbox(tool));
+    }
   },
 
   setupToolsList: function() {
@@ -75,19 +171,10 @@ OptionsPanel.prototype = {
     defaultToolsBox.textContent = "";
     additionalToolsBox.textContent = "";
 
-    let pref = function(key) {
-      try {
-        return Services.prefs.getBoolPref(key);
-      }
-      catch (ex) {
-        return true;
-      }
-    };
-
     let onCheckboxClick = function(id) {
       let toolDefinition = gDevTools._tools.get(id);
       // Set the kill switch pref boolean to true
-      Services.prefs.setBoolPref(toolDefinition.visibilityswitch, this.checked);
+      SetPref(toolDefinition.visibilityswitch, this.checked);
       if (this.checked) {
         gDevTools.emit("tool-registered", id);
       }
@@ -109,15 +196,16 @@ OptionsPanel.prototype = {
                               l10n("options.toolNotSupportedMarker", tool.label));
         checkbox.setAttribute("unsupported", "");
       }
-      checkbox.setAttribute("checked", pref(tool.visibilityswitch));
+      checkbox.setAttribute("checked", InfallibleGetBoolPref(tool.visibilityswitch));
       checkbox.addEventListener("command", onCheckboxClick.bind(checkbox, tool.id));
       return checkbox;
     };
 
     // Populating the default tools lists
     let toggleableTools = gDevTools.getDefaultTools().filter(tool => {
-      return tool.visibilityswitch
+      return tool.visibilityswitch;
     });
+
     for (let tool of toggleableTools) {
       defaultToolsBox.appendChild(createToolCheckbox(tool));
     }
@@ -144,20 +232,20 @@ OptionsPanel.prototype = {
   populatePreferences: function() {
     let prefCheckboxes = this.panelDoc.querySelectorAll("checkbox[data-pref]");
     for (let checkbox of prefCheckboxes) {
-      checkbox.checked = Services.prefs.getBoolPref(checkbox.getAttribute("data-pref"));
+      checkbox.checked = GetPref(checkbox.getAttribute("data-pref"));
       checkbox.addEventListener("command", function() {
         let data = {
           pref: this.getAttribute("data-pref"),
           newValue: this.checked
         };
-        data.oldValue = Services.prefs.getBoolPref(data.pref);
-        Services.prefs.setBoolPref(data.pref, data.newValue);
+        data.oldValue = GetPref(data.pref);
+        SetPref(data.pref, data.newValue);
         gDevTools.emit("pref-changed", data);
       }.bind(checkbox));
     }
     let prefRadiogroups = this.panelDoc.querySelectorAll("radiogroup[data-pref]");
     for (let radiogroup of prefRadiogroups) {
-      let selectedValue = Services.prefs.getCharPref(radiogroup.getAttribute("data-pref"));
+      let selectedValue = GetPref(radiogroup.getAttribute("data-pref"));
       for (let radio of radiogroup.childNodes) {
         radiogroup.selectedIndex = -1;
         if (radio.getAttribute("value") == selectedValue) {
@@ -170,18 +258,18 @@ OptionsPanel.prototype = {
           pref: this.getAttribute("data-pref"),
           newValue: this.selectedItem.getAttribute("value")
         };
-        data.oldValue = Services.prefs.getCharPref(data.pref);
-        Services.prefs.setCharPref(data.pref, data.newValue);
+        data.oldValue = GetPref(data.pref);
+        SetPref(data.pref, data.newValue);
         gDevTools.emit("pref-changed", data);
       }.bind(radiogroup));
     }
     let prefMenulists = this.panelDoc.querySelectorAll("menulist[data-pref]");
     for (let menulist of prefMenulists) {
-      let pref = Services.prefs.getCharPref(menulist.getAttribute("data-pref"));
+      let pref = GetPref(menulist.getAttribute("data-pref"));
       let menuitems = menulist.querySelectorAll("menuitem");
       for (let menuitem of menuitems) {
-        let value = menuitem.getAttribute("value");
-        if (value === pref) {
+        let value = menuitem.value;
+        if (value == pref) { // non strict check to allow int values.
           menulist.selectedItem = menuitem;
           break;
         }
@@ -191,11 +279,22 @@ OptionsPanel.prototype = {
           pref: this.getAttribute("data-pref"),
           newValue: this.value
         };
-        data.oldValue = Services.prefs.getCharPref(data.pref);
-        Services.prefs.setCharPref(data.pref, data.newValue);
+        data.oldValue = GetPref(data.pref);
+        SetPref(data.pref, data.newValue);
         gDevTools.emit("pref-changed", data);
       }.bind(menulist));
     }
+
+    this.target.client.attachTab(this.target.activeTab._actor, (response) => {
+      this._origJavascriptEnabled = response.javascriptEnabled;
+
+      this._populateDisableJSCheckbox();
+    });
+  },
+
+  _populateDisableJSCheckbox: function() {
+    let cbx = this.panelDoc.getElementById("devtools-disable-javascript");
+    cbx.checked = !this._origJavascriptEnabled;
   },
 
   /**
@@ -210,22 +309,40 @@ OptionsPanel.prototype = {
    */
   _disableJSClicked: function(event) {
     let checked = event.target.checked;
-    let linkedBrowser = this.toolbox._host.hostTab.linkedBrowser;
-    let win = linkedBrowser.contentWindow;
-    let docShell = linkedBrowser.docShell;
 
-    if (typeof this.toolbox._origAllowJavascript == "undefined") {
-      this.toolbox._origAllowJavascript = docShell.allowJavascript;
-    }
+    let options = {
+      "javascriptEnabled": !checked
+    };
 
-    docShell.allowJavascript = !checked;
-    win.location.reload();
+    this.target.activeTab.reconfigure(options);
   },
 
-  destroy: function OP_destroy() {
+  destroy: function() {
+    if (this.destroyPromise) {
+      return this.destroyPromise;
+    }
+
+    let deferred = promise.defer();
+
+    this.destroyPromise = deferred.promise;
+
     let disableJSNode = this.panelDoc.getElementById("devtools-disable-javascript");
     disableJSNode.removeEventListener("click", this._disableJSClicked, false);
 
-    this.panelWin = this.panelDoc = this.toolbox = this._disableJSClicked = null;
+    this._removeListeners();
+
+    this.panelWin = this.panelDoc = null;
+    this._disableJSClicked = null;
+
+    // If JavaScript is disabled we need to revert it to it's original value.
+    let options = {
+      "javascriptEnabled": this._origJavascriptEnabled
+    };
+    this.target.activeTab.reconfigure(options, () => {
+      this.toolbox = null;
+      deferred.resolve();
+    }, true);
+
+    return deferred.promise;
   }
 };
