@@ -14,6 +14,7 @@
 
 namespace mozilla {
 
+class WebGLFramebufferAttachable;
 class WebGLTexture;
 class WebGLRenderbuffer;
 namespace gl {
@@ -27,11 +28,7 @@ class WebGLFramebuffer MOZ_FINAL
     , public WebGLContextBoundObject
 {
 public:
-    WebGLFramebuffer(WebGLContext *context);
-
-    ~WebGLFramebuffer() {
-        DeleteOnce();
-    }
+    WebGLFramebuffer(WebGLContext* context);
 
     struct Attachment
     {
@@ -41,9 +38,11 @@ public:
         GLenum mAttachmentPoint;
         GLenum mTexImageTarget;
         GLint mTexImageLevel;
+        mutable bool mNeedsFinalize;
 
         Attachment(GLenum aAttachmentPoint = LOCAL_GL_COLOR_ATTACHMENT0)
             : mAttachmentPoint(aAttachmentPoint)
+            , mNeedsFinalize(false)
         {}
 
         bool IsDefined() const {
@@ -53,22 +52,21 @@ public:
         bool IsDeleteRequested() const;
 
         bool HasAlpha() const;
+        bool IsReadableFloat() const;
 
-        void SetTexImage(WebGLTexture *tex, GLenum target, GLint level);
-        void SetRenderbuffer(WebGLRenderbuffer *rb) {
-            mTexturePtr = nullptr;
-            mRenderbufferPtr = rb;
-        }
-        const WebGLTexture *Texture() const {
+        void SetTexImage(WebGLTexture* tex, GLenum target, GLint level);
+        void SetRenderbuffer(WebGLRenderbuffer* rb);
+
+        const WebGLTexture* Texture() const {
             return mTexturePtr;
         }
-        WebGLTexture *Texture() {
+        WebGLTexture* Texture() {
             return mTexturePtr;
         }
-        const WebGLRenderbuffer *Renderbuffer() const {
+        const WebGLRenderbuffer* Renderbuffer() const {
             return mRenderbufferPtr;
         }
-        WebGLRenderbuffer *Renderbuffer() {
+        WebGLRenderbuffer* Renderbuffer() {
             return mRenderbufferPtr;
         }
         GLenum TexImageTarget() const {
@@ -86,12 +84,12 @@ public:
             mRenderbufferPtr = nullptr;
         }
 
-        const WebGLRectangleObject* RectangleObject() const;
-        bool HasSameDimensionsAs(const Attachment& other) const;
+        const WebGLRectangleObject& RectangleObject() const;
 
+        bool HasImage() const;
         bool IsComplete() const;
 
-        void FinalizeAttachment(GLenum attachmentLoc) const;
+        void FinalizeAttachment(gl::GLContext* gl, GLenum attachmentLoc) const;
     };
 
     void Delete();
@@ -103,15 +101,26 @@ public:
     void FramebufferRenderbuffer(GLenum target,
                                  GLenum attachment,
                                  GLenum rbtarget,
-                                 WebGLRenderbuffer *wrb);
+                                 WebGLRenderbuffer* wrb);
 
     void FramebufferTexture2D(GLenum target,
                               GLenum attachment,
                               GLenum textarget,
-                              WebGLTexture *wtex,
+                              WebGLTexture* wtex,
                               GLint level);
 
-    bool HasIncompleteAttachment() const;
+private:
+    void DetachAttachment(WebGLFramebuffer::Attachment& attachment);
+    void DetachAllAttachments();
+    const WebGLRectangleObject& GetAnyRectObject() const;
+    Attachment* GetAttachmentOrNull(GLenum attachment);
+
+public:
+    bool HasDefinedAttachments() const;
+    bool HasIncompleteAttachments() const;
+    bool AllImageRectsMatch() const;
+    GLenum PrecheckFramebufferStatus() const;
+    GLenum CheckFramebufferStatus() const;
 
     bool HasDepthStencilConflict() const {
         return int(mDepthAttachment.IsDefined()) +
@@ -119,12 +128,10 @@ public:
                int(mDepthStencilAttachment.IsDefined()) >= 2;
     }
 
-    bool HasAttachmentsOfMismatchedDimensions() const;
-
-    const size_t ColorAttachmentCount() const {
+    size_t ColorAttachmentCount() const {
         return mColorAttachments.Length();
     }
-    const Attachment& ColorAttachment(uint32_t colorAttachmentId) const {
+    const Attachment& ColorAttachment(size_t colorAttachmentId) const {
         return mColorAttachments[colorAttachmentId];
     }
 
@@ -142,35 +149,45 @@ public:
 
     const Attachment& GetAttachment(GLenum attachment) const;
 
-    void DetachTexture(const WebGLTexture *tex);
+    void DetachTexture(const WebGLTexture* tex);
 
-    void DetachRenderbuffer(const WebGLRenderbuffer *rb);
+    void DetachRenderbuffer(const WebGLRenderbuffer* rb);
 
-    const WebGLRectangleObject *RectangleObject() {
-        return mColorAttachments[0].RectangleObject();
-    }
+    const WebGLRectangleObject& RectangleObject() const;
 
-    WebGLContext *GetParentObject() const {
+    WebGLContext* GetParentObject() const {
         return Context();
     }
 
     void FinalizeAttachments() const;
 
-    virtual JSObject* WrapObject(JSContext *cx,
-                                 JS::Handle<JSObject*> scope) MOZ_OVERRIDE;
+    virtual JSObject* WrapObject(JSContext* cx) MOZ_OVERRIDE;
 
     NS_INLINE_DECL_CYCLE_COLLECTING_NATIVE_REFCOUNTING(WebGLFramebuffer)
     NS_DECL_CYCLE_COLLECTION_SCRIPT_HOLDER_NATIVE_CLASS(WebGLFramebuffer)
 
+    // mask mirrors glClear.
+    bool HasCompletePlanes(GLbitfield mask);
+
     bool CheckAndInitializeAttachments();
 
-    bool CheckColorAttachementNumber(GLenum attachment, const char * functionName) const;
+    bool CheckColorAttachmentNumber(GLenum attachment, const char* functionName) const;
+
+    void EnsureColorAttachments(size_t colorAttachmentId);
+
+    Attachment* AttachmentFor(GLenum attachment);
+    void NotifyAttachableChanged() const;
+
+private:
+    ~WebGLFramebuffer() {
+        DeleteOnce();
+    }
+
+    mutable GLenum mStatus;
 
     GLuint mGLName;
     bool mHasEverBeenBound;
 
-    void EnsureColorAttachments(size_t colorAttachmentId);
-
     // we only store pointers to attached renderbuffers, not to attached textures, because
     // we will only need to initialize renderbuffers. Textures are already initialized.
     nsTArray<Attachment> mColorAttachments;
