@@ -3,19 +3,23 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#include "WebGLContext.h"
 #include "WebGLTexture.h"
+
 #include "GLContext.h"
-#include "ScopedGLHelpers.h"
-#include "WebGLTexelConversions.h"
 #include "mozilla/dom/WebGLRenderingContextBinding.h"
+#include "mozilla/Scoped.h"
+#include "ScopedGLHelpers.h"
+#include "WebGLContext.h"
+#include "WebGLContextUtils.h"
+#include "WebGLTexelConversions.h"
+
 #include <algorithm>
 
 using namespace mozilla;
 
 JSObject*
-WebGLTexture::WrapObject(JSContext *cx, JS::Handle<JSObject*> scope) {
-    return dom::WebGLTextureBinding::Wrap(cx, scope, this);
+WebGLTexture::WrapObject(JSContext *cx) {
+    return dom::WebGLTextureBinding::Wrap(cx, this);
 }
 
 WebGLTexture::WebGLTexture(WebGLContext *context)
@@ -49,8 +53,8 @@ int64_t
 WebGLTexture::ImageInfo::MemoryUsage() const {
     if (mImageDataStatus == WebGLImageDataStatus::NoImageData)
         return 0;
-    int64_t texelSizeInBits = WebGLContext::GetBitsPerTexel(mInternalFormat, mType);
-    return int64_t(mWidth) * int64_t(mHeight) * texelSizeInBits / 8;
+    int64_t bitsPerTexel = WebGLContext::GetBitsPerTexel(mWebGLFormat, mWebGLType);
+    return int64_t(mWidth) * int64_t(mHeight) * bitsPerTexel/8;
 }
 
 int64_t
@@ -125,7 +129,7 @@ WebGLTexture::Bind(GLenum aTarget) {
         // thanks to the WebKit people for finding this out: GL_TEXTURE_WRAP_R is not
         // present in GLES 2, but is present in GL and it seems as if for cube maps
         // we need to set it to GL_CLAMP_TO_EDGE to get the expected GLES behavior.
-        if (mTarget == LOCAL_GL_TEXTURE_CUBE_MAP && !mContext->gl->IsGLES2())
+        if (mTarget == LOCAL_GL_TEXTURE_CUBE_MAP && !mContext->gl->IsGLES())
             mContext->gl->fTexParameteri(mTarget, LOCAL_GL_TEXTURE_WRAP_R, LOCAL_GL_CLAMP_TO_EDGE);
     }
 
@@ -147,6 +151,9 @@ WebGLTexture::SetImageInfo(GLenum aTarget, GLint aLevel,
     if (aLevel > 0)
         SetCustomMipmap();
 
+    // Invalidate framebuffer status cache
+    NotifyFBsStatusChanged();
+
     SetFakeBlackStatus(WebGLTextureFakeBlackStatus::Unknown);
 }
 
@@ -330,8 +337,8 @@ WebGLTexture::ResolvedFakeBlackStatus() {
         }
     }
 
-    if (ImageInfoBase().mType == LOCAL_GL_FLOAT &&
-        !Context()->IsExtensionEnabled(WebGLContext::OES_texture_float_linear))
+    if (ImageInfoBase().mWebGLType == LOCAL_GL_FLOAT &&
+        !Context()->IsExtensionEnabled(WebGLExtensionID::OES_texture_float_linear))
     {
         if (mMinFilter == LOCAL_GL_LINEAR ||
             mMinFilter == LOCAL_GL_LINEAR_MIPMAP_LINEAR ||
@@ -350,6 +357,26 @@ WebGLTexture::ResolvedFakeBlackStatus() {
                                       "Try enabling the OES_texture_float_linear extension if supported.", msg_rendering_as_black);
             mFakeBlackStatus = WebGLTextureFakeBlackStatus::IncompleteTexture;
         }
+    } else if (ImageInfoBase().mWebGLType == LOCAL_GL_HALF_FLOAT_OES &&
+               !Context()->IsExtensionEnabled(WebGLExtensionID::OES_texture_half_float_linear))
+    {
+        if (mMinFilter == LOCAL_GL_LINEAR ||
+            mMinFilter == LOCAL_GL_LINEAR_MIPMAP_LINEAR ||
+            mMinFilter == LOCAL_GL_LINEAR_MIPMAP_NEAREST ||
+            mMinFilter == LOCAL_GL_NEAREST_MIPMAP_LINEAR)
+        {
+            mContext->GenerateWarning("%s is a texture with a linear minification filter, "
+                                      "which is not compatible with gl.HALF_FLOAT by default. "
+                                      "Try enabling the OES_texture_half_float_linear extension if supported.", msg_rendering_as_black);
+            mFakeBlackStatus = WebGLTextureFakeBlackStatus::IncompleteTexture;
+        }
+        else if (mMagFilter == LOCAL_GL_LINEAR)
+        {
+            mContext->GenerateWarning("%s is a texture with a linear magnification filter, "
+                                      "which is not compatible with gl.HALF_FLOAT by default. "
+                                      "Try enabling the OES_texture_half_float_linear extension if supported.", msg_rendering_as_black);
+            mFakeBlackStatus = WebGLTextureFakeBlackStatus::IncompleteTexture;
+        }
     }
 
     // We have exhausted all cases of incomplete textures, where we would need opaque black.
@@ -410,6 +437,104 @@ WebGLTexture::ResolvedFakeBlackStatus() {
     return mFakeBlackStatus;
 }
 
+
+static bool
+ClearByMask(WebGLContext* context, GLbitfield mask)
+{
+    gl::GLContext* gl = context->GL();
+    MOZ_ASSERT(gl->IsCurrent());
+
+    GLenum status = gl->fCheckFramebufferStatus(LOCAL_GL_FRAMEBUFFER);
+    if (status != LOCAL_GL_FRAMEBUFFER_COMPLETE)
+        return false;
+
+    bool colorAttachmentsMask[WebGLContext::kMaxColorAttachments] = {false};
+    if (mask & LOCAL_GL_COLOR_BUFFER_BIT) {
+        colorAttachmentsMask[0] = true;
+    }
+
+    context->ForceClearFramebufferWithDefaultValues(mask, colorAttachmentsMask);
+    return true;
+}
+
+// `mask` from glClear.
+static bool
+ClearWithTempFB(WebGLContext* context, GLuint tex,
+                GLenum texImageTarget, GLint level,
+                GLenum baseInternalFormat,
+                GLsizei width, GLsizei height)
+{
+    if (texImageTarget != LOCAL_GL_TEXTURE_2D)
+        return false;
+
+    gl::GLContext* gl = context->GL();
+    MOZ_ASSERT(gl->IsCurrent());
+
+    gl::ScopedFramebuffer fb(gl);
+    gl::ScopedBindFramebuffer autoFB(gl, fb.FB());
+    GLbitfield mask = 0;
+
+    switch (baseInternalFormat) {
+    case LOCAL_GL_LUMINANCE:
+    case LOCAL_GL_LUMINANCE_ALPHA:
+    case LOCAL_GL_ALPHA:
+    case LOCAL_GL_RGB:
+    case LOCAL_GL_RGBA:
+    case LOCAL_GL_BGR:
+    case LOCAL_GL_BGRA:
+        mask = LOCAL_GL_COLOR_BUFFER_BIT;
+        gl->fFramebufferTexture2D(LOCAL_GL_FRAMEBUFFER, LOCAL_GL_COLOR_ATTACHMENT0,
+                                  texImageTarget, tex, level);
+        break;
+
+    case LOCAL_GL_DEPTH_COMPONENT:
+        mask = LOCAL_GL_DEPTH_BUFFER_BIT;
+        gl->fFramebufferTexture2D(LOCAL_GL_FRAMEBUFFER, LOCAL_GL_DEPTH_ATTACHMENT,
+                                  texImageTarget, tex, level);
+        break;
+
+    case LOCAL_GL_DEPTH_STENCIL:
+        mask = LOCAL_GL_DEPTH_BUFFER_BIT |
+               LOCAL_GL_STENCIL_BUFFER_BIT;
+        gl->fFramebufferTexture2D(LOCAL_GL_FRAMEBUFFER, LOCAL_GL_DEPTH_ATTACHMENT,
+                                  texImageTarget, tex, level);
+        gl->fFramebufferTexture2D(LOCAL_GL_FRAMEBUFFER, LOCAL_GL_STENCIL_ATTACHMENT,
+                                  texImageTarget, tex, level);
+        break;
+
+    default:
+        return false;
+    }
+    MOZ_ASSERT(mask);
+
+    if (ClearByMask(context, mask))
+        return true;
+
+    // Failed to simply build an FB from the tex, but maybe it needs a
+    // color buffer to be complete.
+
+    if (mask & LOCAL_GL_COLOR_BUFFER_BIT) {
+        // Nope, it already had one.
+        return false;
+    }
+
+    gl::ScopedRenderbuffer rb(gl);
+    {
+        gl::ScopedBindRenderbuffer(gl, rb.RB());
+        gl->fRenderbufferStorage(LOCAL_GL_RENDERBUFFER,
+                                 LOCAL_GL_RGBA4,
+                                 width, height);
+    }
+
+    gl->fFramebufferRenderbuffer(LOCAL_GL_FRAMEBUFFER, LOCAL_GL_COLOR_ATTACHMENT0,
+                                 LOCAL_GL_RENDERBUFFER, rb.RB());
+    mask |= LOCAL_GL_COLOR_BUFFER_BIT;
+
+    // Last chance!
+    return ClearByMask(context, mask);
+}
+
+
 void
 WebGLTexture::DoDeferredImageInitialization(GLenum imageTarget, GLint level)
 {
@@ -417,9 +542,23 @@ WebGLTexture::DoDeferredImageInitialization(GLenum imageTarget, GLint level)
     MOZ_ASSERT(imageInfo.mImageDataStatus == WebGLImageDataStatus::UninitializedImageData);
 
     mContext->MakeContextCurrent();
+
+    // Try to clear with glCLear.
+    GLenum format = imageInfo.mWebGLFormat;
+    GLenum type = imageInfo.mWebGLType;
+    WebGLTexelFormat texelformat = GetWebGLTexelFormat(format, type);
+
+    bool cleared = ClearWithTempFB(mContext, GLName(),
+                                   imageTarget, level,
+                                   format, imageInfo.mHeight, imageInfo.mWidth);
+    if (cleared) {
+        SetImageDataStatus(imageTarget, level, WebGLImageDataStatus::InitializedImageData);
+        return;
+    }
+
+    // That didn't work. Try uploading zeros then.
     gl::ScopedBindTexture autoBindTex(mContext->gl, GLName(), mTarget);
 
-    WebGLTexelFormat texelformat = GetWebGLTexelFormat(imageInfo.mInternalFormat, imageInfo.mType);
     uint32_t texelsize = WebGLTexelConversions::TexelBytesForFormat(texelformat);
     CheckedUint32 checked_byteLength
         = WebGLContext::GetImageSize(
@@ -428,24 +567,35 @@ WebGLTexture::DoDeferredImageInitialization(GLenum imageTarget, GLint level)
                         texelsize,
                         mContext->mPixelStoreUnpackAlignment);
     MOZ_ASSERT(checked_byteLength.isValid()); // should have been checked earlier
-    void *zeros = calloc(1, checked_byteLength.value());
+    ScopedFreePtr<void> zeros;
+    zeros = calloc(1, checked_byteLength.value());
 
-    mContext->UpdateWebGLErrorAndClearGLError();
-    mContext->gl->fTexImage2D(imageTarget, level, imageInfo.mInternalFormat,
-                              imageInfo.mWidth, imageInfo.mHeight,
-                              0, imageInfo.mInternalFormat, imageInfo.mType,
-                              zeros);
-    GLenum error = LOCAL_GL_NO_ERROR;
-    mContext->UpdateWebGLErrorAndClearGLError(&error);
-
-    free(zeros);
-    SetImageDataStatus(imageTarget, level, WebGLImageDataStatus::InitializedImageData);
+    gl::GLContext* gl = mContext->gl;
+    GLenum driverType = DriverTypeFromType(gl, type);
+    GLenum driverInternalFormat = LOCAL_GL_NONE;
+    GLenum driverFormat = LOCAL_GL_NONE;
+    DriverFormatsFromFormatAndType(gl, format, type, &driverInternalFormat, &driverFormat);
 
+    mContext->GetAndFlushUnderlyingGLErrors();
+    gl->fTexImage2D(imageTarget, level, driverInternalFormat,
+                    imageInfo.mWidth, imageInfo.mHeight,
+                    0, driverFormat, driverType,
+                    zeros);
+    GLenum error = mContext->GetAndFlushUnderlyingGLErrors();
     if (error) {
-      // Should only be OUT_OF_MEMORY. Anyway, there's no good way to recover from this here.
-      MOZ_CRASH(); // errors on texture upload have been related to video memory exposure in the past.
-      return;
+        // Should only be OUT_OF_MEMORY. Anyway, there's no good way to recover from this here.
+        printf_stderr("Error: 0x%4x\n", error);
+        MOZ_CRASH(); // errors on texture upload have been related to video memory exposure in the past.
     }
+
+    SetImageDataStatus(imageTarget, level, WebGLImageDataStatus::InitializedImageData);
+}
+
+void
+WebGLTexture::SetFakeBlackStatus(WebGLTextureFakeBlackStatus x)
+{
+    mFakeBlackStatus = x;
+    mContext->SetFakeBlackStatus(WebGLContextFakeBlackStatus::Unknown);
 }
 
 NS_IMPL_CYCLE_COLLECTION_WRAPPERCACHE_0(WebGLTexture)
