@@ -9,14 +9,12 @@
 #include "nsCharSeparatedTokenizer.h"
 #include "mozilla/Preferences.h"
 
-#ifdef MOZ_MEDIA_PLUGINS
-#include "MediaPluginHost.h"
+#ifdef MOZ_ANDROID_OMX
+#include "AndroidMediaPluginHost.h"
 #endif
 
-#ifdef MOZ_OGG
 #include "OggDecoder.h"
 #include "OggReader.h"
-#endif
 #ifdef MOZ_WAVE
 #include "WaveDecoder.h"
 #include "WaveReader.h"
@@ -33,17 +31,21 @@
 #include "GStreamerDecoder.h"
 #include "GStreamerReader.h"
 #endif
-#ifdef MOZ_MEDIA_PLUGINS
-#include "MediaPluginHost.h"
-#include "MediaPluginDecoder.h"
-#include "MediaPluginReader.h"
-#include "MediaPluginHost.h"
+#ifdef MOZ_ANDROID_OMX
+#include "AndroidMediaPluginHost.h"
+#include "AndroidMediaDecoder.h"
+#include "AndroidMediaReader.h"
+#include "AndroidMediaPluginHost.h"
 #endif
 #ifdef MOZ_OMX_DECODER
 #include "MediaOmxDecoder.h"
 #include "MediaOmxReader.h"
 #include "nsIPrincipal.h"
 #include "mozilla/dom/HTMLMediaElement.h"
+#if ANDROID_VERSION >= 16
+#include "MediaCodecDecoder.h"
+#include "MediaCodecReader.h"
+#endif
 #endif
 #ifdef NECKO_PROTOCOL_rtsp
 #include "RtspOmxDecoder.h"
@@ -102,7 +104,6 @@ IsRawType(const nsACString& aType)
 }
 #endif
 
-#ifdef MOZ_OGG
 // See http://www.rfc-editor.org/rfc/rfc5334.txt for the definitions
 // of Ogg media types and codec types
 static const char* const gOggTypes[4] = {
@@ -134,7 +135,6 @@ IsOggType(const nsACString& aType)
 
   return CodecListContains(gOggTypes, aType);
 }
-#endif
 
 #ifdef MOZ_WAVE
 // See http://www.rfc-editor.org/rfc/rfc2361.txt for the definitions
@@ -203,23 +203,26 @@ IsGStreamerSupportedType(const nsACString& aMimeType)
   if (IsWebMType(aMimeType) && !Preferences::GetBool("media.prefer-gstreamer", false))
     return false;
 #endif
-#ifdef MOZ_OGG
   if (IsOggType(aMimeType) && !Preferences::GetBool("media.prefer-gstreamer", false))
     return false;
-#endif
 
   return GStreamerDecoder::CanHandleMediaType(aMimeType, nullptr);
 }
 #endif
 
 #ifdef MOZ_OMX_DECODER
-static const char* const gOmxTypes[7] = {
+static const char* const gOmxTypes[] = {
   "audio/mpeg",
   "audio/mp4",
   "audio/amr",
   "video/mp4",
   "video/3gpp",
+  "video/3gpp2",
   "video/quicktime",
+#ifdef MOZ_OMX_WEBM_DECODER
+  "video/webm",
+  "audio/webm",
+#endif
   nullptr
 };
 
@@ -249,6 +252,16 @@ static char const *const gMpegAudioCodecs[2] = {
   "mp3",          // MP3
   nullptr
 };
+
+#ifdef MOZ_OMX_WEBM_DECODER
+static char const *const gOMXWebMCodecs[4] = {
+  "vorbis",
+  "vp8",
+  "vp8.0",
+  nullptr
+};
+#endif //MOZ_OMX_WEBM_DECODER
+
 #endif
 
 #ifdef NECKO_PROTOCOL_rtsp
@@ -274,11 +287,11 @@ bool DecoderTraits::DecoderWaitsForOnConnected(const nsACString& aMimeType) {
 #endif
 }
 
-#ifdef MOZ_MEDIA_PLUGINS
+#ifdef MOZ_ANDROID_OMX
 static bool
-IsMediaPluginsType(const nsACString& aType)
+IsAndroidMediaType(const nsACString& aType)
 {
-  if (!MediaDecoder::IsMediaPluginsEnabled()) {
+  if (!MediaDecoder::IsAndroidMediaEnabled()) {
     return false;
   }
 
@@ -376,24 +389,27 @@ DecoderTraits::CanHandleMediaType(const char* aMIMEType,
     result = CANPLAY_MAYBE;
   }
 #endif
-#ifdef MOZ_OGG
   if (IsOggType(nsDependentCString(aMIMEType))) {
     codecList = MediaDecoder::IsOpusEnabled() ? gOggCodecsWithOpus : gOggCodecs;
     result = CANPLAY_MAYBE;
   }
-#endif
 #ifdef MOZ_WAVE
   if (IsWaveType(nsDependentCString(aMIMEType))) {
     codecList = gWaveCodecs;
     result = CANPLAY_MAYBE;
   }
 #endif
-#ifdef MOZ_WEBM
+#if defined(MOZ_WEBM) && !defined(MOZ_OMX_WEBM_DECODER)
   if (IsWebMType(nsDependentCString(aMIMEType))) {
     codecList = gWebMCodecs;
     result = CANPLAY_MAYBE;
   }
 #endif
+#ifdef MOZ_FMP4
+  if (IsMP4SupportedType(nsDependentCString(aMIMEType))) {
+    result = aHaveRequestedCodecs ? CANPLAY_YES : CANPLAY_MAYBE;
+  }
+#endif
 #ifdef MOZ_GSTREAMER
   if (GStreamerDecoder::CanHandleMediaType(nsDependentCString(aMIMEType),
                                            aHaveRequestedCodecs ? &aRequestedCodecs : nullptr)) {
@@ -407,11 +423,23 @@ DecoderTraits::CanHandleMediaType(const char* aMIMEType,
     result = CANPLAY_MAYBE;
     if (nsDependentCString(aMIMEType).EqualsASCII("audio/mpeg")) {
       codecList = gMpegAudioCodecs;
+#ifdef MOZ_OMX_WEBM_DECODER
+    } else if (nsDependentCString(aMIMEType).EqualsASCII("audio/webm") ||
+               nsDependentCString(aMIMEType).EqualsASCII("video/webm")) {
+      codecList = gOMXWebMCodecs;
+#endif
     } else {
       codecList = gH264Codecs;
     }
   }
 #endif
+#ifdef MOZ_DIRECTSHOW
+  // Note: DirectShow should come before WMF, so that we prefer DirectShow's
+  // MP3 support over WMF's.
+  if (DirectShowDecoder::GetSupportedCodecs(nsDependentCString(aMIMEType), &codecList)) {
+    result = CANPLAY_MAYBE;
+  }
+#endif
 #ifdef MOZ_WMF
   if (IsWMFSupportedType(nsDependentCString(aMIMEType))) {
     if (!aHaveRequestedCodecs) {
@@ -422,20 +450,20 @@ DecoderTraits::CanHandleMediaType(const char* aMIMEType,
            ? CANPLAY_YES : CANPLAY_NO;
   }
 #endif
-#ifdef MOZ_DIRECTSHOW
-  if (DirectShowDecoder::GetSupportedCodecs(nsDependentCString(aMIMEType), &codecList)) {
-    result = CANPLAY_MAYBE;
-  }
-#endif
 #ifdef MOZ_APPLEMEDIA
   if (IsAppleMediaSupportedType(nsDependentCString(aMIMEType), &codecList)) {
     result = CANPLAY_MAYBE;
   }
 #endif
-#ifdef MOZ_MEDIA_PLUGINS
-  if (MediaDecoder::IsMediaPluginsEnabled() &&
-      GetMediaPluginHost()->FindDecoder(nsDependentCString(aMIMEType), &codecList))
+#ifdef MOZ_ANDROID_OMX
+  if (MediaDecoder::IsAndroidMediaEnabled() &&
+      GetAndroidMediaPluginHost()->FindDecoder(nsDependentCString(aMIMEType), &codecList))
     result = CANPLAY_MAYBE;
+#endif
+#ifdef NECKO_PROTOCOL_rtsp
+  if (IsRtspSupportedType(nsDependentCString(aMIMEType))) {
+    result = CANPLAY_MAYBE;
+  }
 #endif
   if (result == CANPLAY_NO || !aHaveRequestedCodecs || !codecList) {
     return result;
@@ -468,6 +496,12 @@ InstantiateDecoder(const nsACString& aType, MediaDecoderOwner* aOwner)
 {
   nsRefPtr<MediaDecoder> decoder;
 
+#ifdef MOZ_FMP4
+  if (IsMP4SupportedType(aType)) {
+    decoder = new MP4Decoder();
+    return decoder.forget();
+  }
+#endif
 #ifdef MOZ_GSTREAMER
   if (IsGStreamerSupportedType(aType)) {
     decoder = new GStreamerDecoder();
@@ -480,12 +514,10 @@ InstantiateDecoder(const nsACString& aType, MediaDecoderOwner* aOwner)
     return decoder.forget();
   }
 #endif
-#ifdef MOZ_OGG
   if (IsOggType(aType)) {
     decoder = new OggDecoder();
     return decoder.forget();
   }
-#endif
 #ifdef MOZ_WAVE
   if (IsWaveType(aType)) {
     decoder = new WaveDecoder();
@@ -509,7 +541,13 @@ InstantiateDecoder(const nsACString& aType, MediaDecoderOwner* aOwner)
         return nullptr;
       }
     }
+#if ANDROID_VERSION >= 16
+    decoder = MediaDecoder::IsOmxAsyncEnabled()
+      ? static_cast<MediaDecoder*>(new MediaCodecDecoder())
+      : static_cast<MediaDecoder*>(new MediaOmxDecoder());
+#else
     decoder = new MediaOmxDecoder();
+#endif
     return decoder.forget();
   }
 #endif
@@ -519,10 +557,10 @@ InstantiateDecoder(const nsACString& aType, MediaDecoderOwner* aOwner)
     return decoder.forget();
   }
 #endif
-#ifdef MOZ_MEDIA_PLUGINS
-  if (MediaDecoder::IsMediaPluginsEnabled() &&
-      GetMediaPluginHost()->FindDecoder(aType, nullptr)) {
-    decoder = new MediaPluginDecoder(aType);
+#ifdef MOZ_ANDROID_OMX
+  if (MediaDecoder::IsAndroidMediaEnabled() &&
+      GetAndroidMediaPluginHost()->FindDecoder(aType, nullptr)) {
+    decoder = new AndroidMediaDecoder(aType);
     return decoder.forget();
   }
 #endif
@@ -533,19 +571,13 @@ InstantiateDecoder(const nsACString& aType, MediaDecoderOwner* aOwner)
   }
 #endif
 #ifdef MOZ_DIRECTSHOW
-  // Note: DirectShow decoder must come before WMFDecoder, else the pref
-  // "media.directshow.preferred" won't be honored.
+  // Note: DirectShow should come before WMF, so that we prefer DirectShow's
+  // MP3 support over WMF's.
   if (IsDirectShowSupportedType(aType)) {
     decoder = new DirectShowDecoder();
     return decoder.forget();
   }
 #endif
-#ifdef MOZ_FMP4
-  if (IsMP4SupportedType(aType)) {
-    decoder = new MP4Decoder();
-    return decoder.forget();
-  }
-#endif
 #ifdef MOZ_WMF
   if (IsWMFSupportedType(aType)) {
     decoder = new WMFDecoder();
@@ -580,6 +612,11 @@ MediaDecoderReader* DecoderTraits::CreateReader(const nsACString& aType, Abstrac
 {
   MediaDecoderReader* decoderReader = nullptr;
 
+#ifdef MOZ_FMP4
+  if (IsMP4SupportedType(aType)) {
+    decoderReader = new MP4Reader(aDecoder);
+  } else
+#endif
 #ifdef MOZ_GSTREAMER
   if (IsGStreamerSupportedType(aType)) {
     decoderReader = new GStreamerReader(aDecoder);
@@ -590,11 +627,9 @@ MediaDecoderReader* DecoderTraits::CreateReader(const nsACString& aType, Abstrac
     decoderReader = new RawReader(aDecoder);
   } else
 #endif
-#ifdef MOZ_OGG
   if (IsOggType(aType)) {
     decoderReader = new OggReader(aDecoder);
   } else
-#endif
 #ifdef MOZ_WAVE
   if (IsWaveType(aType)) {
     decoderReader = new WaveReader(aDecoder);
@@ -602,13 +637,19 @@ MediaDecoderReader* DecoderTraits::CreateReader(const nsACString& aType, Abstrac
 #endif
 #ifdef MOZ_OMX_DECODER
   if (IsOmxSupportedType(aType)) {
+#if ANDROID_VERSION >= 16
+    decoderReader = MediaDecoder::IsOmxAsyncEnabled()
+      ? static_cast<MediaDecoderReader*>(new MediaCodecReader(aDecoder))
+      : static_cast<MediaDecoderReader*>(new MediaOmxReader(aDecoder));
+#else
     decoderReader = new MediaOmxReader(aDecoder);
+#endif
   } else
 #endif
-#ifdef MOZ_MEDIA_PLUGINS
-  if (MediaDecoder::IsMediaPluginsEnabled() &&
-      GetMediaPluginHost()->FindDecoder(aType, nullptr)) {
-    decoderReader = new MediaPluginReader(aDecoder, aType);
+#ifdef MOZ_ANDROID_OMX
+  if (MediaDecoder::IsAndroidMediaEnabled() &&
+      GetAndroidMediaPluginHost()->FindDecoder(aType, nullptr)) {
+    decoderReader = new AndroidMediaReader(aDecoder, aType);
   } else
 #endif
 #ifdef MOZ_WEBM
@@ -623,11 +664,6 @@ MediaDecoderReader* DecoderTraits::CreateReader(const nsACString& aType, Abstrac
     decoderReader = new DirectShowReader(aDecoder);
   } else
 #endif
-#ifdef MOZ_FMP4
-  if (IsMP4SupportedType(aType)) {
-    decoderReader = new MP4Reader(aDecoder);
-  } else
-#endif
 #ifdef MOZ_WMF
   if (IsWMFSupportedType(aType)) {
     decoderReader = new WMFReader(aDecoder);
@@ -647,9 +683,7 @@ MediaDecoderReader* DecoderTraits::CreateReader(const nsACString& aType, Abstrac
 bool DecoderTraits::IsSupportedInVideoDocument(const nsACString& aType)
 {
   return
-#ifdef MOZ_OGG
     IsOggType(aType) ||
-#endif
 #ifdef MOZ_OMX_DECODER
     // We support amr inside WebApps on firefoxOS but not in general web content.
     // Ensure we dont create a VideoDocument when accessing amr URLs directly.
@@ -661,8 +695,8 @@ bool DecoderTraits::IsSupportedInVideoDocument(const nsACString& aType)
 #ifdef MOZ_GSTREAMER
     IsGStreamerSupportedType(aType) ||
 #endif
-#ifdef MOZ_MEDIA_PLUGINS
-    (MediaDecoder::IsMediaPluginsEnabled() && IsMediaPluginsType(aType)) ||
+#ifdef MOZ_ANDROID_OMX
+    (MediaDecoder::IsAndroidMediaEnabled() && IsAndroidMediaType(aType)) ||
 #endif
 #ifdef MOZ_FMP4
     IsMP4SupportedType(aType) ||
@@ -676,6 +710,9 @@ bool DecoderTraits::IsSupportedInVideoDocument(const nsACString& aType)
 #endif
 #ifdef MOZ_APPLEMEDIA
     IsAppleMediaSupportedType(aType) ||
+#endif
+#ifdef NECKO_PROTOCOL_rtsp
+    IsRtspSupportedType(aType) ||
 #endif
     false;
 }
