@@ -12,8 +12,12 @@
 #define AUDIO_READ_BYTES 4096
 
 // Maximum number of audio frames we will accept from the audio decoder in one
-// go.
-#define MAX_AUDIO_FRAMES 4096
+// go.  Carefully select this to work well with both the mp3 1152 max frames
+// per block and power-of-2 allocation sizes.  Since we must pre-allocate the
+// buffer we cannot use AudioCompactor without paying for an additional
+// allocation and copy.  Therefore, choosing a value that divides exactly into
+// 1152 is most memory efficient.
+#define MAX_AUDIO_FRAMES 128
 
 namespace mozilla {
 
@@ -86,18 +90,12 @@ static void _AudioSampleCallback(void *aThis,
  * put it in |aData|, and return true.
  * Otherwise, put as much data as is left into |aData|, set |aNumBytes| to the
  * amount of data we have left, and return false.
- *
- * This function also calls NotifyBytesConsumed() on the media resource and
- * passes the read data on to the MP3 frame parser for stream duration
- * estimation.
  */
 nsresult
-AppleMP3Reader::ReadAndNotify(uint32_t *aNumBytes, char *aData)
+AppleMP3Reader::Read(uint32_t *aNumBytes, char *aData)
 {
   MediaResource *resource = mDecoder->GetResource();
 
-  uint64_t offset = resource->Tell();
-
   // Loop until we have all the data asked for, or we've reached EOS
   uint32_t totalBytes = 0;
   uint32_t numBytes;
@@ -112,20 +110,6 @@ AppleMP3Reader::ReadAndNotify(uint32_t *aNumBytes, char *aData)
     }
   } while(totalBytes < *aNumBytes && numBytes);
 
-  mDecoder->NotifyBytesConsumed(totalBytes);
-
-  // Pass the buffer to the MP3 frame parser to improve our duration estimate.
-  if (mMP3FrameParser.IsMP3()) {
-    mMP3FrameParser.Parse(aData, totalBytes, offset);
-    uint64_t duration = mMP3FrameParser.GetDuration();
-    if (duration != mDuration) {
-      LOGD("Updating media duration to %lluus\n", duration);
-      mDuration = duration;
-      ReentrantMonitorAutoEnter mon(mDecoder->GetReentrantMonitor());
-      mDecoder->UpdateEstimatedMediaDuration(duration);
-    }
-  }
-
   *aNumBytes = totalBytes;
 
   // We will have read some data in the last iteration iff we filled the buffer.
@@ -182,6 +166,10 @@ AppleMP3Reader::PassthroughInputDataCallback(AudioConverterRef aAudioConverter,
                                              AudioStreamPacketDescription **aPacketDesc,
                                              void *aUserData)
 {
+  if (!aPacketDesc) {
+    return kAudioFileStreamError_UnspecifiedError;
+  }
+
   PassthroughUserData *userData = (PassthroughUserData *)aUserData;
   if (userData->mDone) {
     // We make sure this callback is run _once_, with all the data we received
@@ -221,7 +209,8 @@ AppleMP3Reader::AudioSampleCallback(UInt32 aNumBytes,
   LOGD("got %u bytes, %u packets\n", aNumBytes, aNumPackets);
 
   // 1 frame per packet * num channels * 32-bit float
-  uint32_t decodedSize = MAX_AUDIO_FRAMES * mAudioChannels * 4;
+  uint32_t decodedSize = MAX_AUDIO_FRAMES * mAudioChannels *
+                         sizeof(AudioDataValue);
 
   // descriptions for _decompressed_ audio packets. ignored.
   nsAutoArrayPtr<AudioStreamPacketDescription>
@@ -258,6 +247,14 @@ AppleMP3Reader::AudioSampleCallback(UInt32 aNumBytes,
       break;
     }
 
+    // If we decoded zero frames then AudiOConverterFillComplexBuffer is out
+    // of data to provide.  We drained its internal buffer completely on the
+    // last pass.
+    if (numFrames == 0 && rv == kNeedMoreData) {
+      LOGD("FillComplexBuffer out of data exactly\n");
+      break;
+    }
+
     int64_t time = FramesToUsecs(mCurrentAudioFrame, mAudioSampleRate).value();
     int64_t duration = FramesToUsecs(numFrames, mAudioSampleRate).value();
 
@@ -289,7 +286,7 @@ AppleMP3Reader::DecodeAudioData()
   char bytes[AUDIO_READ_BYTES];
   uint32_t numBytes = AUDIO_READ_BYTES;
 
-  nsresult readrv = ReadAndNotify(&numBytes, bytes);
+  nsresult readrv = Read(&numBytes, bytes);
 
   // This function calls |AudioSampleCallback| above, synchronously, when it
   // finds compressed MP3 frame.
@@ -336,6 +333,12 @@ AppleMP3Reader::HasVideo()
   return false;
 }
 
+bool
+AppleMP3Reader::IsMediaSeekable()
+{
+  // not used
+  return true;
+}
 
 /*
  * Query the MP3 parser for a piece of metadata.
@@ -363,7 +366,7 @@ GetProperty(AudioFileStreamID aAudioFileStream,
 
 
 nsresult
-AppleMP3Reader::ReadMetadata(VideoInfo* aInfo,
+AppleMP3Reader::ReadMetadata(MediaInfo* aInfo,
                              MetadataTags** aTags)
 {
   MOZ_ASSERT(mDecoder->OnDecodeThread(), "Should be on decode thread");
@@ -377,16 +380,21 @@ AppleMP3Reader::ReadMetadata(VideoInfo* aInfo,
    */
   OSStatus rv;
   nsresult readrv;
+  uint32_t offset = 0;
   do {
     char bytes[AUDIO_READ_BYTES];
     uint32_t numBytes = AUDIO_READ_BYTES;
-    readrv = ReadAndNotify(&numBytes, bytes);
+    readrv = Read(&numBytes, bytes);
 
     rv = AudioFileStreamParseBytes(mAudioFileStream,
                                    numBytes,
                                    bytes,
                                    0 /* flags */);
 
+    mMP3FrameParser.Parse(bytes, numBytes, offset);
+
+    offset += numBytes;
+
     // We have to do our decoder setup from the callback. When it's done it will
     // set mStreamReady.
   } while (!mStreamReady && !rv && NS_SUCCEEDED(readrv));
@@ -401,12 +409,18 @@ AppleMP3Reader::ReadMetadata(VideoInfo* aInfo,
     return NS_ERROR_FAILURE;
   }
 
-  aInfo->mAudioRate = mAudioSampleRate;
-  aInfo->mAudioChannels = mAudioChannels;
-  aInfo->mHasAudio = mStreamReady;
+  if (!mMP3FrameParser.IsMP3()) {
+    LOGE("Frame parser failed to parse MP3 stream\n");
+    return NS_ERROR_FAILURE;
+  }
+
+  aInfo->mAudio.mRate = mAudioSampleRate;
+  aInfo->mAudio.mChannels = mAudioChannels;
+  aInfo->mAudio.mHasAudio = mStreamReady;
 
   {
     ReentrantMonitorAutoEnter mon(mDecoder->GetReentrantMonitor());
+    mDuration = mMP3FrameParser.GetDuration();
     mDecoder->SetMediaDuration(mDuration);
   }
 
@@ -518,16 +532,25 @@ AppleMP3Reader::Seek(int64_t aTime,
   return NS_OK;
 }
 
-
-nsresult
-AppleMP3Reader::GetBuffered(dom::TimeRanges* aBuffered,
-                            int64_t aStartTime)
+void
+AppleMP3Reader::NotifyDataArrived(const char* aBuffer,
+                                  uint32_t aLength,
+                                  int64_t aOffset)
 {
-  ReentrantMonitorAutoEnter mon(mDecoder->GetReentrantMonitor());
-  GetEstimatedBufferedTimeRanges(mDecoder->GetResource(),
-                                 mDecoder->GetMediaDuration(),
-                                 aBuffered);
-  return NS_OK;
+  MOZ_ASSERT(NS_IsMainThread());
+  if (!mMP3FrameParser.NeedsData()) {
+    return;
+  }
+
+  mMP3FrameParser.Parse(aBuffer, aLength, aOffset);
+
+  uint64_t duration = mMP3FrameParser.GetDuration();
+  if (duration != mDuration) {
+    LOGD("Updating media duration to %lluus\n", duration);
+    mDuration = duration;
+    ReentrantMonitorAutoEnter mon(mDecoder->GetReentrantMonitor());
+    mDecoder->UpdateEstimatedMediaDuration(duration);
+  }
 }
 
 } // namespace mozilla
