@@ -24,6 +24,9 @@ extern "C" {
 #include "VorbisUtils.h"
 #include "MediaMetadataManager.h"
 #include "nsISeekableStream.h"
+#include "gfx2DGlue.h"
+
+using namespace mozilla::gfx;
 
 namespace mozilla {
 
@@ -288,7 +291,7 @@ nsresult OggReader::ReadMetadata(MediaInfo* aInfo,
 
     nsIntSize frameSize(mTheoraState->mInfo.frame_width,
                         mTheoraState->mInfo.frame_height);
-    if (VideoInfo::ValidateVideoRegion(frameSize, picture, displaySize)) {
+    if (IsValidVideoRegion(frameSize, picture, displaySize)) {
       // Video track's frame sizes will not overflow. Activate the video track.
       mInfo.mVideo.mHasVideo = true;
       mInfo.mVideo.mDisplay = displaySize;
@@ -310,7 +313,7 @@ nsresult OggReader::ReadMetadata(MediaInfo* aInfo,
   if (mVorbisState && ReadHeaders(mVorbisState)) {
     mInfo.mAudio.mHasAudio = true;
     mInfo.mAudio.mRate = mVorbisState->mInfo.rate;
-    mInfo.mAudio.mChannels = mVorbisState->mInfo.channels > 2 ? 2 : mVorbisState->mInfo.channels;
+    mInfo.mAudio.mChannels = mVorbisState->mInfo.channels;
     // Copy Vorbis info data for time computations on other threads.
     memcpy(&mVorbisInfo, &mVorbisState->mInfo, sizeof(mVorbisInfo));
     mVorbisInfo.codec_setup = nullptr;
@@ -323,7 +326,7 @@ nsresult OggReader::ReadMetadata(MediaInfo* aInfo,
   if (mOpusState && ReadHeaders(mOpusState)) {
     mInfo.mAudio.mHasAudio = true;
     mInfo.mAudio.mRate = mOpusState->mRate;
-    mInfo.mAudio.mChannels = mOpusState->mChannels > 2 ? 2 : mOpusState->mChannels;
+    mInfo.mAudio.mChannels = mOpusState->mChannels;
     mOpusSerial = mOpusState->mSerial;
     mOpusPreSkip = mOpusState->mPreSkip;
 
@@ -375,12 +378,6 @@ nsresult OggReader::ReadMetadata(MediaInfo* aInfo,
         LOG(PR_LOG_DEBUG, ("Got Ogg duration from seeking to end %lld", endTime));
       }
       mDecoder->GetResource()->EndSeekingForMetadata();
-    } else if (mDecoder->GetMediaDuration() == -1) {
-      // We don't have a duration, and we don't know enough about the resource
-      // to try a seek. Abort trying to get a duration. This happens for example
-      // when the server says it accepts range requests, but does not give us a
-      // Content-Length.
-      mDecoder->SetTransportSeekable(false);
     }
   } else {
     return NS_ERROR_FAILURE;
@@ -390,6 +387,15 @@ nsresult OggReader::ReadMetadata(MediaInfo* aInfo,
   return NS_OK;
 }
 
+bool
+OggReader::IsMediaSeekable()
+{
+  if (mIsChained) {
+    return false;
+  }
+  return true;
+}
+
 nsresult OggReader::DecodeVorbis(ogg_packet* aPacket) {
   NS_ASSERTION(aPacket->granulepos != -1, "Must know vorbis granulepos!");
 
@@ -416,13 +422,9 @@ nsresult OggReader::DecodeVorbis(ogg_packet* aPacket) {
       }
     }
 
-    // More than 2 decoded channels must be downmixed to stereo.
-    if (channels > 2) {
-      if (channels > 8) {
-        // No channel mapping for more than 8 channels.
-        return NS_ERROR_FAILURE;
-      }
-      DownmixToStereo(buffer, channels, frames);
+    // No channel mapping for more than 8 channels.
+    if (channels > 8) {
+      return NS_ERROR_FAILURE;
     }
 
     int64_t duration = mVorbisState->Time((int64_t)frames);
@@ -535,13 +537,9 @@ nsresult OggReader::DecodeOpus(ogg_packet* aPacket) {
   }
 #endif
 
-  // More than 2 decoded channels must be downmixed to stereo.
-  if (channels > 2) {
-    // Opus doesn't provide a channel mapping for more than 8 channels,
-    // so we can't downmix more than that.
-    if (channels > 8)
-      return NS_ERROR_FAILURE;
-    DownmixToStereo(buffer, channels, frames);
+  // No channel mapping for more than 8 channels.
+  if (channels > 8) {
+    return NS_ERROR_FAILURE;
   }
 
   LOG(PR_LOG_DEBUG, ("Opus decoder pushing %d frames", frames));
@@ -560,61 +558,6 @@ nsresult OggReader::DecodeOpus(ogg_packet* aPacket) {
 }
 #endif /* MOZ_OPUS */
 
-void OggReader::DownmixToStereo(nsAutoArrayPtr<AudioDataValue>& buffer,
-                              uint32_t& channels, int32_t frames)
-{
-  uint32_t out_channels;
-  out_channels = 2;
-  // dBuffer stores the downmixed samples.
-  nsAutoArrayPtr<AudioDataValue> dBuffer(new AudioDataValue[frames * out_channels]);
-#ifdef MOZ_SAMPLE_TYPE_FLOAT32
-  // Downmix matrix. Per-row normalization 1 for rows 3,4 and 2 for rows 5-8.
-  static const float dmatrix[6][8][2]= {
-      /*3*/{{0.5858f,0},{0.4142f,0.4142f},{0,     0.5858f}},
-      /*4*/{{0.4226f,0},{0,      0.4226f},{0.366f,0.2114f},{0.2114f,0.366f}},
-      /*5*/{{0.6510f,0},{0.4600f,0.4600f},{0,     0.6510f},{0.5636f,0.3254f},{0.3254f,0.5636f}},
-      /*6*/{{0.5290f,0},{0.3741f,0.3741f},{0,     0.5290f},{0.4582f,0.2645f},{0.2645f,0.4582f},{0.3741f,0.3741f}},
-      /*7*/{{0.4553f,0},{0.3220f,0.3220f},{0,     0.4553f},{0.3943f,0.2277f},{0.2277f,0.3943f},{0.2788f,0.2788f},{0.3220f,0.3220f}},
-      /*8*/{{0.3886f,0},{0.2748f,0.2748f},{0,     0.3886f},{0.3366f,0.1943f},{0.1943f,0.3366f},{0.3366f,0.1943f},{0.1943f,0.3366f},{0.2748f,0.2748f}},
-  };
-  for (int32_t i = 0; i < frames; i++) {
-    float sampL = 0.0;
-    float sampR = 0.0;
-    for (uint32_t j = 0; j < channels; j++) {
-      sampL+=buffer[i*channels+j]*dmatrix[channels-3][j][0];
-      sampR+=buffer[i*channels+j]*dmatrix[channels-3][j][1];
-    }
-    dBuffer[i*out_channels]=sampL;
-    dBuffer[i*out_channels+1]=sampR;
-  }
-#else
-  // Downmix matrix. Per-row normalization 1 for rows 3,4 and 2 for rows 5-8.
-  // Coefficients in Q14.
-  static const int16_t dmatrix[6][8][2]= {
-      /*3*/{{9598, 0},{6786,6786},{0,   9598}},
-      /*4*/{{6925, 0},{0,   6925},{5997,3462},{3462,5997}},
-      /*5*/{{10663,0},{7540,7540},{0,  10663},{9234,5331},{5331,9234}},
-      /*6*/{{8668, 0},{6129,6129},{0,   8668},{7507,4335},{4335,7507},{6129,6129}},
-      /*7*/{{7459, 0},{5275,5275},{0,   7459},{6460,3731},{3731,6460},{4568,4568},{5275,5275}},
-      /*8*/{{6368, 0},{4502,4502},{0,   6368},{5514,3184},{3184,5514},{5514,3184},{3184,5514},{4502,4502}}
-  };
-  for (int32_t i = 0; i < frames; i++) {
-    int32_t sampL = 0;
-    int32_t sampR = 0;
-    for (uint32_t j = 0; j < channels; j++) {
-      sampL+=buffer[i*channels+j]*dmatrix[channels-3][j][0];
-      sampR+=buffer[i*channels+j]*dmatrix[channels-3][j][1];
-    }
-    sampL = (sampL + 8192)>>14;
-    dBuffer[i*out_channels]   = static_cast<AudioDataValue>(MOZ_CLIP_TO_15(sampL));
-    sampR = (sampR + 8192)>>14;
-    dBuffer[i*out_channels+1] = static_cast<AudioDataValue>(MOZ_CLIP_TO_15(sampR));
-  }
-#endif
-  channels = out_channels;
-  buffer = dBuffer;
-}
-
 bool OggReader::DecodeAudioData()
 {
   NS_ASSERTION(mDecoder->OnDecodeThread(), "Should be on decode thread.");
@@ -684,8 +627,6 @@ bool OggReader::ReadOggChain()
   OpusState* newOpusState = nullptr;
 #endif /* MOZ_OPUS */
   VorbisState* newVorbisState = nullptr;
-  int channels = 0;
-  long rate = 0;
   MetadataTags* tags = nullptr;
 
   if (HasVideo() || HasSkeleton() || !HasAudio()) {
@@ -730,6 +671,7 @@ bool OggReader::ReadOggChain()
     return false;
   }
 
+  nsAutoPtr<MediaInfo> info(new MediaInfo());
   if ((newVorbisState && ReadHeaders(newVorbisState)) &&
       (mVorbisState->mInfo.rate == newVorbisState->mInfo.rate) &&
       (mVorbisState->mInfo.channels == newVorbisState->mInfo.channels)) {
@@ -738,8 +680,8 @@ bool OggReader::ReadOggChain()
     mVorbisSerial = mVorbisState->mSerial;
     LOG(PR_LOG_DEBUG, ("New vorbis ogg link, serial=%d\n", mVorbisSerial));
     chained = true;
-    rate = mVorbisState->mInfo.rate;
-    channels = mVorbisState->mInfo.channels;
+    info->mAudio.mRate = mVorbisState->mInfo.rate;
+    info->mAudio.mChannels = mVorbisState->mInfo.channels;
     tags = mVorbisState->GetTags();
   }
 
@@ -751,8 +693,8 @@ bool OggReader::ReadOggChain()
     mOpusState = newOpusState;
     mOpusSerial = mOpusState->mSerial;
     chained = true;
-    rate = mOpusState->mRate;
-    channels = mOpusState->mChannels;
+    info->mAudio.mRate = mOpusState->mRate;
+    info->mAudio.mChannels = mOpusState->mChannels;
     tags = mOpusState->GetTags();
   }
 #endif
@@ -760,13 +702,12 @@ bool OggReader::ReadOggChain()
   if (chained) {
     SetChained(true);
     {
+      info->mAudio.mHasAudio = HasAudio();
+      info->mVideo.mHasVideo = HasVideo();
+      int rate = info->mAudio.mRate;
       ReentrantMonitorAutoEnter mon(mDecoder->GetReentrantMonitor());
       mDecoder->QueueMetadata((mDecodedAudioFrames * USECS_PER_S) / rate,
-                               channels,
-                               rate,
-                               HasAudio(),
-                               HasVideo(),
-                               tags);
+                              info.forget(), tags);
     }
     return true;
   }
@@ -828,7 +769,7 @@ nsresult OggReader::DecodeTheora(ogg_packet* aPacket, int64_t aTimeThreshold)
                                      b,
                                      isKeyframe,
                                      aPacket->granulepos,
-                                     mPicture);
+                                     ToIntRect(mPicture));
     if (!v) {
       // There may be other reasons for this error, but for
       // simplicity just assume the worst case: out of memory.
@@ -1150,8 +1091,8 @@ nsresult OggReader::GetSeekRanges(nsTArray<SeekRange>& aRanges)
     if (startTime != -1 &&
         ((endTime = RangeEndTime(endOffset)) != -1))
     {
-      NS_ASSERTION(startTime < endTime,
-                   "Start time must be before end time");
+      NS_WARN_IF_FALSE(startTime < endTime,
+                       "Start time must be before end time");
       aRanges.AppendElement(SeekRange(startOffset,
                                       endOffset,
                                       startTime,
@@ -1375,9 +1316,9 @@ nsresult OggReader::SeekInUnbuffered(int64_t aTarget,
 }
 
 nsresult OggReader::Seek(int64_t aTarget,
-                           int64_t aStartTime,
-                           int64_t aEndTime,
-                           int64_t aCurrentTime)
+                         int64_t aStartTime,
+                         int64_t aEndTime,
+                         int64_t aCurrentTime)
 {
   NS_ASSERTION(mDecoder->OnDecodeThread(), "Should be on decode thread.");
   if (mIsChained)
@@ -1439,10 +1380,39 @@ nsresult OggReader::Seek(int64_t aTarget,
     }
   }
 
-  // The decode position must now be either close to the seek target, or
-  // we've seeked to before the keyframe before the seek target. Decode
-  // forward to the seek target frame.
-  return DecodeToTarget(aTarget);
+  if (HasVideo()) {
+    // Decode forwards until we find the next keyframe. This is required,
+    // as although the seek should finish on a page containing a keyframe,
+    // there may be non-keyframes in the page before the keyframe.
+    // When doing fastSeek we display the first frame after the seek, so
+    // we need to advance the decode to the keyframe otherwise we'll get
+    // visual artifacts in the first frame output after the seek.
+    // First, we must check to see if there's already a keyframe in the frames
+    // that we may have already decoded, and discard frames up to the
+    // keyframe.
+    VideoData* v;
+    while ((v = mVideoQueue.PeekFront()) && !v->mKeyframe) {
+      delete mVideoQueue.PopFront();
+    }
+    if (mVideoQueue.GetSize() == 0) {
+      // We didn't find a keyframe in the frames already here, so decode
+      // forwards until we find a keyframe.
+      bool skip = true;
+      while (DecodeVideoFrame(skip, 0) && skip) {
+        ReentrantMonitorAutoEnter mon(mDecoder->GetReentrantMonitor());
+        if (mDecoder->IsShutdown()) {
+          return NS_ERROR_FAILURE;
+        }
+      }
+    }
+#ifdef DEBUG
+    v = mVideoQueue.PeekFront();
+    if (!v || !v->mKeyframe) {
+      NS_WARNING("Ogg seek didn't end up before a key frame!");
+    }
+#endif
+  }
+  return NS_OK;
 }
 
 // Reads a page from the media resource.
