@@ -1,16 +1,12 @@
 #include <stagefright/foundation/ABase.h>
 #include <stagefright/foundation/AHandlerReflector.h>
 #include <stagefright/foundation/ALooper.h>
-#include <stagefright/MediaSource.h>
-#include <stagefright/DataSource.h>
-#include <stagefright/MediaSource.h>
 #include <utils/RefBase.h>
 #include <stagefright/MediaExtractor.h>
 
 #include "GonkNativeWindow.h"
 #include "GonkNativeWindowClient.h"
-#include "GrallocImages.h"
-#include "gfxipc/FenceUtils.h"
+#include "mozilla/layers/FenceUtils.h"
 #include "MP3FrameParser.h"
 #include "MPAPI.h"
 #include "MediaResource.h"
@@ -21,61 +17,8 @@ namespace android {
 class OmxDecoder;
 };
 
-namespace mozilla {
-namespace layers {
-
-class VideoGraphicBuffer : public GraphicBufferLocked {
-  // XXX change this to an actual smart pointer at some point
-  android::MediaBuffer *mMediaBuffer;
-  android::wp<android::OmxDecoder> mOmxDecoder;
-  public:
-    VideoGraphicBuffer(const android::wp<android::OmxDecoder> aOmxDecoder,
-                       android::MediaBuffer *aBuffer,
-                       SurfaceDescriptor& aDescriptor);
-    ~VideoGraphicBuffer();
-    void Unlock();
-};
-
-}
-}
-
 namespace android {
 
-// MediaStreamSource is a DataSource that reads from a MPAPI media stream.
-class MediaStreamSource : public DataSource {
-  typedef mozilla::MediaResource MediaResource;
-  typedef mozilla::AbstractMediaDecoder AbstractMediaDecoder;
-
-  Mutex mLock;
-  nsRefPtr<MediaResource> mResource;
-  AbstractMediaDecoder *mDecoder;
-public:
-  MediaStreamSource(MediaResource* aResource,
-                    AbstractMediaDecoder *aDecoder);
-
-  virtual status_t initCheck() const;
-  virtual ssize_t readAt(off64_t offset, void *data, size_t size);
-  virtual ssize_t readAt(off_t offset, void *data, size_t size) {
-    return readAt(static_cast<off64_t>(offset), data, size);
-  }
-  virtual status_t getSize(off_t *size) {
-    off64_t size64;
-    status_t status = getSize(&size64);
-    *size = size64;
-    return status;
-  }
-  virtual status_t getSize(off64_t *size);
-  virtual uint32_t flags() {
-    return kWantsPrefetching;
-  }
-
-  virtual ~MediaStreamSource();
-
-private:
-  MediaStreamSource(const MediaStreamSource &);
-  MediaStreamSource &operator=(const MediaStreamSource &);
-};
-
 class OmxDecoder : public OMXCodecProxy::EventListener {
   typedef MPAPI::AudioFrame AudioFrame;
   typedef MPAPI::VideoFrame VideoFrame;
@@ -83,6 +26,7 @@ class OmxDecoder : public OMXCodecProxy::EventListener {
   typedef mozilla::MediaResource MediaResource;
   typedef mozilla::AbstractMediaDecoder AbstractMediaDecoder;
   typedef mozilla::layers::FenceHandle FenceHandle;
+  typedef mozilla::layers::TextureClient TextureClient;
 
   enum {
     kPreferSoftwareCodecs = 1,
@@ -101,8 +45,11 @@ class OmxDecoder : public OMXCodecProxy::EventListener {
   sp<GonkNativeWindowClient> mNativeWindowClient;
   sp<MediaSource> mVideoTrack;
   sp<OMXCodecProxy> mVideoSource;
+  sp<MediaSource> mAudioOffloadTrack;
   sp<MediaSource> mAudioTrack;
   sp<MediaSource> mAudioSource;
+  int32_t mDisplayWidth;
+  int32_t mDisplayHeight;
   int32_t mVideoWidth;
   int32_t mVideoHeight;
   int32_t mVideoColorFormat;
@@ -216,9 +163,12 @@ public:
     *durationUs = mDurationUs;
   }
 
-  void GetVideoParameters(int32_t *width, int32_t *height) {
-    *width = mVideoWidth;
-    *height = mVideoHeight;
+  void GetVideoParameters(int32_t* aDisplayWidth, int32_t* aDisplayHeight,
+                          int32_t* aWidth, int32_t* aHeight) {
+    *aDisplayWidth = mDisplayWidth;
+    *aDisplayHeight = mDisplayHeight;
+    *aWidth = mVideoWidth;
+    *aHeight = mVideoHeight;
   }
 
   void GetAudioParameters(int32_t *numChannels, int32_t *sampleRate) {
@@ -255,7 +205,11 @@ public:
   // Called on ALooper thread.
   void onMessageReceived(const sp<AMessage> &msg);
 
-  bool ProcessCachedData(int64_t aOffset, bool aWaitForCompletion);
+  int64_t ProcessCachedData(int64_t aOffset, bool aWaitForCompletion);
+
+  sp<MediaSource> GetAudioOffloadTrack() { return mAudioOffloadTrack; }
+
+  static void RecycleCallback(TextureClient* aClient, void* aClosure);
 };
 
 }
