@@ -13,7 +13,7 @@
 #include "nsISeekableStream.h"
 
 #include <stdint.h>
-#include "mozilla/Util.h"
+#include "mozilla/ArrayUtils.h"
 #include "mozilla/CheckedInt.h"
 #include "mozilla/Endian.h"
 #include <algorithm>
@@ -125,8 +125,8 @@ nsresult WaveReader::Init(MediaDecoderReader* aCloneDonor)
   return NS_OK;
 }
 
-nsresult WaveReader::ReadMetadata(VideoInfo* aInfo,
-                                    MetadataTags** aTags)
+nsresult WaveReader::ReadMetadata(MediaInfo* aInfo,
+                                  MetadataTags** aTags)
 {
   NS_ASSERTION(mDecoder->OnDecodeThread(), "Should be on decode thread.");
 
@@ -142,10 +142,9 @@ nsresult WaveReader::ReadMetadata(VideoInfo* aInfo,
     return NS_ERROR_FAILURE;
   }
 
-  mInfo.mHasAudio = true;
-  mInfo.mHasVideo = false;
-  mInfo.mAudioRate = mSampleRate;
-  mInfo.mAudioChannels = mChannels;
+  mInfo.mAudio.mHasAudio = true;
+  mInfo.mAudio.mRate = mSampleRate;
+  mInfo.mAudio.mChannels = mChannels;
 
   *aInfo = mInfo;
 
@@ -159,6 +158,13 @@ nsresult WaveReader::ReadMetadata(VideoInfo* aInfo,
   return NS_OK;
 }
 
+bool
+WaveReader::IsMediaSeekable()
+{
+  // not used
+  return true;
+}
+
 template <typename T> T UnsignedByteToAudioSample(uint8_t aValue);
 template <typename T> T SignedShortToAudioSample(int16_t aValue);
 
@@ -273,7 +279,7 @@ static double RoundToUsecs(double aSeconds) {
 
 nsresult WaveReader::GetBuffered(dom::TimeRanges* aBuffered, int64_t aStartTime)
 {
-  if (!mInfo.mHasAudio) {
+  if (!mInfo.HasAudio()) {
     return NS_OK;
   }
   int64_t startOffset = mDecoder->GetResource()->GetNextCachedData(mWavePCMOffset);
@@ -309,7 +315,6 @@ WaveReader::ReadAll(char* aBuf, int64_t aSize, int64_t* aBytesRead)
     if (read == 0) {
       return false;
     }
-    mDecoder->NotifyBytesConsumed(read);
     got += read;
     if (aBytesRead) {
       *aBytesRead = got;
@@ -426,7 +431,7 @@ WaveReader::LoadFormatChunk(uint32_t aChunkSize)
   // but the channels check is intentionally limited to mono or stereo
   // when the media is intended for direct playback because that's what the
   // audio backend currently supports.
-  unsigned int actualFrameSize = sampleFormat == 8 ? 1 : 2 * channels;
+  unsigned int actualFrameSize = (sampleFormat == 8 ? 1 : 2) * channels;
   if (rate < 100 || rate > 96000 ||
       (((channels < 1 || channels > MAX_CHANNELS) ||
        (frameSize != 1 && frameSize != 2 && frameSize != 4)) &&
@@ -579,7 +584,7 @@ WaveReader::LoadListChunk(uint32_t aChunkSize,
     uint32_t length = ReadUint32LE(&p);
 
     // Subchunk shall not exceed parent chunk.
-    if (p + length > end) {
+    if (uint32_t(end - p) < length) {
       break;
     }
 
