@@ -20,17 +20,18 @@ class WaveShaperNode : public AudioNode
 {
 public:
   explicit WaveShaperNode(AudioContext *aContext);
-  virtual ~WaveShaperNode();
 
   NS_DECL_ISUPPORTS_INHERITED
   NS_DECL_CYCLE_COLLECTION_SCRIPT_HOLDER_CLASS_INHERITED(WaveShaperNode, AudioNode)
 
-  virtual JSObject* WrapObject(JSContext *aCx,
-                               JS::Handle<JSObject*> aScope) MOZ_OVERRIDE;
+  virtual JSObject* WrapObject(JSContext *aCx) MOZ_OVERRIDE;
 
-  JSObject* GetCurve(JSContext* aCx) const
+  void GetCurve(JSContext* aCx, JS::MutableHandle<JSObject*> aRetval) const
   {
-    return mCurve;
+    if (mCurve) {
+      JS::ExposeObjectToActiveJS(mCurve);
+    }
+    aRetval.set(mCurve);
   }
   void SetCurve(const Nullable<Float32Array>& aData);
 
@@ -40,6 +41,26 @@ public:
   }
   void SetOversample(OverSampleType aType);
 
+  virtual size_t SizeOfExcludingThis(MallocSizeOf aMallocSizeOf) const MOZ_OVERRIDE
+  {
+    // Possibly track in the future:
+    // - mCurve
+    return AudioNode::SizeOfExcludingThis(aMallocSizeOf);
+  }
+
+  virtual const char* NodeType() const
+  {
+    return "WaveShaperNode";
+  }
+
+  virtual size_t SizeOfIncludingThis(MallocSizeOf aMallocSizeOf) const MOZ_OVERRIDE
+  {
+    return aMallocSizeOf(this) + SizeOfExcludingThis(aMallocSizeOf);
+  }
+
+protected:
+  virtual ~WaveShaperNode();
+
 private:
   void ClearCurve();
 
