@@ -28,7 +28,6 @@
 #include "nsIDocument.h"
 #include "nsBindingManager.h"
 #include "nsIDOMNodeList.h"
-#include "nsINameSpaceManager.h"
 #include "nsIObserverService.h"
 #include "nsIRDFCompositeDataSource.h"
 #include "nsIRDFInferDataSource.h"
@@ -38,6 +37,7 @@
 #include "nsIXULBuilderListener.h"
 #include "nsIRDFRemoteDataSource.h"
 #include "nsIRDFService.h"
+#include "nsIScriptContext.h"
 #include "nsIScriptGlobalObject.h"
 #include "nsIServiceManager.h"
 #include "nsISimpleEnumerator.h"
@@ -45,6 +45,7 @@
 #include "nsIURL.h"
 #include "nsIXPConnect.h"
 #include "nsContentCID.h"
+#include "nsNameSpaceManager.h"
 #include "nsRDFCID.h"
 #include "nsXULContentUtils.h"
 #include "nsString.h"
@@ -68,16 +69,11 @@
 #include "nsXULTemplateQueryProcessorStorage.h"
 #include "nsContentUtils.h"
 #include "ChildIterator.h"
-#include "nsCxPusher.h"
+#include "mozilla/dom/ScriptSettings.h"
 
 using namespace mozilla::dom;
 using namespace mozilla;
 
-//----------------------------------------------------------------------
-
-static NS_DEFINE_CID(kRDFContainerUtilsCID,      NS_RDFCONTAINERUTILS_CID);
-static NS_DEFINE_CID(kRDFServiceCID,             NS_RDFSERVICE_CID);
-
 //----------------------------------------------------------------------
 //
 // nsXULTemplateBuilder
@@ -107,6 +103,7 @@ nsXULTemplateBuilder::nsXULTemplateBuilder(void)
       mTop(nullptr),
       mObservedDocument(nullptr)
 {
+    MOZ_COUNT_CTOR(nsXULTemplateBuilder);
 }
 
 static PLDHashOperator
@@ -133,6 +130,8 @@ nsXULTemplateBuilder::~nsXULTemplateBuilder(void)
         NS_IF_RELEASE(gScriptSecurityManager);
         NS_IF_RELEASE(gObserverService);
     }
+
+    MOZ_COUNT_DTOR(nsXULTemplateBuilder);
 }
 
 
@@ -144,10 +143,12 @@ nsXULTemplateBuilder::InitGlobals()
     if (gRefCnt++ == 0) {
         // Initialize the global shared reference to the service
         // manager and get some shared resource objects.
+        NS_DEFINE_CID(kRDFServiceCID, NS_RDFSERVICE_CID);
         rv = CallGetService(kRDFServiceCID, &gRDFService);
         if (NS_FAILED(rv))
             return rv;
 
+        NS_DEFINE_CID(kRDFContainerUtilsCID, NS_RDFCONTAINERUTILS_CID);
         rv = CallGetService(kRDFContainerUtilsCID, &gRDFContainerUtils);
         if (NS_FAILED(rv))
             return rv;
@@ -174,6 +175,23 @@ nsXULTemplateBuilder::InitGlobals()
     return NS_OK;
 }
 
+void
+nsXULTemplateBuilder::StartObserving(nsIDocument* aDocument)
+{
+    aDocument->AddObserver(this);
+    mObservedDocument = aDocument;
+    gObserverService->AddObserver(this, DOM_WINDOW_DESTROYED_TOPIC, false);
+}
+
+void
+nsXULTemplateBuilder::StopObserving()
+{
+    MOZ_ASSERT(mObservedDocument);
+    mObservedDocument->RemoveObserver(this);
+    mObservedDocument = nullptr;
+    gObserverService->RemoveObserver(this, DOM_WINDOW_DESTROYED_TOPIC);
+}
+
 void
 nsXULTemplateBuilder::CleanUp(bool aIsFinal)
 {
@@ -196,10 +214,7 @@ void
 nsXULTemplateBuilder::Uninit(bool aIsFinal)
 {
     if (mObservedDocument && aIsFinal) {
-        gObserverService->RemoveObserver(this, DOM_WINDOW_DESTROYED_TOPIC);
-        gObserverService->RemoveObserver(this, NS_XPCOM_SHUTDOWN_OBSERVER_ID);
-        mObservedDocument->RemoveObserver(this);
-        mObservedDocument = nullptr;
+        StopObserving();
     }
 
     if (mQueryProcessor)
@@ -431,14 +446,7 @@ nsXULTemplateBuilder::Init(nsIContent* aElement)
     nsresult rv = LoadDataSources(doc, &shouldDelay);
 
     if (NS_SUCCEEDED(rv)) {
-        // Add ourselves as a document observer
-        doc->AddObserver(this);
-
-        mObservedDocument = doc;
-        gObserverService->AddObserver(this, NS_XPCOM_SHUTDOWN_OBSERVER_ID,
-                                      false);
-        gObserverService->AddObserver(this, DOM_WINDOW_DESTROYED_TOPIC,
-                                      false);
+        StartObserving(doc);
     }
 
     return rv;
@@ -1065,7 +1073,7 @@ nsXULTemplateBuilder::RemoveListener(nsIXULBuilderListener* aListener)
 NS_IMETHODIMP
 nsXULTemplateBuilder::Observe(nsISupports* aSubject,
                               const char* aTopic,
-                              const PRUnichar* aData)
+                              const char16_t* aData)
 {
     // Uuuuber hack to clean up circular references that the cycle collector
     // doesn't know about. See bug 394514.
@@ -1076,8 +1084,6 @@ nsXULTemplateBuilder::Observe(nsISupports* aSubject,
             if (doc && doc == mObservedDocument)
                 NodeWillBeDestroyed(doc);
         }
-    } else if (!strcmp(aTopic, NS_XPCOM_SHUTDOWN_OBSERVER_ID)) {
-        UninitTrue();
     }
     return NS_OK;
 }
@@ -1127,7 +1133,8 @@ nsXULTemplateBuilder::ContentRemoved(nsIDocument* aDocument,
         nsContentUtils::AddScriptRunner(
             NS_NewRunnableMethod(this, &nsXULTemplateBuilder::UninitFalse));
 
-        aDocument->RemoveObserver(this);
+        MOZ_ASSERT(aDocument == mObservedDocument);
+        StopObserving();
 
         nsCOMPtr<nsIXULDocument> xuldoc = do_QueryInterface(aDocument);
         if (xuldoc)
@@ -1373,34 +1380,29 @@ nsXULTemplateBuilder::InitHTMLTemplateRoot()
     if (! global)
         return NS_ERROR_UNEXPECTED;
 
-    nsIScriptContext *context = global->GetContext();
-    if (! context)
-        return NS_ERROR_UNEXPECTED;
+    nsCOMPtr<nsIGlobalObject> innerWin =
+        do_QueryInterface(doc->GetInnerWindow());
 
-    AutoPushJSContext jscontext(context->GetNativeContext());
-    NS_ASSERTION(context != nullptr, "no jscontext");
-    if (! jscontext)
-        return NS_ERROR_UNEXPECTED;
+    // We are going to run script via JS_SetProperty, so we need a script entry
+    // point, but as this is XUL related it does not appear in the HTML spec.
+    AutoEntryScript entryScript(innerWin, true);
+    JSContext* jscontext = entryScript.cx();
 
-    JS::Rooted<JSObject*> scope(jscontext, global->GetGlobalJSObject());
     JS::Rooted<JS::Value> v(jscontext);
-    nsCOMPtr<nsIXPConnectJSObjectHolder> wrapper;
-    rv = nsContentUtils::WrapNative(jscontext, scope, mRoot, mRoot, v.address(),
-                                    getter_AddRefs(wrapper));
+    rv = nsContentUtils::WrapNative(jscontext, mRoot, mRoot, &v);
     NS_ENSURE_SUCCESS(rv, rv);
 
-    JS::Rooted<JSObject*> jselement(jscontext, JSVAL_TO_OBJECT(v));
+    JS::Rooted<JSObject*> jselement(jscontext, v.toObjectOrNull());
 
     if (mDB) {
         // database
         JS::Rooted<JS::Value> jsdatabase(jscontext);
-        rv = nsContentUtils::WrapNative(jscontext, scope, mDB,
+        rv = nsContentUtils::WrapNative(jscontext, mDB,
                                         &NS_GET_IID(nsIRDFCompositeDataSource),
-                                        jsdatabase.address(), getter_AddRefs(wrapper));
+                                        &jsdatabase);
         NS_ENSURE_SUCCESS(rv, rv);
 
-        bool ok;
-        ok = JS_SetProperty(jscontext, jselement, "database", jsdatabase);
+        bool ok = JS_SetProperty(jscontext, jselement, "database", jsdatabase);
         NS_ASSERTION(ok, "unable to set database property");
         if (! ok)
             return NS_ERROR_FAILURE;
@@ -1409,15 +1411,13 @@ nsXULTemplateBuilder::InitHTMLTemplateRoot()
     {
         // builder
         JS::Rooted<JS::Value> jsbuilder(jscontext);
-        nsCOMPtr<nsIXPConnectJSObjectHolder> wrapper;
-        rv = nsContentUtils::WrapNative(jscontext, jselement,
+        rv = nsContentUtils::WrapNative(jscontext,
                                         static_cast<nsIXULTemplateBuilder*>(this),
                                         &NS_GET_IID(nsIXULTemplateBuilder),
-                                        jsbuilder.address(), getter_AddRefs(wrapper));
+                                        &jsbuilder);
         NS_ENSURE_SUCCESS(rv, rv);
 
-        bool ok;
-        ok = JS_SetProperty(jscontext, jselement, "builder", jsbuilder);
+        bool ok = JS_SetProperty(jscontext, jselement, "builder", jsbuilder);
         if (! ok)
             return NS_ERROR_FAILURE;
     }
@@ -1470,13 +1470,13 @@ nsXULTemplateBuilder::ParseAttribute(const nsAString& aAttributeValue,
         // A variable is either prefixed with '?' (in the extended
         // syntax) or "rdf:" (in the simple syntax).
         bool isvar;
-        if (*iter == PRUnichar('?') && (++iter != done_parsing)) {
+        if (*iter == char16_t('?') && (++iter != done_parsing)) {
             isvar = true;
         }
-        else if ((*iter == PRUnichar('r') && (++iter != done_parsing)) &&
-                 (*iter == PRUnichar('d') && (++iter != done_parsing)) &&
-                 (*iter == PRUnichar('f') && (++iter != done_parsing)) &&
-                 (*iter == PRUnichar(':') && (++iter != done_parsing))) {
+        else if ((*iter == char16_t('r') && (++iter != done_parsing)) &&
+                 (*iter == char16_t('d') && (++iter != done_parsing)) &&
+                 (*iter == char16_t('f') && (++iter != done_parsing)) &&
+                 (*iter == char16_t(':') && (++iter != done_parsing))) {
             isvar = true;
         }
         else {
@@ -1497,7 +1497,7 @@ nsXULTemplateBuilder::ParseAttribute(const nsAString& aAttributeValue,
             (*aTextCallback)(this, Substring(mark, backup), aClosure);
         }
 
-        if (*iter == PRUnichar('?')) {
+        if (*iter == char16_t('?')) {
             // Well, it was not really a variable, but "??". We use one
             // question mark (the second one, actually) literally.
             mark = iter;
@@ -1510,10 +1510,10 @@ nsXULTemplateBuilder::ParseAttribute(const nsAString& aAttributeValue,
         // whichever comes first.
         nsAString::const_iterator first(backup);
 
-        PRUnichar c = 0;
+        char16_t c = 0;
         while (iter != done_parsing) {
             c = *iter;
-            if ((c == PRUnichar(' ')) || (c == PRUnichar('^')))
+            if ((c == char16_t(' ')) || (c == char16_t('^')))
                 break;
 
             ++iter;
@@ -1524,7 +1524,7 @@ nsXULTemplateBuilder::ParseAttribute(const nsAString& aAttributeValue,
         // Back up so we don't consume the terminating character
         // *unless* the terminating character was a caret: the caret
         // means "concatenate with no space in between".
-        if (c != PRUnichar('^'))
+        if (c != char16_t('^'))
             --iter;
 
         (*aVariableCallback)(this, Substring(first, last), aClosure);
@@ -1796,7 +1796,7 @@ nsXULTemplateBuilder::CompileTemplate(nsIContent* aTemplate,
          rulenode;
          rulenode = rulenode->GetNextSibling()) {
 
-        nsINodeInfo *ni = rulenode->NodeInfo();
+        mozilla::dom::NodeInfo *ni = rulenode->NodeInfo();
 
         // don't allow more queries than can be supported
         if (*aPriority == INT16_MAX)
@@ -2073,7 +2073,7 @@ nsXULTemplateBuilder::DetermineMemberVariable(nsIContent* aElement)
          child = child->GetNextSibling()) {
         nsAutoString uri;
         child->GetAttr(kNameSpaceID_None, nsGkAtoms::uri, uri);
-        if (!uri.IsEmpty() && uri[0] == PRUnichar('?')) {
+        if (!uri.IsEmpty() && uri[0] == char16_t('?')) {
             return NS_NewAtom(uri);
         }
 
@@ -2115,7 +2115,7 @@ nsXULTemplateBuilder::DetermineRDFQueryRef(nsIContent* aQueryElement, nsIAtom**
         content->GetAttr(kNameSpaceID_None, nsGkAtoms::tag, tag);
 
         if (!tag.IsEmpty())
-            *aTag = NS_NewAtom(tag).get();
+            *aTag = NS_NewAtom(tag).take();
     }
 }
 
@@ -2223,7 +2223,7 @@ nsXULTemplateBuilder::CompileWhereCondition(nsTemplateRule* aRule,
     }
 
     nsCOMPtr<nsIAtom> svar;
-    if (subject[0] == PRUnichar('?'))
+    if (subject[0] == char16_t('?'))
         svar = do_GetAtom(subject);
 
     nsAutoString relstring;
@@ -2247,7 +2247,7 @@ nsXULTemplateBuilder::CompileWhereCondition(nsTemplateRule* aRule,
                               nsGkAtoms::_true, eCaseMatters);
 
     nsCOMPtr<nsIAtom> vvar;
-    if (!shouldMultiple && (value[0] == PRUnichar('?'))) {
+    if (!shouldMultiple && (value[0] == char16_t('?'))) {
         vvar = do_GetAtom(value);
     }
 
@@ -2341,7 +2341,7 @@ nsXULTemplateBuilder::CompileBinding(nsTemplateRule* aRule,
     }
 
     nsCOMPtr<nsIAtom> svar;
-    if (subject[0] == PRUnichar('?')) {
+    if (subject[0] == char16_t('?')) {
         svar = do_GetAtom(subject);
     }
     else {
@@ -2367,7 +2367,7 @@ nsXULTemplateBuilder::CompileBinding(nsTemplateRule* aRule,
     }
 
     nsCOMPtr<nsIAtom> ovar;
-    if (object[0] == PRUnichar('?')) {
+    if (object[0] == char16_t('?')) {
         ovar = do_GetAtom(object);
     }
     else {
