@@ -17,7 +17,7 @@ class Matrix4x4;
 
 namespace layers {
 
-/* static */ LayersBackend Compositor::sBackend = LAYERS_NONE;
+/* static */ LayersBackend Compositor::sBackend = LayersBackend::LAYERS_NONE;
 /* static */ LayersBackend
 Compositor::GetBackend()
 {
@@ -25,6 +25,21 @@ Compositor::GetBackend()
   return sBackend;
 }
 
+/* static */ void
+Compositor::SetBackend(LayersBackend backend)
+{
+  if (sBackend != LayersBackend::LAYERS_NONE && sBackend != backend) {
+    // Assert this once we figure out bug 972891.
+    //MOZ_CRASH("Trying to use more than one OMTC compositor.");
+
+#ifdef XP_MACOSX
+    printf("ERROR: Changing compositor from %u to %u.\n",
+           unsigned(sBackend), unsigned(backend));
+#endif
+  }
+  sBackend = backend;
+}
+
 /* static */ void
 Compositor::AssertOnCompositorThread()
 {
@@ -36,14 +51,14 @@ Compositor::AssertOnCompositorThread()
 bool
 Compositor::ShouldDrawDiagnostics(DiagnosticFlags aFlags)
 {
-  if ((aFlags & DIAGNOSTIC_TILE) && !(mDiagnosticTypes & DIAGNOSTIC_TILE_BORDERS)) {
+  if ((aFlags & DiagnosticFlags::TILE) && !(mDiagnosticTypes & DiagnosticTypes::TILE_BORDERS)) {
     return false;
   }
-  if ((aFlags & DIAGNOSTIC_BIGIMAGE) &&
-      !(mDiagnosticTypes & DIAGNOSTIC_BIGIMAGE_BORDERS)) {
+  if ((aFlags & DiagnosticFlags::BIGIMAGE) &&
+      !(mDiagnosticTypes & DiagnosticTypes::BIGIMAGE_BORDERS)) {
     return false;
   }
-  if (!mDiagnosticTypes) {
+  if (mDiagnosticTypes == DiagnosticTypes::NO_DIAGNOSTIC) {
     return false;
   }
   return true;
@@ -53,7 +68,8 @@ void
 Compositor::DrawDiagnostics(DiagnosticFlags aFlags,
                             const nsIntRegion& aVisibleRegion,
                             const gfx::Rect& aClipRect,
-                            const gfx::Matrix4x4& aTransform)
+                            const gfx::Matrix4x4& aTransform,
+                            uint32_t aFlashCounter)
 {
   if (!ShouldDrawDiagnostics(aFlags)) {
     return;
@@ -64,34 +80,68 @@ Compositor::DrawDiagnostics(DiagnosticFlags aFlags,
 
     while (const nsIntRect* rect = screenIter.Next())
     {
-      DrawDiagnostics(aFlags | DIAGNOSTIC_REGION_RECT,
-                      ToRect(*rect), aClipRect, aTransform);
+      DrawDiagnostics(aFlags | DiagnosticFlags::REGION_RECT,
+                      ToRect(*rect), aClipRect, aTransform, aFlashCounter);
     }
   }
 
   DrawDiagnostics(aFlags, ToRect(aVisibleRegion.GetBounds()),
-                  aClipRect, aTransform);
+                  aClipRect, aTransform, aFlashCounter);
 }
 
 void
 Compositor::DrawDiagnostics(DiagnosticFlags aFlags,
                             const gfx::Rect& aVisibleRect,
                             const gfx::Rect& aClipRect,
-                            const gfx::Matrix4x4& aTransform)
+                            const gfx::Matrix4x4& aTransform,
+                            uint32_t aFlashCounter)
 {
   if (!ShouldDrawDiagnostics(aFlags)) {
     return;
   }
 
-  DrawDiagnosticsInternal(aFlags, aVisibleRect,
-                          aClipRect, aTransform);
+  DrawDiagnosticsInternal(aFlags, aVisibleRect, aClipRect, aTransform,
+                          aFlashCounter);
+}
+
+gfx::Rect
+Compositor::ClipRectInLayersCoordinates(gfx::Rect aClip) const {
+  gfx::Rect result;
+  aClip = aClip + GetCurrentRenderTarget()->GetOrigin();
+  gfx::IntSize destSize = GetWidgetSize();
+
+  switch (mScreenRotation) {
+    case ROTATION_0:
+      result = aClip;
+      break;
+    case ROTATION_90:
+      result = gfx::Rect(aClip.y,
+                         destSize.width - aClip.x - aClip.width,
+                         aClip.height, aClip.width);
+      break;
+    case ROTATION_270:
+      result = gfx::Rect(destSize.height - aClip.y - aClip.height,
+                         aClip.x,
+                         aClip.height, aClip.width);
+      break;
+    case ROTATION_180:
+      result = gfx::Rect(destSize.width - aClip.x - aClip.width,
+                         destSize.height - aClip.y - aClip.height,
+                         aClip.width, aClip.height);
+      break;
+      // ScreenRotation has a sentinel value, need to catch it in the switch
+      // statement otherwise the build fails (-WError)
+    default: {}
+  }
+  return result;
 }
 
 void
 Compositor::DrawDiagnosticsInternal(DiagnosticFlags aFlags,
                                     const gfx::Rect& aVisibleRect,
                                     const gfx::Rect& aClipRect,
-                                    const gfx::Matrix4x4& aTransform)
+                                    const gfx::Matrix4x4& aTransform,
+                                    uint32_t aFlashCounter)
 {
 #ifdef MOZ_B2G
   int lWidth = 4;
@@ -100,31 +150,38 @@ Compositor::DrawDiagnosticsInternal(DiagnosticFlags aFlags,
 #else
   int lWidth = 2;
 #endif
-  float opacity = 0.7;
+  float opacity = 0.7f;
 
   gfx::Color color;
-  if (aFlags & DIAGNOSTIC_CONTENT) {
-    color = gfx::Color(0.0, 1.0, 0.0, 1.0); // green
-    if (aFlags & DIAGNOSTIC_COMPONENT_ALPHA) {
-      color = gfx::Color(0.0, 1.0, 1.0, 1.0); // greenish blue
+  if (aFlags & DiagnosticFlags::CONTENT) {
+    color = gfx::Color(0.0f, 1.0f, 0.0f, 1.0f); // green
+    if (aFlags & DiagnosticFlags::COMPONENT_ALPHA) {
+      color = gfx::Color(0.0f, 1.0f, 1.0f, 1.0f); // greenish blue
     }
-  } else if (aFlags & DIAGNOSTIC_IMAGE) {
-    color = gfx::Color(1.0, 0.0, 0.0, 1.0); // red
-  } else if (aFlags & DIAGNOSTIC_COLOR) {
-    color = gfx::Color(0.0, 0.0, 1.0, 1.0); // blue
-  } else if (aFlags & DIAGNOSTIC_CONTAINER) {
-    color = gfx::Color(0.8, 0.0, 0.8, 1.0); // purple
+  } else if (aFlags & DiagnosticFlags::IMAGE) {
+    color = gfx::Color(1.0f, 0.0f, 0.0f, 1.0f); // red
+  } else if (aFlags & DiagnosticFlags::COLOR) {
+    color = gfx::Color(0.0f, 0.0f, 1.0f, 1.0f); // blue
+  } else if (aFlags & DiagnosticFlags::CONTAINER) {
+    color = gfx::Color(0.8f, 0.0f, 0.8f, 1.0f); // purple
   }
 
   // make tile borders a bit more transparent to keep layer borders readable.
-  if (aFlags & DIAGNOSTIC_TILE ||
-      aFlags & DIAGNOSTIC_BIGIMAGE ||
-      aFlags & DIAGNOSTIC_REGION_RECT) {
+  if (aFlags & DiagnosticFlags::TILE ||
+      aFlags & DiagnosticFlags::BIGIMAGE ||
+      aFlags & DiagnosticFlags::REGION_RECT) {
     lWidth = 1;
-    opacity = 0.5;
-    color.r *= 0.7;
-    color.g *= 0.7;
-    color.b *= 0.7;
+    opacity = 0.5f;
+    color.r *= 0.7f;
+    color.g *= 0.7f;
+    color.b *= 0.7f;
+  }
+
+  if (mDiagnosticTypes & DiagnosticTypes::FLASH_BORDERS) {
+    float flash = (float)aFlashCounter / (float)DIAGNOSTIC_FLASH_COUNTER_MAX;
+    color.r *= flash;
+    color.g *= flash;
+    color.b *= flash;
   }
 
   EffectChain effects;
