@@ -9,8 +9,6 @@
 // typedefs conflicts.
 #include "mozilla/ArrayUtils.h"
 
-#include "ipc/AutoOpenSurface.h"
-
 #include "ThebesLayerD3D9.h"
 #include "gfxPlatform.h"
 
@@ -52,7 +50,7 @@ void
 ThebesLayerD3D9::InvalidateRegion(const nsIntRegion &aRegion)
 {
   mInvalidRegion.Or(mInvalidRegion, aRegion);
-  mInvalidRegion.SimplifyOutward(10);
+  mInvalidRegion.SimplifyOutward(20);
   mValidRegion.Sub(mValidRegion, mInvalidRegion);
 }
 
@@ -98,11 +96,6 @@ ThebesLayerD3D9::CopyRegion(IDirect3DTexture9* aSrc, const nsIntPoint &aSrcOffse
   aValidRegion->And(*aValidRegion, retainedRegion);
 }
 
-static uint64_t RectArea(const nsIntRect& aRect)
-{
-  return aRect.width*uint64_t(aRect.height);
-}
-
 void
 ThebesLayerD3D9::UpdateTextures(SurfaceMode aMode)
 {
@@ -120,7 +113,7 @@ ThebesLayerD3D9::UpdateTextures(SurfaceMode aMode)
       // the new visible region
       retainRegion.And(mValidRegion, mVisibleRegion);
 
-      CreateNewTextures(gfxIntSize(visibleRect.width, visibleRect.height), aMode);
+      CreateNewTextures(gfx::IntSize(visibleRect.width, visibleRect.height), aMode);
 
       // If our texture creation failed this can mean a device reset is pending and we
       // should silently ignore the failure. In the future when device failures
@@ -131,7 +124,7 @@ ThebesLayerD3D9::UpdateTextures(SurfaceMode aMode)
       } else {
         CopyRegion(oldTexture, mTextureRect.TopLeft(), mTexture, visibleRect.TopLeft(),
                    retainRegion, &mValidRegion);
-        if (aMode == SURFACE_COMPONENT_ALPHA) {
+        if (aMode == SurfaceMode::SURFACE_COMPONENT_ALPHA) {
           CopyRegion(oldTextureOnWhite, mTextureRect.TopLeft(), mTextureOnWhite, visibleRect.TopLeft(),
                      retainRegion, &mValidRegion);
         }
@@ -140,7 +133,7 @@ ThebesLayerD3D9::UpdateTextures(SurfaceMode aMode)
       mTextureRect = visibleRect;
     }
   } else {
-    CreateNewTextures(gfxIntSize(visibleRect.width, visibleRect.height), aMode);
+    CreateNewTextures(gfx::IntSize(visibleRect.width, visibleRect.height), aMode);
     mTextureRect = visibleRect;
     
     NS_ASSERTION(mValidRegion.IsEmpty(), "Someone forgot to empty the region");
@@ -182,9 +175,9 @@ ThebesLayerD3D9::RenderThebesLayer(ReadbackProcessor* aReadback)
   nsIntRect newTextureRect = mVisibleRegion.GetBounds();
 
   SurfaceMode mode = GetSurfaceMode();
-  if (mode == SURFACE_COMPONENT_ALPHA &&
+  if (mode == SurfaceMode::SURFACE_COMPONENT_ALPHA &&
       (!mParent || !mParent->SupportsComponentAlphaChildren())) {
-    mode = SURFACE_SINGLE_CHANNEL_ALPHA;
+    mode = SurfaceMode::SURFACE_SINGLE_CHANNEL_ALPHA;
   }
   // If we have a transform that requires resampling of our texture, then
   // we need to make sure we don't sample pixels that haven't been drawn.
@@ -196,11 +189,11 @@ ThebesLayerD3D9::RenderThebesLayer(ReadbackProcessor* aReadback)
       neededRegion.GetNumRects() > 1) {
     if (MayResample()) {
       neededRegion = newTextureRect;
-      if (mode == SURFACE_OPAQUE) {
+      if (mode == SurfaceMode::SURFACE_OPAQUE) {
         // We're going to paint outside the visible region, but layout hasn't
         // promised that it will paint opaquely there, so we'll have to
         // treat this layer as transparent.
-        mode = SURFACE_SINGLE_CHANNEL_ALPHA;
+        mode = SurfaceMode::SURFACE_SINGLE_CHANNEL_ALPHA;
       }
     }
   }
@@ -246,7 +239,7 @@ ThebesLayerD3D9::RenderThebesLayer(ReadbackProcessor* aReadback)
 
   SetShaderTransformAndOpacity();
 
-  if (mode == SURFACE_COMPONENT_ALPHA) {
+  if (mode == SurfaceMode::SURFACE_COMPONENT_ALPHA) {
     mD3DManager->SetShaderMode(DeviceManagerD3D9::COMPONENTLAYERPASS1,
                                GetMaskLayer());
     device()->SetTexture(0, mTexture);
@@ -315,17 +308,17 @@ ThebesLayerD3D9::VerifyContentType(SurfaceMode aMode)
   mTexture->GetLevelDesc(0, &desc);
 
   switch (aMode) {
-  case SURFACE_OPAQUE:
+  case SurfaceMode::SURFACE_OPAQUE:
     if (desc.Format == D3DFMT_X8R8G8B8 && !mTextureOnWhite)
       return;
     break;
 
-  case SURFACE_SINGLE_CHANNEL_ALPHA:
+  case SurfaceMode::SURFACE_SINGLE_CHANNEL_ALPHA:
     if (desc.Format == D3DFMT_A8R8G8B8 && !mTextureOnWhite)
       return;
     break;
 
-  case SURFACE_COMPONENT_ALPHA:
+  case SurfaceMode::SURFACE_COMPONENT_ALPHA:
     if (mTextureOnWhite) {
       NS_ASSERTION(desc.Format == D3DFMT_X8R8G8B8, "Wrong format for component alpha texture");
       return;
@@ -405,61 +398,116 @@ OpaqueRenderer::End()
   mD3D9ThebesSurface = nullptr;
 
 }
+class TransparentRenderer {
+public:
+  TransparentRenderer(const nsIntRegion& aUpdateRegion) :
+    mUpdateRegion(aUpdateRegion) {}
+  ~TransparentRenderer() { End(); }
+  already_AddRefed<gfxImageSurface> Begin(LayerD3D9* aLayer);
+  void End();
+  IDirect3DTexture9* GetTexture() { return mTmpTexture; }
+
+private:
+  const nsIntRegion& mUpdateRegion;
+  nsRefPtr<IDirect3DTexture9> mTmpTexture;
+  nsRefPtr<gfxImageSurface> mD3D9ThebesSurface;
+};
+
+already_AddRefed<gfxImageSurface>
+TransparentRenderer::Begin(LayerD3D9* aLayer)
+{
+  nsIntRect bounds = mUpdateRegion.GetBounds();
+
+  HRESULT hr = aLayer->device()->
+      CreateTexture(bounds.width, bounds.height, 1, 0, D3DFMT_A8R8G8B8,
+                    D3DPOOL_SYSTEMMEM, getter_AddRefs(mTmpTexture), nullptr);
+
+  if (FAILED(hr)) {
+    aLayer->ReportFailure(NS_LITERAL_CSTRING("Failed to create temporary texture in system memory."), hr);
+    return nullptr;
+  }
+
+  D3DLOCKED_RECT r;
+  hr = mTmpTexture->LockRect(0, &r, nullptr, 0);
+  if (FAILED(hr)) {
+    // Uh-oh, bail.
+    NS_WARNING("Failed to lock the texture");
+    return nullptr;
+  }
+  nsRefPtr<gfxImageSurface> result =
+        new gfxImageSurface((unsigned char *)r.pBits,
+                            bounds.Size(),
+                            r.Pitch,
+                            gfxImageFormat::ARGB32);
+
+  if (!result || result->CairoStatus()) {
+    NS_WARNING("Failed to d3d9 cairo surface.");
+    return nullptr;
+  }
+  mD3D9ThebesSurface = result;
+
+  return result.forget();
+}
+
+void
+TransparentRenderer::End()
+{
+  // gfxImageSurface returned from ::Begin() should be released before the
+  // texture is used. This will assert that this is the case
+#if 1
+  if (mD3D9ThebesSurface) {
+    mD3D9ThebesSurface->AddRef();
+    nsrefcnt c = mD3D9ThebesSurface->Release();
+    if (c != 1)
+      NS_RUNTIMEABORT("Reference mD3D9ThebesSurface must be released by caller of Begin() before calling End()");
+  }
+#endif
+  mD3D9ThebesSurface = nullptr;
+  if (mTmpTexture)
+	  mTmpTexture->UnlockRect(0);
+}
 
 static void
 FillSurface(gfxASurface* aSurface, const nsIntRegion& aRegion,
             const nsIntPoint& aOffset, const gfxRGBA& aColor)
 {
-  nsRefPtr<gfxContext> ctx = new gfxContext(aSurface);
-  ctx->Translate(-gfxPoint(aOffset.x, aOffset.y));
-  gfxUtils::ClipToRegion(ctx, aRegion);
-  ctx->SetColor(aColor);
-  ctx->Paint();
+  nsIntRegionRectIterator iter(aRegion);
+  const nsIntRect* r;
+  while ((r = iter.Next()) != nullptr) {
+    nsIntRect rect = *r + aOffset;
+    gfxUtils::ClearThebesSurface(aSurface, &rect, aColor);
+  }
 }
 
 void
 ThebesLayerD3D9::DrawRegion(nsIntRegion &aRegion, SurfaceMode aMode,
                             const nsTArray<ReadbackProcessor::Update>& aReadbackUpdates)
 {
-  HRESULT hr;
   nsIntRect visibleRect = mVisibleRegion.GetBounds();
 
   nsRefPtr<gfxASurface> destinationSurface;
   nsIntRect bounds = aRegion.GetBounds();
   nsRefPtr<IDirect3DTexture9> tmpTexture;
   OpaqueRenderer opaqueRenderer(aRegion);
+  TransparentRenderer transparentRenderer(aRegion);
   OpaqueRenderer opaqueRendererOnWhite(aRegion);
 
   switch (aMode)
   {
-    case SURFACE_OPAQUE:
+    case SurfaceMode::SURFACE_OPAQUE:
       destinationSurface = opaqueRenderer.Begin(this);
       break;
 
-    case SURFACE_SINGLE_CHANNEL_ALPHA: {
-      hr = device()->CreateTexture(bounds.width, bounds.height, 1,
-                                   0, D3DFMT_A8R8G8B8,
-                                   D3DPOOL_SYSTEMMEM, getter_AddRefs(tmpTexture), nullptr);
-
-      if (FAILED(hr)) {
-        ReportFailure(NS_LITERAL_CSTRING("Failed to create temporary texture in system memory."), hr);
-        return;
-      }
-
-      // XXX - We may consider retaining a SYSTEMMEM texture texture the size
-      // of our DEFAULT texture and then use UpdateTexture and add dirty rects
-      // to update in a single call.
-      nsRefPtr<gfxWindowsSurface> dest = new gfxWindowsSurface(
-          gfxIntSize(bounds.width, bounds.height), gfxImageFormatARGB32);
+    case SurfaceMode::SURFACE_SINGLE_CHANNEL_ALPHA: {
+	  destinationSurface = transparentRenderer.Begin(this);
       // If the contents of this layer don't require component alpha in the
       // end of rendering, it's safe to enable Cleartype since all the Cleartype
       // glyphs must be over (or under) opaque pixels.
-      dest->SetSubpixelAntialiasingEnabled(!(mContentFlags & CONTENT_COMPONENT_ALPHA));
-      destinationSurface = dest.forget();
+      destinationSurface->SetSubpixelAntialiasingEnabled(!(mContentFlags & CONTENT_COMPONENT_ALPHA));
       break;
     }
 
-    case SURFACE_COMPONENT_ALPHA: {
+    case SurfaceMode::SURFACE_COMPONENT_ALPHA: {
       nsRefPtr<gfxWindowsSurface> onBlack = opaqueRenderer.Begin(this);
       nsRefPtr<gfxWindowsSurface> onWhite = opaqueRendererOnWhite.Begin(this);
       if (onBlack && onWhite) {
@@ -479,24 +527,20 @@ ThebesLayerD3D9::DrawRegion(nsIntRegion &aRegion, SurfaceMode aMode,
   if (!destinationSurface)
     return;
 
-  nsRefPtr<gfxContext> context;
-  if (gfxPlatform::GetPlatform()->SupportsAzureContentForType(BACKEND_CAIRO)) {
-     RefPtr<DrawTarget> dt =
-        gfxPlatform::GetPlatform()->CreateDrawTargetForSurface(destinationSurface,
-                                                               IntSize(destinationSurface->GetSize().width,
-                                                                       destinationSurface->GetSize().height));
+  MOZ_ASSERT(gfxPlatform::GetPlatform()->SupportsAzureContentForType(BackendType::CAIRO));
+  RefPtr<DrawTarget> dt =
+    gfxPlatform::GetPlatform()->CreateDrawTargetForSurface(destinationSurface,
+                                                           IntSize(destinationSurface->GetSize().width,
+                                                                   destinationSurface->GetSize().height));
 
-    context = new gfxContext(dt);
-  } else {
-    context = new gfxContext(destinationSurface);
-  }
+  nsRefPtr<gfxContext> context = new gfxContext(dt);
 
   context->Translate(gfxPoint(-bounds.x, -bounds.y));
   LayerManagerD3D9::CallbackInfo cbInfo = mD3DManager->GetCallbackInfo();
-  cbInfo.Callback(this, context, aRegion, CLIP_NONE, nsIntRegion(), cbInfo.CallbackData);
+  cbInfo.Callback(this, context, aRegion, DrawRegionClip::CLIP_NONE, nsIntRegion(), cbInfo.CallbackData);
 
   for (uint32_t i = 0; i < aReadbackUpdates.Length(); ++i) {
-    NS_ASSERTION(aMode == SURFACE_OPAQUE,
+    NS_ASSERTION(aMode == SurfaceMode::SURFACE_OPAQUE,
                  "Transparent surfaces should not be used for readback");
     const ReadbackProcessor::Update& update = aReadbackUpdates[i];
     nsIntPoint offset = update.mLayer->GetBackgroundLayerOffset();
@@ -513,12 +557,13 @@ ThebesLayerD3D9::DrawRegion(nsIntRegion &aRegion, SurfaceMode aMode,
 
   // Release the cairo d3d9 surface before we try to composite it
   context = nullptr;
+  dt = nullptr;
 
   nsAutoTArray<IDirect3DTexture9*,2> srcTextures;
   nsAutoTArray<IDirect3DTexture9*,2> destTextures;
   switch (aMode)
   {
-    case SURFACE_OPAQUE:
+    case SurfaceMode::SURFACE_OPAQUE:
       // Must release reference to dest surface before ending drawing
       destinationSurface = nullptr;
       opaqueRenderer.End();
@@ -526,38 +571,15 @@ ThebesLayerD3D9::DrawRegion(nsIntRegion &aRegion, SurfaceMode aMode,
       destTextures.AppendElement(mTexture);
       break;
 
-    case SURFACE_SINGLE_CHANNEL_ALPHA: {
-      LockTextureRectD3D9 textureLock(tmpTexture);
-      if (!textureLock.HasLock()) {
-        NS_WARNING("Failed to lock ThebesLayer tmpTexture texture.");
-        return;
-      }
-
-      D3DLOCKED_RECT r = textureLock.GetLockRect();
-
-      nsRefPtr<gfxImageSurface> imgSurface =
-        new gfxImageSurface((unsigned char *)r.pBits,
-                            bounds.Size(),
-                            r.Pitch,
-                            gfxImageFormatARGB32);
-
-      if (destinationSurface) {
-        nsRefPtr<gfxContext> context = new gfxContext(imgSurface);
-        context->SetSource(destinationSurface);
-        context->SetOperator(gfxContext::OPERATOR_SOURCE);
-        context->Paint();
-      }
-
-      // Must release reference to dest surface before ending drawing
+    case SurfaceMode::SURFACE_SINGLE_CHANNEL_ALPHA:
+	  // Must release reference to dest surface before ending drawing
       destinationSurface = nullptr;
-      imgSurface = nullptr;
-
-      srcTextures.AppendElement(tmpTexture);
+      transparentRenderer.End();
+      srcTextures.AppendElement(transparentRenderer.GetTexture());
       destTextures.AppendElement(mTexture);
       break;
-    }
 
-    case SURFACE_COMPONENT_ALPHA: {
+    case SurfaceMode::SURFACE_COMPONENT_ALPHA: {
       // Must release reference to dest surface before ending drawing
       destinationSurface = nullptr;
       opaqueRenderer.End();
@@ -598,8 +620,8 @@ ThebesLayerD3D9::DrawRegion(nsIntRegion &aRegion, SurfaceMode aMode,
 }
 
 void
-ThebesLayerD3D9::CreateNewTextures(const gfxIntSize &aSize,
-                                   SurfaceMode aMode)
+ThebesLayerD3D9::CreateNewTextures(const gfx::IntSize &aSize,
+                                  SurfaceMode aMode)
 {
   if (aSize.width == 0 || aSize.height == 0) {
     // Nothing to do.
@@ -610,7 +632,7 @@ ThebesLayerD3D9::CreateNewTextures(const gfxIntSize &aSize,
   mTextureOnWhite = nullptr;
   HRESULT hr = device()->CreateTexture(aSize.width, aSize.height, 1,
                                        D3DUSAGE_RENDERTARGET,
-                                       aMode != SURFACE_SINGLE_CHANNEL_ALPHA ? D3DFMT_X8R8G8B8 : D3DFMT_A8R8G8B8,
+                                       aMode != SurfaceMode::SURFACE_SINGLE_CHANNEL_ALPHA ? D3DFMT_X8R8G8B8 : D3DFMT_A8R8G8B8,
                                        D3DPOOL_DEFAULT, getter_AddRefs(mTexture), nullptr);
   if (FAILED(hr)) {
     ReportFailure(NS_LITERAL_CSTRING("ThebesLayerD3D9::CreateNewTextures(): Failed to create texture"),
@@ -618,7 +640,7 @@ ThebesLayerD3D9::CreateNewTextures(const gfxIntSize &aSize,
     return;
   }
 
-  if (aMode == SURFACE_COMPONENT_ALPHA) {
+  if (aMode == SurfaceMode::SURFACE_COMPONENT_ALPHA) {
     hr = device()->CreateTexture(aSize.width, aSize.height, 1,
                                  D3DUSAGE_RENDERTARGET,
                                  D3DFMT_X8R8G8B8,
