@@ -22,19 +22,26 @@ public:
 
   void InitWith(MacIOSurface* aSurface);
 
+  virtual bool Lock(OpenMode aMode) MOZ_OVERRIDE;
+
+  virtual void Unlock() MOZ_OVERRIDE;
+
+  virtual bool IsLocked() const MOZ_OVERRIDE;
+
   virtual bool IsAllocated() const MOZ_OVERRIDE { return !!mSurface; }
 
   virtual bool ToSurfaceDescriptor(SurfaceDescriptor& aOutDescriptor) MOZ_OVERRIDE;
 
   virtual gfx::IntSize GetSize() const;
 
-  virtual TextureClientData* DropTextureData() MOZ_OVERRIDE;
+  virtual bool HasInternalBuffer() const MOZ_OVERRIDE { return false; }
 
 protected:
   RefPtr<MacIOSurface> mSurface;
+  bool mIsLocked;
 };
 
 }
 }
 
-#endif // MOZILLA_GFX_MACIOSURFACETEXTURECLIENTOGL_H
\ No newline at end of file
+#endif // MOZILLA_GFX_MACIOSURFACETEXTURECLIENTOGL_H
