@@ -42,16 +42,17 @@ public:
   NS_DECL_NSIOBSERVER
 
   ObserverToDestroyFeaturesAlreadyReported() {}
+private:
   virtual ~ObserverToDestroyFeaturesAlreadyReported() {}
 };
 
-NS_IMPL_ISUPPORTS1(ObserverToDestroyFeaturesAlreadyReported,
-                   nsIObserver)
+NS_IMPL_ISUPPORTS(ObserverToDestroyFeaturesAlreadyReported,
+                  nsIObserver)
 
 NS_IMETHODIMP
 ObserverToDestroyFeaturesAlreadyReported::Observe(nsISupports* aSubject,
                                                   const char* aTopic,
-                                                  const PRUnichar* aData)
+                                                  const char16_t* aData)
 {
   if (!strcmp(aTopic, "xpcom-shutdown")) {
     if (gFeaturesAlreadyReported) {
@@ -106,7 +107,7 @@ void
 ScopedGfxFeatureReporter::WriteAppNote(char statusChar)
 {
   nsCOMPtr<nsIRunnable> r = new AppNoteWritingRunnable(statusChar, mFeature);
-  NS_DispatchToMainThread(r.get(), NS_DISPATCH_NORMAL);
+  NS_DispatchToMainThread(r);
 }
 
 } // end namespace mozilla
