@@ -53,7 +53,7 @@ public:
                                gfxContext *aContextForTightBoundingBox,
                                Spacing *aSpacing);
 
-    virtual bool ProvidesGlyphWidths();
+    virtual bool ProvidesGlyphWidths() const;
 
     virtual int32_t GetGlyphWidth(gfxContext *aCtx, uint16_t aGID);
 
@@ -71,10 +71,6 @@ public:
     virtual cairo_scaled_font_t *GetCairoScaledFont();
 
 protected:
-    friend class gfxDWriteShaper;
-
-    virtual void CreatePlatformShaper();
-
     bool GetFakeMetricsForArialBlack(DWRITE_FONT_METRICS *aFontMetrics);
 
     void ComputeMetrics(AntialiasOption anAAOption);
