@@ -8,9 +8,9 @@
 #define gc_GCInternals_h
 
 #include "jscntxt.h"
-#include "jsworkers.h"
 
 #include "gc/Zone.h"
+#include "vm/HelperThreads.h"
 #include "vm/Runtime.h"
 
 namespace js {
@@ -19,11 +19,12 @@ namespace gc {
 void
 MarkPersistentRootedChains(JSTracer *trc);
 
-void
-MarkRuntime(JSTracer *trc, bool useSavedRoots = false);
+#ifdef JSGC_FJGENERATIONAL
+class ForkJoinNurseryCollectionTracer;
 
 void
-BufferGrayRoots(GCMarker *gcmarker);
+MarkForkJoinStack(ForkJoinNurseryCollectionTracer *trc);
+#endif
 
 class AutoCopyFreeListToArenas
 {
@@ -37,7 +38,7 @@ class AutoCopyFreeListToArenas
 
 struct AutoFinishGC
 {
-    AutoFinishGC(JSRuntime *rt);
+    explicit AutoFinishGC(JSRuntime *rt);
 };
 
 /*
@@ -47,7 +48,7 @@ struct AutoFinishGC
 class AutoTraceSession
 {
   public:
-    AutoTraceSession(JSRuntime *rt, HeapState state = Tracing);
+    explicit AutoTraceSession(JSRuntime *rt, HeapState state = Tracing);
     ~AutoTraceSession();
 
   protected:
@@ -74,7 +75,7 @@ class IncrementalSafety
 {
     const char *reason_;
 
-    IncrementalSafety(const char *reason) : reason_(reason) {}
+    explicit IncrementalSafety(const char *reason) : reason_(reason) {}
 
   public:
     static IncrementalSafety Safe() { return IncrementalSafety(nullptr); }
@@ -96,53 +97,31 @@ class IncrementalSafety
 IncrementalSafety
 IsIncrementalGCSafe(JSRuntime *rt);
 
-#ifdef JSGC_ROOT_ANALYSIS
-void *
-GetAddressableGCThing(JSRuntime *rt, uintptr_t w);
-#endif
-
 #ifdef JS_GC_ZEAL
-void
-StartVerifyPreBarriers(JSRuntime *rt);
-
-void
-EndVerifyPreBarriers(JSRuntime *rt);
-
-void
-StartVerifyPostBarriers(JSRuntime *rt);
-
-void
-EndVerifyPostBarriers(JSRuntime *rt);
-
-void
-FinishVerifier(JSRuntime *rt);
 
 class AutoStopVerifyingBarriers
 {
-    JSRuntime *runtime;
+    GCRuntime *gc;
     bool restartPreVerifier;
     bool restartPostVerifier;
     MOZ_DECL_USE_GUARD_OBJECT_NOTIFIER
 
   public:
     AutoStopVerifyingBarriers(JSRuntime *rt, bool isShutdown
-                       MOZ_GUARD_OBJECT_NOTIFIER_PARAM)
-      : runtime(rt)
+                              MOZ_GUARD_OBJECT_NOTIFIER_PARAM)
+      : gc(&rt->gc)
     {
-        restartPreVerifier = !isShutdown && rt->gcVerifyPreData;
-        restartPostVerifier = !isShutdown && rt->gcVerifyPostData && rt->gcGenerationalEnabled;
-        if (rt->gcVerifyPreData)
-            EndVerifyPreBarriers(rt);
-        if (rt->gcVerifyPostData)
-            EndVerifyPostBarriers(rt);
+        restartPreVerifier = gc->endVerifyPreBarriers() && !isShutdown;
+        restartPostVerifier = gc->endVerifyPostBarriers() && !isShutdown &&
+            JS::IsGenerationalGCEnabled(rt);
         MOZ_GUARD_OBJECT_NOTIFIER_INIT;
     }
 
     ~AutoStopVerifyingBarriers() {
         if (restartPreVerifier)
-            StartVerifyPreBarriers(runtime);
+            gc->startVerifyPreBarriers();
         if (restartPostVerifier)
-            StartVerifyPostBarriers(runtime);
+            gc->startVerifyPostBarriers();
     }
 };
 #else
