@@ -21,7 +21,7 @@ bool
 C1Spewer::init(const char *path)
 {
     spewout_ = fopen(path, "w");
-    return (spewout_ != nullptr);
+    return spewout_ != nullptr;
 }
 
 void
@@ -31,12 +31,11 @@ C1Spewer::beginFunction(MIRGraph *graph, HandleScript script)
         return;
 
     this->graph  = graph;
-    this->script.repoint(script);
 
     fprintf(spewout_, "begin_compilation\n");
     if (script) {
-        fprintf(spewout_, "  name \"%s:%d\"\n", script->filename(), script->lineno);
-        fprintf(spewout_, "  method \"%s:%d\"\n", script->filename(), script->lineno);
+        fprintf(spewout_, "  name \"%s:%d\"\n", script->filename(), (int)script->lineno());
+        fprintf(spewout_, "  method \"%s:%d\"\n", script->filename(), (int)script->lineno());
     } else {
         fprintf(spewout_, "  name \"asm.js compilation\"\n");
         fprintf(spewout_, "  method \"asm.js compilation\"\n");
@@ -124,11 +123,11 @@ C1Spewer::spewIntervals(FILE *fp, LinearScanAllocator *regalloc, LInstruction *i
                 fprintf(fp, "%s", live->getAllocation()->toString());
                 fprintf(fp, "\" %d -1", id);
                 for (size_t j = 0; j < live->numRanges(); j++) {
-                    fprintf(fp, " [%d, %d[", live->getRange(j)->from.pos(),
-                            live->getRange(j)->to.pos());
+                    fprintf(fp, " [%u, %u[", live->getRange(j)->from.bits(),
+                            live->getRange(j)->to.bits());
                 }
                 for (UsePositionIterator usePos(live->usesBegin()); usePos != live->usesEnd(); usePos++)
-                    fprintf(fp, " %d M", usePos->pos.pos());
+                    fprintf(fp, " %u M", usePos->pos.bits());
                 fprintf(fp, " \"\"\n");
             }
         }
