@@ -27,16 +27,14 @@ class TempAllocator
     CompilerRootNode *rootList_;
 
   public:
-    TempAllocator(LifoAlloc *lifoAlloc)
+    explicit TempAllocator(LifoAlloc *lifoAlloc)
       : lifoScope_(lifoAlloc),
         rootList_(nullptr)
     { }
 
     void *allocateInfallible(size_t bytes)
     {
-        void *p = lifoScope_.alloc().allocInfallible(bytes);
-        JS_ASSERT(p);
-        return p;
+        return lifoScope_.alloc().allocInfallible(bytes);
     }
 
     void *allocate(size_t bytes)
@@ -47,6 +45,17 @@ class TempAllocator
         return p;
     }
 
+    template <size_t ElemSize>
+    void *allocateArray(size_t n)
+    {
+        if (n & mozilla::tl::MulOverflowMask<ElemSize>::value)
+            return nullptr;
+        void *p = lifoScope_.alloc().alloc(n * ElemSize);
+        if (!ensureBallast())
+            return nullptr;
+        return p;
+    }
+
     LifoAlloc *lifoAlloc()
     {
         return &lifoScope_.alloc();
@@ -65,17 +74,17 @@ class TempAllocator
 };
 
 // Stack allocated rooter for all roots associated with a TempAllocator
-class AutoTempAllocatorRooter : private AutoGCRooter
+class AutoTempAllocatorRooter : private JS::AutoGCRooter
 {
   public:
     explicit AutoTempAllocatorRooter(JSContext *cx, TempAllocator *temp
                                      MOZ_GUARD_OBJECT_NOTIFIER_PARAM)
-      : AutoGCRooter(cx, IONALLOC), temp(temp)
+      : JS::AutoGCRooter(cx, IONALLOC), temp(temp)
     {
         MOZ_GUARD_OBJECT_NOTIFIER_INIT;
     }
 
-    friend void AutoGCRooter::trace(JSTracer *trc);
+    friend void JS::AutoGCRooter::trace(JSTracer *trc);
     void trace(JSTracer *trc);
 
   private:
@@ -88,7 +97,7 @@ class IonAllocPolicy
     TempAllocator &alloc_;
 
   public:
-    IonAllocPolicy(TempAllocator &alloc)
+    MOZ_IMPLICIT IonAllocPolicy(TempAllocator &alloc)
       : alloc_(alloc)
     {}
     void *malloc_(size_t bytes) {
