@@ -7,13 +7,16 @@
 #ifndef jit_IonCode_h
 #define jit_IonCode_h
 
+#include "mozilla/Atomics.h"
 #include "mozilla/MemoryReporting.h"
 #include "mozilla/PodOperations.h"
 
 #include "jsinfer.h"
 #include "jstypes.h"
 
+#include "assembler/jit/ExecutableAllocator.h"
 #include "gc/Heap.h"
+#include "jit/IonOptimizationLevels.h"
 #include "jit/IonTypes.h"
 
 namespace JSC {
@@ -26,42 +29,37 @@ class AsmJSModule;
 
 namespace jit {
 
-// The maximum size of any buffer associated with an assembler or code object.
-// This is chosen to not overflow a signed integer, leaving room for an extra
-// bit on offsets.
-static const uint32_t MAX_BUFFER_SIZE = (1 << 30) - 1;
-
-// Maximum number of scripted arg slots.
-static const uint32_t SNAPSHOT_MAX_NARGS = 127;
-
 class MacroAssembler;
 class CodeOffsetLabel;
 class PatchableBackedge;
 
-class IonCode : public gc::BarrieredCell<IonCode>
+class JitCode : public gc::BarrieredCell<JitCode>
 {
   protected:
     uint8_t *code_;
     JSC::ExecutablePool *pool_;
-    uint32_t bufferSize_;             // Total buffer size.
+    uint32_t bufferSize_;             // Total buffer size. Does not include headerSize_.
     uint32_t insnSize_;               // Instruction stream size.
     uint32_t dataSize_;               // Size of the read-only data area.
     uint32_t jumpRelocTableBytes_;    // Size of the jump relocation table.
     uint32_t dataRelocTableBytes_;    // Size of the data relocation table.
     uint32_t preBarrierTableBytes_;   // Size of the prebarrier table.
-    bool invalidated_;                // Whether the code object has been invalidated.
+    uint8_t headerSize_ : 5;          // Number of bytes allocated before codeStart.
+    uint8_t kind_ : 3;                // JSC::CodeKind, for the memory reporters.
+    bool invalidated_ : 1;            // Whether the code object has been invalidated.
                                       // This is necessary to prevent GC tracing.
 
 #if JS_BITS_PER_WORD == 32
-    // Ensure IonCode is gc::Cell aligned.
+    // Ensure JitCode is gc::Cell aligned.
     uint32_t padding_;
 #endif
 
-    IonCode()
+    JitCode()
       : code_(nullptr),
         pool_(nullptr)
     { }
-    IonCode(uint8_t *code, uint32_t bufferSize, JSC::ExecutablePool *pool)
+    JitCode(uint8_t *code, uint32_t bufferSize, uint32_t headerSize, JSC::ExecutablePool *pool,
+            JSC::CodeKind kind)
       : code_(code),
         pool_(pool),
         bufferSize_(bufferSize),
@@ -70,8 +68,13 @@ class IonCode : public gc::BarrieredCell<IonCode>
         jumpRelocTableBytes_(0),
         dataRelocTableBytes_(0),
         preBarrierTableBytes_(0),
+        headerSize_(headerSize),
+        kind_(kind),
         invalidated_(false)
-    { }
+    {
+        MOZ_ASSERT(JSC::CodeKind(kind_) == kind);
+        MOZ_ASSERT(headerSize_ == headerSize);
+    }
 
     uint32_t dataOffset() const {
         return insnSize_;
@@ -101,7 +104,7 @@ class IonCode : public gc::BarrieredCell<IonCode>
 
     void togglePreBarriers(bool enabled);
 
-    // If this IonCode object has been, effectively, corrupted due to
+    // If this JitCode object has been, effectively, corrupted due to
     // invalidation patching, then we have to remember this so we don't try and
     // trace relocation entries that may now be corrupt.
     bool invalidated() const {
@@ -114,31 +117,33 @@ class IonCode : public gc::BarrieredCell<IonCode>
 
     void copyFrom(MacroAssembler &masm);
 
-    static IonCode *FromExecutable(uint8_t *buffer) {
-        IonCode *code = *(IonCode **)(buffer - sizeof(IonCode *));
+    static JitCode *FromExecutable(uint8_t *buffer) {
+        JitCode *code = *(JitCode **)(buffer - sizeof(JitCode *));
         JS_ASSERT(code->raw() == buffer);
         return code;
     }
 
     static size_t offsetOfCode() {
-        return offsetof(IonCode, code_);
+        return offsetof(JitCode, code_);
     }
 
     uint8_t *jumpRelocTable() {
         return code_ + jumpRelocTableOffset();
     }
 
-    // Allocates a new IonCode object which will be managed by the GC. If no
+    // Allocates a new JitCode object which will be managed by the GC. If no
     // object can be allocated, nullptr is returned. On failure, |pool| is
     // automatically released, so the code may be freed.
     template <AllowGC allowGC>
-    static IonCode *New(JSContext *cx, uint8_t *code, uint32_t bufferSize, JSC::ExecutablePool *pool);
+    static JitCode *New(JSContext *cx, uint8_t *code, uint32_t bufferSize, uint32_t headerSize,
+                        JSC::ExecutablePool *pool, JSC::CodeKind kind);
 
   public:
-    static inline ThingRootKind rootKind() { return THING_ROOT_ION_CODE; }
+    static inline ThingRootKind rootKind() { return THING_ROOT_JIT_CODE; }
 };
 
 class SnapshotWriter;
+class RecoverWriter;
 class SafepointWriter;
 class SafepointIndex;
 class OsiIndex;
@@ -164,10 +169,10 @@ struct IonScript
 {
   private:
     // Code pointer containing the actual method.
-    EncapsulatedPtr<IonCode> method_;
+    PreBarrieredJitCode method_;
 
     // Deoptimization table used by this method.
-    EncapsulatedPtr<IonCode> deoptTable_;
+    PreBarrieredJitCode deoptTable_;
 
     // Entrypoint for OSR, or nullptr.
     jsbytecode *osrPc_;
@@ -194,13 +199,18 @@ struct IonScript
     // Flag set when it is likely that one of our (transitive) call
     // targets is not compiled.  Used in ForkJoin.cpp to decide when
     // we should add call targets to the worklist.
-    bool hasUncompiledCallTarget_;
+    mozilla::Atomic<bool, mozilla::Relaxed> hasUncompiledCallTarget_;
+
+    // Flag set when this script is used as an entry script to parallel
+    // execution. If this is true, then the parent JSScript must be in its
+    // JitCompartment's parallel entry script set.
+    bool isParallelEntryScript_;
 
     // Flag set if IonScript was compiled with SPS profiling enabled.
     bool hasSPSInstrumentation_;
 
     // Flag for if this script is getting recompiled.
-    bool recompiling_;
+    uint32_t recompiling_;
 
     // Any kind of data needed by the runtime, these can be either cache
     // information or profiling info.
@@ -221,8 +231,7 @@ struct IonScript
     uint32_t safepointsStart_;
     uint32_t safepointsSize_;
 
-    // Number of STACK_SLOT_SIZE-length slots this function reserves on the
-    // stack.
+    // Number of bytes this function reserves on the stack.
     uint32_t frameSlots_;
 
     // Frame size is the value that can be added to the StackPointer along
@@ -239,7 +248,12 @@ struct IonScript
 
     // Offset from the start of the code buffer to its snapshot buffer.
     uint32_t snapshots_;
-    uint32_t snapshotsSize_;
+    uint32_t snapshotsListSize_;
+    uint32_t snapshotsRVATableSize_;
+
+    // List of instructions needed to recover stack frames.
+    uint32_t recovers_;
+    uint32_t recoversSize_;
 
     // Constant table for constants stored in snapshots.
     uint32_t constantTable_;
@@ -256,11 +270,21 @@ struct IonScript
     uint32_t backedgeEntries_;
 
     // Number of references from invalidation records.
-    size_t refcount_;
+    uint32_t refcount_;
+
+    // If this is a parallel script, the number of major GC collections it has
+    // been idle, otherwise 0.
+    //
+    // JSScripts with parallel IonScripts are preserved across GC if the
+    // parallel age is < MAX_PARALLEL_AGE.
+    uint32_t parallelAge_;
 
     // Identifier of the compilation which produced this code.
     types::RecompileInfo recompileInfo_;
 
+    // The optimization level this script was compiled in.
+    OptimizationLevel optimizationLevel_;
+
     // Number of times we tried to enter this script via OSR but failed due to
     // a LOOPENTRY pc other than osrPc_.
     uint32_t osrPcMismatchCounter_;
@@ -281,8 +305,8 @@ struct IonScript
     SnapshotOffset *bailoutTable() {
         return (SnapshotOffset *) &bottomBuffer()[bailoutTable_];
     }
-    EncapsulatedValue *constants() {
-        return (EncapsulatedValue *) &bottomBuffer()[constantTable_];
+    PreBarrieredValue *constants() {
+        return (PreBarrieredValue *) &bottomBuffer()[constantTable_];
     }
     const SafepointIndex *safepointIndices() const {
         return const_cast<IonScript *>(this)->safepointIndices();
@@ -321,7 +345,6 @@ struct IonScript
             }
         }
     }
-    void detachDependentAsmJSModules(FreeOp *fop);
 
   private:
     void trace(JSTracer *trc);
@@ -332,10 +355,13 @@ struct IonScript
 
     static IonScript *New(JSContext *cx, types::RecompileInfo recompileInfo,
                           uint32_t frameLocals, uint32_t frameSize,
-                          size_t snapshotsSize, size_t snapshotEntries,
-                          size_t constants, size_t safepointIndexEntries, size_t osiIndexEntries,
-                          size_t cacheEntries, size_t runtimeSize, size_t safepointsSize,
-                          size_t callTargetEntries, size_t backedgeEntries);
+                          size_t snapshotsListSize, size_t snapshotsRVATableSize,
+                          size_t recoversSize, size_t bailoutEntries,
+                          size_t constants, size_t safepointIndexEntries,
+                          size_t osiIndexEntries, size_t cacheEntries,
+                          size_t runtimeSize, size_t safepointsSize,
+                          size_t callTargetEntries, size_t backedgeEntries,
+                          OptimizationLevel optimizationLevel);
     static void Trace(JSTracer *trc, IonScript *script);
     static void Destroy(FreeOp *fop, IonScript *script);
 
@@ -348,16 +374,22 @@ struct IonScript
     static inline size_t offsetOfSkipArgCheckEntryOffset() {
         return offsetof(IonScript, skipArgCheckEntryOffset_);
     }
+    static inline size_t offsetOfRefcount() {
+        return offsetof(IonScript, refcount_);
+    }
+    static inline size_t offsetOfRecompiling() {
+        return offsetof(IonScript, recompiling_);
+    }
 
   public:
-    IonCode *method() const {
+    JitCode *method() const {
         return method_;
     }
-    void setMethod(IonCode *code) {
+    void setMethod(JitCode *code) {
         JS_ASSERT(!invalidated());
         method_ = code;
     }
-    void setDeoptTable(IonCode *code) {
+    void setDeoptTable(JitCode *code) {
         deoptTable_ = code;
     }
     void setOsrPc(jsbytecode *osrPc) {
@@ -422,6 +454,15 @@ struct IonScript
     bool hasUncompiledCallTarget() const {
         return hasUncompiledCallTarget_;
     }
+    void setIsParallelEntryScript() {
+        isParallelEntryScript_ = true;
+    }
+    void clearIsParallelEntryScript() {
+        isParallelEntryScript_ = false;
+    }
+    bool isParallelEntryScript() const {
+        return isParallelEntryScript_;
+    }
     void setHasSPSInstrumentation() {
         hasSPSInstrumentation_ = true;
     }
@@ -434,8 +475,17 @@ struct IonScript
     const uint8_t *snapshots() const {
         return reinterpret_cast<const uint8_t *>(this) + snapshots_;
     }
-    size_t snapshotsSize() const {
-        return snapshotsSize_;
+    size_t snapshotsListSize() const {
+        return snapshotsListSize_;
+    }
+    size_t snapshotsRVATableSize() const {
+        return snapshotsRVATableSize_;
+    }
+    const uint8_t *recovers() const {
+        return reinterpret_cast<const uint8_t *>(this) + recovers_;
+    }
+    size_t recoversSize() const {
+        return recoversSize_;
     }
     const uint8_t *safepoints() const {
         return reinterpret_cast<const uint8_t *>(this) + safepointsStart_;
@@ -449,7 +499,7 @@ struct IonScript
     size_t sizeOfIncludingThis(mozilla::MallocSizeOf mallocSizeOf) const {
         return mallocSizeOf(this);
     }
-    EncapsulatedValue &getConstant(size_t index) {
+    PreBarrieredValue &getConstant(size_t index) {
         JS_ASSERT(index < numConstants());
         return constants()[index];
     }
@@ -493,10 +543,11 @@ struct IonScript
         return (CacheLocation *) &runtimeData()[locIndex];
     }
     void toggleBarriers(bool enabled);
-    void purgeCaches(JS::Zone *zone);
+    void purgeCaches();
     void destroyCaches();
-    void destroyBackedges(JSRuntime *rt);
+    void unlinkFromRuntime(FreeOp *fop);
     void copySnapshots(const SnapshotWriter *writer);
+    void copyRecovers(const RecoverWriter *writer);
     void copyBailoutTable(const SnapshotOffset *table);
     void copyConstants(const Value *vp);
     void copySafepointIndices(const SafepointIndex *firstSafepointIndex, MacroAssembler &masm);
@@ -505,8 +556,9 @@ struct IonScript
     void copyCacheEntries(const uint32_t *caches, MacroAssembler &masm);
     void copySafepoints(const SafepointWriter *writer);
     void copyCallTargetEntries(JSScript **callTargets);
-    void copyPatchableBackedges(JSContext *cx, IonCode *code,
-                                PatchableBackedgeInfo *backedges);
+    void copyPatchableBackedges(JSContext *cx, JitCode *code,
+                                PatchableBackedgeInfo *backedges,
+                                MacroAssembler &masm);
 
     bool invalidated() const {
         return refcount_ != 0;
@@ -526,6 +578,12 @@ struct IonScript
     const types::RecompileInfo& recompileInfo() const {
         return recompileInfo_;
     }
+    types::RecompileInfo& recompileInfoRef() {
+        return recompileInfo_;
+    }
+    OptimizationLevel optimizationLevel() const {
+        return optimizationLevel_;
+    }
     uint32_t incrOsrPcMismatchCounter() {
         return ++osrPcMismatchCounter_;
     }
@@ -545,6 +603,25 @@ struct IonScript
         recompiling_ = false;
     }
 
+    static const uint32_t MAX_PARALLEL_AGE = 5;
+
+    enum ShouldIncreaseAge {
+        IncreaseAge = true,
+        KeepAge = false
+    };
+
+    void resetParallelAge() {
+        MOZ_ASSERT(isParallelEntryScript());
+        parallelAge_ = 0;
+    }
+    uint32_t parallelAge() const {
+        return parallelAge_;
+    }
+    uint32_t shouldPreserveParallelCode(ShouldIncreaseAge increaseAge = KeepAge) {
+        MOZ_ASSERT(isParallelEntryScript());
+        return (increaseAge ? ++parallelAge_ : parallelAge_) < MAX_PARALLEL_AGE;
+    }
+
     static void writeBarrierPre(Zone *zone, IonScript *ionScript);
 };
 
@@ -569,11 +646,6 @@ struct IonBlockCounts
     // Text information about the code generated for this block.
     char *code_;
 
-    // Number of bytes of code generated in this block. Spill code is counted
-    // separately from other, instruction implementing code.
-    uint32_t instructionBytes_;
-    uint32_t spillBytes_;
-
   public:
 
     bool init(uint32_t id, uint32_t offset, uint32_t numSuccessors) {
@@ -634,22 +706,6 @@ struct IonBlockCounts
     const char *code() const {
         return code_;
     }
-
-    void setInstructionBytes(uint32_t bytes) {
-        instructionBytes_ = bytes;
-    }
-
-    uint32_t instructionBytes() const {
-        return instructionBytes_;
-    }
-
-    void setSpillBytes(uint32_t bytes) {
-        spillBytes_ = bytes;
-    }
-
-    uint32_t spillBytes() const {
-        return spillBytes_;
-    }
 };
 
 // Execution information for a compiled script which may persist after the
@@ -706,40 +762,25 @@ struct VMFunction;
 class JitCompartment;
 class JitRuntime;
 
-struct AutoFlushCache
+struct AutoFlushICache
 {
   private:
+#if defined(JS_CODEGEN_ARM) || defined(JS_CODEGEN_MIPS)
     uintptr_t start_;
     uintptr_t stop_;
     const char *name_;
-    JitRuntime *runtime_;
-    bool used_;
+    bool inhibit_;
+    AutoFlushICache *prev_;
+#endif
 
   public:
-    void update(uintptr_t p, size_t len);
-    static void updateTop(uintptr_t p, size_t len);
-    ~AutoFlushCache();
-    AutoFlushCache(const char *nonce, JitRuntime *rt);
-    void flushAnyway();
+    static void setRange(uintptr_t p, size_t len);
+    static void flush(uintptr_t p, size_t len);
+    static void setInhibit();
+    ~AutoFlushICache();
+    explicit AutoFlushICache(const char *nonce, bool inhibit=false);
 };
 
-// If you are currently in the middle of modifing Ion-compiled code, which
-// is going to be flushed at *some* point, but determine that you *must*
-// call a function *right* *now*, two things can go wrong:
-//   1)  The flusher that you were using is still active, but you are about to
-//       enter jitted code, so it needs to be flushed
-//   2) the called function can re-enter a compilation/modification path which
-//       will use your AFC, and thus not flush when his compilation is done
-
-struct AutoFlushInhibitor
-{
-  private:
-    JitRuntime *runtime_;
-    AutoFlushCache *afc;
-  public:
-    AutoFlushInhibitor(JitRuntime *rt);
-    ~AutoFlushInhibitor();
-};
 } // namespace jit
 
 namespace gc {
