@@ -10,12 +10,16 @@
 #include "mozilla/Array.h"
 
 #include "jit/IonTypes.h"
-#if defined(JS_CPU_X86)
+#if defined(JS_CODEGEN_X86)
 # include "jit/x86/Architecture-x86.h"
-#elif defined(JS_CPU_X64)
+#elif defined(JS_CODEGEN_X64)
 # include "jit/x64/Architecture-x64.h"
-#elif defined(JS_CPU_ARM)
+#elif defined(JS_CODEGEN_ARM)
 # include "jit/arm/Architecture-arm.h"
+#elif defined(JS_CODEGEN_MIPS)
+# include "jit/mips/Architecture-mips.h"
+#else
+# error "Unknown architecture!"
 #endif
 
 namespace js {
@@ -24,13 +28,18 @@ namespace jit {
 struct Register {
     typedef Registers Codes;
     typedef Codes::Code Code;
+    typedef Codes::SetType SetType;
     Code code_;
-
     static Register FromCode(uint32_t i) {
         JS_ASSERT(i < Registers::Total);
         Register r = { (Registers::Code)i };
         return r;
     }
+    static Register FromName(const char *name) {
+        Registers::Code code = Registers::FromName(name);
+        Register r = { code };
+        return r;
+    }
     Code code() const {
         JS_ASSERT((uint32_t)code_ < Registers::Total);
         return code_;
@@ -38,58 +47,52 @@ struct Register {
     const char *name() const {
         return Registers::GetName(code());
     }
-    bool operator ==(const Register &other) const {
+    bool operator ==(Register other) const {
         return code_ == other.code_;
     }
-    bool operator !=(const Register &other) const {
+    bool operator !=(Register other) const {
         return code_ != other.code_;
     }
     bool volatile_() const {
         return !!((1 << code()) & Registers::VolatileMask);
     }
-};
-
-struct FloatRegister {
-    typedef FloatRegisters Codes;
-    typedef Codes::Code Code;
-
-    Code code_;
-
-    static FloatRegister FromCode(uint32_t i) {
-        JS_ASSERT(i < FloatRegisters::Total);
-        FloatRegister r = { (FloatRegisters::Code)i };
-        return r;
-    }
-    Code code() const {
-        JS_ASSERT((uint32_t)code_ < FloatRegisters::Total);
-        return code_;
-    }
-    const char *name() const {
-        return FloatRegisters::GetName(code());
-    }
-    bool operator ==(const FloatRegister &other) const {
+    bool aliases(const Register &other) const {
         return code_ == other.code_;
     }
-    bool operator !=(const FloatRegister &other) const {
-        return code_ != other.code_;
+    uint32_t numAliased() const {
+        return 1;
     }
-    bool volatile_() const {
-        return !!((1 << code()) & FloatRegisters::VolatileMask);
+
+    // N.B. FloatRegister is an explicit outparam here because msvc-2010
+    // miscompiled it on win64 when the value was simply returned.  This
+    // now has an explicit outparam for compatability.
+    void aliased(uint32_t aliasIdx, Register *ret) const {
+        JS_ASSERT(aliasIdx == 0);
+        *ret = *this;
+    }
+    static uint32_t SetSize(SetType x) {
+        return Codes::SetSize(x);
+    }
+    static uint32_t FirstBit(SetType x) {
+        return Codes::FirstBit(x);
+    }
+    static uint32_t LastBit(SetType x) {
+        return Codes::LastBit(x);
     }
 };
 
 class RegisterDump
 {
   protected: // Silence Clang warning.
-    uintptr_t regs_[Registers::Total];
-    double fpregs_[FloatRegisters::Total];
+    mozilla::Array<uintptr_t, Registers::Total> regs_;
+    mozilla::Array<double, FloatRegisters::TotalPhys> fpregs_;
 
   public:
     static size_t offsetOfRegister(Register reg) {
         return offsetof(RegisterDump, regs_) + reg.code() * sizeof(uintptr_t);
     }
     static size_t offsetOfRegister(FloatRegister reg) {
-        return offsetof(RegisterDump, fpregs_) + reg.code() * sizeof(double);
+        return offsetof(RegisterDump, fpregs_) + reg.getRegisterDumpOffsetInBytes();
     }
 };
 
@@ -100,8 +103,8 @@ class MachineState
     mozilla::Array<double *, FloatRegisters::Total> fpregs_;
 
   public:
-    static MachineState FromBailout(uintptr_t regs[Registers::Total],
-                                    double fpregs[FloatRegisters::Total]);
+    static MachineState FromBailout(mozilla::Array<uintptr_t, Registers::Total> &regs,
+                                    mozilla::Array<double, FloatRegisters::TotalPhys> &fpregs);
 
     void setRegisterLocation(Register reg, uintptr_t *up) {
         regs_[reg.code()] = up;
