@@ -6,35 +6,34 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "jsapi-tests/tests.h"
-#include "vm/String.h"
 
 BEGIN_TEST(testIntString_bug515273)
 {
     JS::RootedValue v(cx);
 
-    EVAL("'1';", v.address());
-    JSString *str = JSVAL_TO_STRING(v);
+    EVAL("'1';", &v);
+    JSString *str = v.toString();
     CHECK(JS_StringHasBeenInterned(cx, str));
     CHECK(JS_FlatStringEqualsAscii(JS_ASSERT_STRING_IS_FLAT(str), "1"));
 
-    EVAL("'42';", v.address());
-    str = JSVAL_TO_STRING(v);
+    EVAL("'42';", &v);
+    str = v.toString();
     CHECK(JS_StringHasBeenInterned(cx, str));
     CHECK(JS_FlatStringEqualsAscii(JS_ASSERT_STRING_IS_FLAT(str), "42"));
 
-    EVAL("'111';", v.address());
-    str = JSVAL_TO_STRING(v);
+    EVAL("'111';", &v);
+    str = v.toString();
     CHECK(JS_StringHasBeenInterned(cx, str));
     CHECK(JS_FlatStringEqualsAscii(JS_ASSERT_STRING_IS_FLAT(str), "111"));
 
     /* Test other types of static strings. */
-    EVAL("'a';", v.address());
-    str = JSVAL_TO_STRING(v);
+    EVAL("'a';", &v);
+    str = v.toString();
     CHECK(JS_StringHasBeenInterned(cx, str));
     CHECK(JS_FlatStringEqualsAscii(JS_ASSERT_STRING_IS_FLAT(str), "a"));
 
-    EVAL("'bc';", v.address());
-    str = JSVAL_TO_STRING(v);
+    EVAL("'bc';", &v);
+    str = v.toString();
     CHECK(JS_StringHasBeenInterned(cx, str));
     CHECK(JS_FlatStringEqualsAscii(JS_ASSERT_STRING_IS_FLAT(str), "bc"));
 
