@@ -7,6 +7,8 @@
 #ifndef jsapi_tests_tests_h
 #define jsapi_tests_tests_h
 
+#include "mozilla/TypeTraits.h"
+
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -23,7 +25,7 @@ class JSAPITestString {
     js::Vector<char, 0, js::SystemAllocPolicy> chars;
   public:
     JSAPITestString() {}
-    JSAPITestString(const char *s) { *this += s; }
+    explicit JSAPITestString(const char *s) { *this += s; }
     JSAPITestString(const JSAPITestString &s) { *this += s; }
 
     const char *begin() const { return chars.begin(); }
@@ -54,7 +56,7 @@ class JSAPITest
 
     JSRuntime *rt;
     JSContext *cx;
-    JSObject *global;
+    JS::Heap<JSObject *> global;
     bool knownFail;
     JSAPITestString msgs;
     JSCompartment *oldCompartment;
@@ -74,8 +76,9 @@ class JSAPITest
             JS_LeaveCompartment(cx, oldCompartment);
             oldCompartment = nullptr;
         }
+        global = nullptr;
         if (cx) {
-            JS_RemoveObjectRoot(cx, &global);
+            JS::RemoveObjectRoot(cx, &global);
             JS_LeaveCompartment(cx, nullptr);
             JS_EndRequest(cx);
             JS_DestroyContext(cx);
@@ -96,9 +99,9 @@ class JSAPITest
 
 #define EVAL(s, vp) do { if (!evaluate(s, __FILE__, __LINE__, vp)) return false; } while (false)
 
-    bool evaluate(const char *bytes, const char *filename, int lineno, jsval *vp);
+    bool evaluate(const char *bytes, const char *filename, int lineno, JS::MutableHandleValue vp);
 
-    JSAPITestString jsvalToSource(jsval v) {
+    JSAPITestString jsvalToSource(JS::HandleValue v) {
         JSString *str = JS_ValueToSource(cx, v);
         if (str) {
             JSAutoByteString bytes(cx, str);
@@ -146,41 +149,53 @@ class JSAPITest
     }
 
     JSAPITestString toSource(JSAtom *v) {
-        return jsvalToSource(STRING_TO_JSVAL((JSString*)v));
+        JS::RootedValue val(cx, JS::StringValue((JSString *)v));
+        return jsvalToSource(val);
     }
 
     JSAPITestString toSource(JSVersion v) {
         return JSAPITestString(JS_VersionToString(v));
     }
 
-    template<typename T>
-    bool checkEqual(const T &actual, const T &expected,
+    // Note that in some still-supported GCC versions (we think anything before
+    // GCC 4.6), this template does not work when the second argument is
+    // nullptr. It infers type U = long int. Use CHECK_NULL instead.
+    template <typename T, typename U>
+    bool checkEqual(const T &actual, const U &expected,
                     const char *actualExpr, const char *expectedExpr,
-                    const char *filename, int lineno) {
+                    const char *filename, int lineno)
+    {
+        static_assert(mozilla::IsSigned<T>::value == mozilla::IsSigned<U>::value,
+                      "using CHECK_EQUAL with different-signed inputs triggers compiler warnings");
+        static_assert(mozilla::IsUnsigned<T>::value == mozilla::IsUnsigned<U>::value,
+                      "using CHECK_EQUAL with different-signed inputs triggers compiler warnings");
         return (actual == expected) ||
             fail(JSAPITestString("CHECK_EQUAL failed: expected (") +
                  expectedExpr + ") = " + toSource(expected) +
                  ", got (" + actualExpr + ") = " + toSource(actual), filename, lineno);
     }
 
-    // There are many cases where the static types of 'actual' and 'expected'
-    // are not identical, and C++ is understandably cautious about automatic
-    // coercions. So catch those cases and forcibly coerce, then use the
-    // identical-type specialization. This may do bad things if the types are
-    // actually *not* compatible.
-    template<typename T, typename U>
-    bool checkEqual(const T &actual, const U &expected,
-                   const char *actualExpr, const char *expectedExpr,
-                   const char *filename, int lineno) {
-        return checkEqual(U(actual), expected, actualExpr, expectedExpr, filename, lineno);
-    }
-
 #define CHECK_EQUAL(actual, expected) \
     do { \
         if (!checkEqual(actual, expected, #actual, #expected, __FILE__, __LINE__)) \
             return false; \
     } while (false)
 
+    template <typename T>
+    bool checkNull(const T *actual, const char *actualExpr,
+                   const char *filename, int lineno) {
+        return (actual == nullptr) ||
+            fail(JSAPITestString("CHECK_NULL failed: expected nullptr, got (") +
+                 actualExpr + ") = " + toSource(actual),
+                 filename, lineno);
+    }
+
+#define CHECK_NULL(actual) \
+    do { \
+        if (!checkNull(actual, #actual, __FILE__, __LINE__)) \
+            return false; \
+    } while (false)
+
     bool checkSame(jsval actualArg, jsval expectedArg,
                    const char *actualExpr, const char *expectedExpr,
                    const char *filename, int lineno) {
@@ -201,7 +216,7 @@ class JSAPITest
 #define CHECK(expr) \
     do { \
         if (!(expr)) \
-            return fail("CHECK failed: " #expr, __FILE__, __LINE__); \
+            return fail(JSAPITestString("CHECK failed: " #expr), __FILE__, __LINE__); \
     } while (false)
 
     bool fail(JSAPITestString msg = JSAPITestString(), const char *filename = "-", int lineno = 0) {
@@ -228,7 +243,9 @@ class JSAPITest
         static const JSClass c = {
             "global", JSCLASS_GLOBAL_FLAGS,
             JS_PropertyStub, JS_DeletePropertyStub, JS_PropertyStub, JS_StrictPropertyStub,
-            JS_EnumerateStub, JS_ResolveStub, JS_ConvertStub
+            JS_EnumerateStub, JS_ResolveStub, JS_ConvertStub, nullptr,
+            nullptr, nullptr, nullptr,
+            JS_GlobalObjectTraceHook
         };
         return &c;
     }
@@ -277,10 +294,11 @@ class JSAPITest
     }
 
     virtual JSRuntime * createRuntime() {
-        JSRuntime *rt = JS_NewRuntime(8L * 1024 * 1024, JS_USE_HELPER_THREADS);
+        JSRuntime *rt = JS_NewRuntime(8L * 1024 * 1024);
         if (!rt)
             return nullptr;
         setNativeStackQuota(rt);
+        JS::RuntimeOptionsRef(rt).setVarObjFix(true);
         return rt;
     }
 
@@ -301,7 +319,6 @@ class JSAPITest
         JSContext *cx = JS_NewContext(rt, 8192);
         if (!cx)
             return nullptr;
-        JS::ContextOptionsRef(cx).setVarObjFix(true);
         JS_SetErrorReporter(cx, &reportError);
         return cx;
     }
@@ -404,7 +421,7 @@ class TempFile {
 class TestJSPrincipals : public JSPrincipals
 {
   public:
-    TestJSPrincipals(int rc = 0)
+    explicit TestJSPrincipals(int rc = 0)
       : JSPrincipals()
     {
         refcount = rc;
