@@ -16,6 +16,12 @@
 
 namespace js {
 class ErrorObject;
+
+JSErrorReport *
+CopyErrorReport(JSContext *cx, JSErrorReport *report);
+
+JSString *
+ComputeStackString(JSContext *cx);
 }
 
 /*
@@ -23,8 +29,26 @@ class ErrorObject;
  * the error number.  If there is, then create an appropriate exception object,
  * set it as the pending exception, and set the JSREPORT_EXCEPTION flag on the
  * error report.  Exception-aware host error reporters should probably ignore
- * error reports so flagged.  Returns true if an associated exception is
- * found and set, false otherwise.
+ * error reports so flagged.
+ *
+ * Return true if cx->throwing and cx->exception were set.
+ *
+ * This means that:
+ *
+ *   - If the error is successfully converted to an exception and stored in
+ *     cx->exception, the return value is true. This is the "normal", happiest
+ *     case for the caller.
+ *
+ *   - If we try to convert, but fail with OOM or some other error that ends up
+ *     setting cx->throwing to true and setting cx->exception, then we also
+ *     return true (because callers want to treat that case the same way).
+ *     The original error described by *reportp typically won't be reported
+ *     anywhere; instead OOM is reported.
+ *
+ *   - If *reportp is just a warning, or the error code is unrecognized, or if
+ *     we decided to do nothing in order to avoid recursion, then return
+ *     false. In those cases, this error is just being swept under the rug
+ *     unless the caller decides to call CallErrorReporter explicitly.
  */
 extern bool
 js_ErrorToException(JSContext *cx, const char *message, JSErrorReport *reportp,
@@ -50,22 +74,17 @@ extern bool
 js_ReportUncaughtException(JSContext *cx);
 
 extern JSErrorReport *
-js_ErrorFromException(jsval exn);
-
-extern const JSErrorFormatString *
-js_GetLocalizedErrorMessage(js::ExclusiveContext *cx, void *userRef, const char *locale,
-                            const unsigned errorNumber);
+js_ErrorFromException(JSContext *cx, js::HandleObject obj);
 
 /*
- * Make a copy of errobj parented to scope.
+ * Make a copy of errobj parented to cx's compartment's global.
  *
- * cx must be in the same compartment as scope. errobj may be in a different
- * compartment, but it must be an Error object (not a wrapper of one) and it
- * must not be one of the prototype objects created by js_InitExceptionClasses
- * (errobj->getPrivate() must not be nullptr).
+ * errobj may be in a different compartment than cx, but it must be an Error
+ * object (not a wrapper of one) and it must not be one of the standard error
+ * prototype objects (errobj->getPrivate() must not be nullptr).
  */
 extern JSObject *
-js_CopyErrorObject(JSContext *cx, JS::Handle<js::ErrorObject*> errobj, js::HandleObject scope);
+js_CopyErrorObject(JSContext *cx, JS::Handle<js::ErrorObject*> errobj);
 
 static inline JSProtoKey
 GetExceptionProtoKey(JSExnType exn)
@@ -75,4 +94,13 @@ GetExceptionProtoKey(JSExnType exn)
     return JSProtoKey(JSProto_Error + int(exn));
 }
 
+static inline JSExnType
+ExnTypeFromProtoKey(JSProtoKey key)
+{
+    JSExnType type = static_cast<JSExnType>(key - JSProto_Error);
+    JS_ASSERT(type >= JSEXN_ERR);
+    JS_ASSERT(type < JSEXN_LIMIT);
+    return type;
+}
+
 #endif /* jsexn_h */
