@@ -35,9 +35,10 @@
 using namespace js;
 
 using mozilla::Abs;
-using mozilla::DoubleEqualsInt32;
-using mozilla::DoubleIsInt32;
+using mozilla::NumberEqualsInt32;
+using mozilla::NumberIsInt32;
 using mozilla::ExponentComponent;
+using mozilla::FloatingPoint;
 using mozilla::IsFinite;
 using mozilla::IsInfinite;
 using mozilla::IsNaN;
@@ -45,32 +46,9 @@ using mozilla::IsNegative;
 using mozilla::IsNegativeZero;
 using mozilla::PositiveInfinity;
 using mozilla::NegativeInfinity;
-using mozilla::SpecificNaN;
 using JS::ToNumber;
 using JS::GenericNaN;
 
-#ifndef M_E
-#define M_E             2.7182818284590452354
-#endif
-#ifndef M_LOG2E
-#define M_LOG2E         1.4426950408889634074
-#endif
-#ifndef M_LOG10E
-#define M_LOG10E        0.43429448190325182765
-#endif
-#ifndef M_LN2
-#define M_LN2           0.69314718055994530942
-#endif
-#ifndef M_LN10
-#define M_LN10          2.30258509299404568402
-#endif
-#ifndef M_SQRT2
-#define M_SQRT2         1.41421356237309504880
-#endif
-#ifndef M_SQRT1_2
-#define M_SQRT1_2       0.70710678118654752440
-#endif
-
 static const JSConstDoubleSpec math_constants[] = {
     {M_E,       "E",            0, {0,0,0}},
     {M_LOG2E,   "LOG2E",        0, {0,0,0}},
@@ -89,7 +67,7 @@ MathCache::MathCache() {
     /* See comments in lookup(). */
     JS_ASSERT(IsNegativeZero(-0.0));
     JS_ASSERT(!IsNegativeZero(+0.0));
-    JS_ASSERT(hash(-0.0) != hash(+0.0));
+    JS_ASSERT(hash(-0.0, MathCache::Sin) != hash(+0.0, MathCache::Sin));
 }
 
 size_t
@@ -110,6 +88,19 @@ const Class js::MathClass = {
     JS_ConvertStub
 };
 
+bool
+js_math_abs_handle(JSContext *cx, js::HandleValue v, js::MutableHandleValue r)
+{
+    double x;
+    if (!ToNumber(cx, v, &x))
+        return false;
+
+    double z = Abs(x);
+    r.setNumber(z);
+
+    return true;
+}
+
 bool
 js_math_abs(JSContext *cx, unsigned argc, Value *vp)
 {
@@ -120,13 +111,7 @@ js_math_abs(JSContext *cx, unsigned argc, Value *vp)
         return true;
     }
 
-    double x;
-    if (!ToNumber(cx, args[0], &x))
-        return false;
-
-    double z = Abs(x);
-    args.rval().setNumber(z);
-    return true;
+    return js_math_abs_handle(cx, args[0], args.rval());
 }
 
 #if defined(SOLARIS) && defined(__GNUC__)
@@ -139,7 +124,7 @@ double
 js::math_acos_impl(MathCache *cache, double x)
 {
     ACOS_IF_OUT_OF_RANGE(x);
-    return cache->lookup(acos, x);
+    return cache->lookup(acos, x, MathCache::Acos);
 }
 
 double
@@ -184,7 +169,7 @@ double
 js::math_asin_impl(MathCache *cache, double x)
 {
     ASIN_IF_OUT_OF_RANGE(x);
-    return cache->lookup(asin, x);
+    return cache->lookup(asin, x, MathCache::Asin);
 }
 
 double
@@ -222,7 +207,7 @@ js::math_asin(JSContext *cx, unsigned argc, Value *vp)
 double
 js::math_atan_impl(MathCache *cache, double x)
 {
-    return cache->lookup(atan, x);
+    return cache->lookup(atan, x, MathCache::Atan);
 }
 
 double
@@ -284,22 +269,28 @@ js::ecmaAtan2(double y, double x)
     return atan2(y, x);
 }
 
+bool
+js::math_atan2_handle(JSContext *cx, HandleValue y, HandleValue x, MutableHandleValue res)
+{
+    double dy;
+    if (!ToNumber(cx, y, &dy))
+        return false;
+
+    double dx;
+    if (!ToNumber(cx, x, &dx))
+        return false;
+
+    double z = ecmaAtan2(dy, dx);
+    res.setDouble(z);
+    return true;
+}
+
 bool
 js::math_atan2(JSContext *cx, unsigned argc, Value *vp)
 {
     CallArgs args = CallArgsFromVp(argc, vp);
 
-    double y;
-    if (!ToNumber(cx, args.get(0), &y))
-        return false;
-
-    double x;
-    if (!ToNumber(cx, args.get(1), &x))
-        return false;
-
-    double z = ecmaAtan2(y, x);
-    args.rval().setDouble(z);
-    return true;
+    return math_atan2_handle(cx, args.get(0), args.get(1), args.rval());
 }
 
 double
@@ -331,10 +322,33 @@ js::math_ceil(JSContext *cx, unsigned argc, Value *vp)
     return true;
 }
 
+bool
+js::math_clz32(JSContext *cx, unsigned argc, Value *vp)
+{
+    CallArgs args = CallArgsFromVp(argc, vp);
+
+    if (args.length() == 0) {
+        args.rval().setInt32(32);
+        return true;
+    }
+
+    uint32_t n;
+    if (!ToUint32(cx, args[0], &n))
+        return false;
+
+    if (n == 0) {
+        args.rval().setInt32(32);
+        return true;
+    }
+
+    args.rval().setInt32(mozilla::CountLeadingZeroes32(n));
+    return true;
+}
+
 double
 js::math_cos_impl(MathCache *cache, double x)
 {
-    return cache->lookup(cos, x);
+    return cache->lookup(cos, x, MathCache::Cos);
 }
 
 double
@@ -369,9 +383,9 @@ js::math_cos(JSContext *cx, unsigned argc, Value *vp)
 #ifdef _WIN32
 #define EXP_IF_OUT_OF_RANGE(x)                  \
     if (!IsNaN(x)) {                            \
-        if (x == PositiveInfinity())            \
-            return PositiveInfinity();          \
-        if (x == NegativeInfinity())            \
+        if (x == PositiveInfinity<double>())    \
+            return PositiveInfinity<double>();  \
+        if (x == NegativeInfinity<double>())    \
             return 0.0;                         \
     }
 #else
@@ -382,7 +396,7 @@ double
 js::math_exp_impl(MathCache *cache, double x)
 {
     EXP_IF_OUT_OF_RANGE(x);
-    return cache->lookup(exp, x);
+    return cache->lookup(exp, x, MathCache::Exp);
 }
 
 double
@@ -423,6 +437,19 @@ js::math_floor_impl(double x)
     return floor(x);
 }
 
+bool
+js::math_floor_handle(JSContext *cx, HandleValue v, MutableHandleValue r)
+{
+    double d;
+    if(!ToNumber(cx, v, &d))
+        return false;
+
+    double z = math_floor_impl(d);
+    r.setNumber(z);
+
+    return true;
+}
+
 bool
 js::math_floor(JSContext *cx, unsigned argc, Value *vp)
 {
@@ -433,13 +460,7 @@ js::math_floor(JSContext *cx, unsigned argc, Value *vp)
         return true;
     }
 
-    double x;
-    if (!ToNumber(cx, args[0], &x))
-        return false;
-
-    double z = math_floor_impl(x);
-    args.rval().setNumber(z);
-    return true;
+    return math_floor_handle(cx, args[0], args.rval());
 }
 
 bool
@@ -460,6 +481,27 @@ js::math_imul(JSContext *cx, unsigned argc, Value *vp)
     return true;
 }
 
+// Implements Math.fround (20.2.2.16) up to step 3
+bool
+js::RoundFloat32(JSContext *cx, HandleValue v, float *out)
+{
+    double d;
+    bool success = ToNumber(cx, v, &d);
+    *out = static_cast<float>(d);
+    return success;
+}
+
+bool
+js::RoundFloat32(JSContext *cx, HandleValue arg, MutableHandleValue res)
+{
+    float f;
+    if (!RoundFloat32(cx, arg, &f))
+        return false;
+
+    res.setDouble(static_cast<double>(f));
+    return true;
+}
+
 bool
 js::math_fround(JSContext *cx, unsigned argc, Value *vp)
 {
@@ -470,11 +512,10 @@ js::math_fround(JSContext *cx, unsigned argc, Value *vp)
         return true;
     }
 
-    double x;
-    if (!ToNumber(cx, args[0], &x))
+    float f;
+    if (!RoundFloat32(cx, args[0], &f))
         return false;
 
-    float f = x;
     args.rval().setDouble(static_cast<double>(f));
     return true;
 }
@@ -489,7 +530,7 @@ double
 js::math_log_impl(MathCache *cache, double x)
 {
     LOG_IF_OUT_OF_RANGE(x);
-    return cache->lookup(log, x);
+    return cache->lookup(log, x, MathCache::Log);
 }
 
 double
@@ -524,42 +565,74 @@ js::math_log(JSContext *cx, unsigned argc, Value *vp)
     return true;
 }
 
+static double
+max_double(double x, double y)
+{
+    // Math.max(num, NaN) => NaN, Math.max(-0, +0) => +0
+    if (x > y || IsNaN(x) || (x == y && IsNegative(y)))
+        return x;
+    return y;
+}
+
 bool
 js_math_max(JSContext *cx, unsigned argc, Value *vp)
 {
     CallArgs args = CallArgsFromVp(argc, vp);
 
-    double maxval = NegativeInfinity();
+    double maxval = NegativeInfinity<double>();
     for (unsigned i = 0; i < args.length(); i++) {
         double x;
         if (!ToNumber(cx, args[i], &x))
             return false;
-        // Math.max(num, NaN) => NaN, Math.max(-0, +0) => +0
-        if (x > maxval || IsNaN(x) || (x == maxval && IsNegative(maxval)))
-            maxval = x;
+        maxval = max_double(x, maxval);
     }
     args.rval().setNumber(maxval);
     return true;
 }
 
+static double
+min_double(double x, double y)
+{
+    // Math.min(num, NaN) => NaN, Math.min(-0, +0) => -0
+    if (x < y || IsNaN(x) || (x == y && IsNegativeZero(x)))
+        return x;
+    return y;
+}
+
 bool
 js_math_min(JSContext *cx, unsigned argc, Value *vp)
 {
     CallArgs args = CallArgsFromVp(argc, vp);
 
-    double minval = PositiveInfinity();
+    double minval = PositiveInfinity<double>();
     for (unsigned i = 0; i < args.length(); i++) {
         double x;
         if (!ToNumber(cx, args[i], &x))
             return false;
-        // Math.min(num, NaN) => NaN, Math.min(-0, +0) => -0
-        if (x < minval || IsNaN(x) || (x == minval && IsNegativeZero(x)))
-            minval = x;
+        minval = min_double(x, minval);
     }
     args.rval().setNumber(minval);
     return true;
 }
 
+bool
+js_minmax_impl(JSContext *cx, bool max, HandleValue a, HandleValue b, MutableHandleValue res)
+{
+    double x, y;
+
+    if (!ToNumber(cx, a, &x))
+        return false;
+    if (!ToNumber(cx, b, &y))
+        return false;
+
+    if (max)
+        res.setNumber(max_double(x, y));
+    else
+        res.setNumber(min_double(x, y));
+
+    return true;
+}
+
 // Disable PGO for Math.pow() and related functions (see bug 791214).
 #if defined(_MSC_VER)
 # pragma optimize("g", off)
@@ -607,7 +680,7 @@ js::ecmaPow(double x, double y)
      * check for NaN since a comparison with NaN is always false.
      */
     int32_t yi;
-    if (DoubleEqualsInt32(y, &yi))
+    if (NumberEqualsInt32(y, &yi))
         return powi(x, yi);
 
     /*
@@ -616,9 +689,21 @@ js::ecmaPow(double x, double y)
      */
     if (!IsFinite(y) && (x == 1.0 || x == -1.0))
         return GenericNaN();
+
     /* pow(x, +-0) is always 1, even for x = NaN (MSVC gets this wrong). */
     if (y == 0)
         return 1;
+
+    /*
+     * Special case for square roots. Note that pow(x, 0.5) != sqrt(x)
+     * when x = -0.0, so we have to guard for this.
+     */
+    if (IsFinite(x) && x != 0.0) {
+        if (y == 0.5)
+            return sqrt(x);
+        if (y == -0.5)
+            return 1.0 / sqrt(x);
+    }
     return pow(x, y);
 }
 #if defined(_MSC_VER)
@@ -629,44 +714,28 @@ js::ecmaPow(double x, double y)
 #if defined(_MSC_VER)
 # pragma optimize("g", off)
 #endif
+bool
+js_math_pow_handle(JSContext *cx, HandleValue base, HandleValue power, MutableHandleValue result)
+{
+    double x;
+    if (!ToNumber(cx, base, &x))
+        return false;
+
+    double y;
+    if (!ToNumber(cx, power, &y))
+        return false;
+
+    double z = ecmaPow(x, y);
+    result.setNumber(z);
+    return true;
+}
+
 bool
 js_math_pow(JSContext *cx, unsigned argc, Value *vp)
 {
     CallArgs args = CallArgsFromVp(argc, vp);
 
-    double x;
-    if (!ToNumber(cx, args.get(0), &x))
-        return false;
-
-    double y;
-    if (!ToNumber(cx, args.get(1), &y))
-        return false;
-
-    /*
-     * Special case for square roots. Note that pow(x, 0.5) != sqrt(x)
-     * when x = -0.0, so we have to guard for this.
-     */
-    if (IsFinite(x) && x != 0.0) {
-        if (y == 0.5) {
-            args.rval().setNumber(sqrt(x));
-            return true;
-        }
-        if (y == -0.5) {
-            args.rval().setNumber(1.0/sqrt(x));
-            return true;
-        }
-    }
-
-    /* pow(x, +-0) is always 1, even for x = NaN. */
-    if (y == 0) {
-        args.rval().setInt32(1);
-        return true;
-    }
-
-    double z = ecmaPow(x, y);
-
-    args.rval().setNumber(z);
-    return true;
+    return js_math_pow_handle(cx, args.get(0), args.get(1), args.rval());
 }
 #if defined(_MSC_VER)
 # pragma optimize("", on)
@@ -765,20 +834,46 @@ js_math_random(JSContext *cx, unsigned argc, Value *vp)
     return true;
 }
 
+bool
+js::math_round_handle(JSContext *cx, HandleValue arg, MutableHandleValue res)
+{
+    double d;
+    if (!ToNumber(cx, arg, &d))
+        return false;
+
+    d = math_round_impl(d);
+    res.setNumber(d);
+    return true;
+}
+
 double
 js::math_round_impl(double x)
 {
-    int32_t i;
-    if (DoubleIsInt32(x, &i))
-        return double(i);
+    int32_t ignored;
+    if (NumberIsInt32(x, &ignored))
+        return x;
 
     /* Some numbers are so big that adding 0.5 would give the wrong number. */
-    if (ExponentComponent(x) >= 52)
+    if (ExponentComponent(x) >= int_fast16_t(FloatingPoint<double>::kExponentShift))
         return x;
 
     return js_copysign(floor(x + 0.5), x);
 }
 
+float
+js::math_roundf_impl(float x)
+{
+    int32_t ignored;
+    if (NumberIsInt32(x, &ignored))
+        return x;
+
+    /* Some numbers are so big that adding 0.5 would give the wrong number. */
+    if (ExponentComponent(x) >= int_fast16_t(FloatingPoint<float>::kExponentShift))
+        return x;
+
+    return js_copysign(floorf(x + 0.5f), x);
+}
+
 bool /* ES5 15.8.2.15. */
 js::math_round(JSContext *cx, unsigned argc, Value *vp)
 {
@@ -789,19 +884,13 @@ js::math_round(JSContext *cx, unsigned argc, Value *vp)
         return true;
     }
 
-    double x;
-    if (!ToNumber(cx, args[0], &x))
-        return false;
-
-    double z = math_round_impl(x);
-    args.rval().setNumber(z);
-    return true;
+    return js::math_round_handle(cx, args[0], args.rval());
 }
 
 double
 js::math_sin_impl(MathCache *cache, double x)
 {
-    return cache->lookup(sin, x);
+    return cache->lookup(sin, x, MathCache::Sin);
 }
 
 double
@@ -833,6 +922,22 @@ js::math_sin(JSContext *cx, unsigned argc, Value *vp)
     return true;
 }
 
+bool
+js::math_sqrt_handle(JSContext *cx, HandleValue number, MutableHandleValue result)
+{
+    double x;
+    if (!ToNumber(cx, number, &x))
+        return false;
+
+    MathCache *mathCache = cx->runtime()->getMathCache(cx);
+    if (!mathCache)
+        return false;
+
+    double z = mathCache->lookup(sqrt, x, MathCache::Sqrt);
+    result.setDouble(z);
+    return true;
+}
+
 bool
 js_math_sqrt(JSContext *cx, unsigned argc, Value *vp)
 {
@@ -843,23 +948,13 @@ js_math_sqrt(JSContext *cx, unsigned argc, Value *vp)
         return true;
     }
 
-    double x;
-    if (!ToNumber(cx, args[0], &x))
-        return false;
-
-    MathCache *mathCache = cx->runtime()->getMathCache(cx);
-    if (!mathCache)
-        return false;
-
-    double z = mathCache->lookup(sqrt, x);
-    args.rval().setDouble(z);
-    return true;
+    return math_sqrt_handle(cx, args[0], args.rval());
 }
 
 double
 js::math_tan_impl(MathCache *cache, double x)
 {
-    return cache->lookup(tan, x);
+    return cache->lookup(tan, x, MathCache::Tan);
 }
 
 double
@@ -921,7 +1016,7 @@ static bool math_function(JSContext *cx, unsigned argc, Value *vp)
 double
 js::math_log10_impl(MathCache *cache, double x)
 {
-    return cache->lookup(log10, x);
+    return cache->lookup(log10, x, MathCache::Log10);
 }
 
 double
@@ -946,7 +1041,7 @@ double log2(double x)
 double
 js::math_log2_impl(MathCache *cache, double x)
 {
-    return cache->lookup(log2, x);
+    return cache->lookup(log2, x, MathCache::Log2);
 }
 
 double
@@ -989,7 +1084,7 @@ double
 js::math_log1p_impl(MathCache *cache, double x)
 {
     LOG1P_IF_OUT_OF_RANGE(x);
-    return cache->lookup(log1p, x);
+    return cache->lookup(log1p, x, MathCache::Log1p);
 }
 
 double
@@ -1031,7 +1126,7 @@ double expm1(double x)
 double
 js::math_expm1_impl(MathCache *cache, double x)
 {
-    return cache->lookup(expm1, x);
+    return cache->lookup(expm1, x, MathCache::Expm1);
 }
 
 double
@@ -1061,7 +1156,7 @@ double sqrt1pm1(double x)
 double
 js::math_cosh_impl(MathCache *cache, double x)
 {
-    return cache->lookup(cosh, x);
+    return cache->lookup(cosh, x, MathCache::Cosh);
 }
 
 double
@@ -1079,7 +1174,7 @@ js::math_cosh(JSContext *cx, unsigned argc, Value *vp)
 double
 js::math_sinh_impl(MathCache *cache, double x)
 {
-    return cache->lookup(sinh, x);
+    return cache->lookup(sinh, x, MathCache::Sinh);
 }
 
 double
@@ -1097,7 +1192,7 @@ js::math_sinh(JSContext *cx, unsigned argc, Value *vp)
 double
 js::math_tanh_impl(MathCache *cache, double x)
 {
-    return cache->lookup(tanh, x);
+    return cache->lookup(tanh, x, MathCache::Tanh);
 }
 
 double
@@ -1146,7 +1241,7 @@ double acosh(double x)
 double
 js::math_acosh_impl(MathCache *cache, double x)
 {
-    return cache->lookup(acosh, x);
+    return cache->lookup(acosh, x, MathCache::Acosh);
 }
 
 double
@@ -1200,9 +1295,9 @@ double
 js::math_asinh_impl(MathCache *cache, double x)
 {
 #ifdef HAVE_ASINH
-    return cache->lookup(asinh, x);
+    return cache->lookup(asinh, x, MathCache::Asinh);
 #else
-    return cache->lookup(my_asinh, x);
+    return cache->lookup(my_asinh, x, MathCache::Asinh);
 #endif
 }
 
@@ -1253,7 +1348,7 @@ double atanh(double x)
 double
 js::math_atanh_impl(MathCache *cache, double x)
 {
-    return cache->lookup(atanh, x);
+    return cache->lookup(atanh, x, MathCache::Atanh);
 }
 
 double
@@ -1278,7 +1373,7 @@ js::ecmaHypot(double x, double y)
      * is NaN, not Infinity.
      */
     if (mozilla::IsInfinite(x) || mozilla::IsInfinite(y)) {
-        return mozilla::PositiveInfinity();
+        return mozilla::PositiveInfinity<double>();
     }
 #endif
     return hypot(x, y);
@@ -1327,7 +1422,7 @@ js::math_hypot(JSContext *cx, unsigned argc, Value *vp)
         }
     }
 
-    double result = isInfinite ? PositiveInfinity() :
+    double result = isInfinite ? PositiveInfinity<double>() :
                     isNaN ? GenericNaN() :
                     scale * sqrt(sumsq);
     args.rval().setNumber(result);
@@ -1344,7 +1439,7 @@ double trunc(double x)
 double
 js::math_trunc_impl(MathCache *cache, double x)
 {
-    return cache->lookup(trunc, x);
+    return cache->lookup(trunc, x, MathCache::Trunc);
 }
 
 double
@@ -1370,7 +1465,7 @@ static double sign(double x)
 double
 js::math_sign_impl(MathCache *cache, double x)
 {
-    return cache->lookup(sign, x);
+    return cache->lookup(sign, x, MathCache::Sign);
 }
 
 double
@@ -1401,7 +1496,7 @@ double cbrt(double x)
 double
 js::math_cbrt_impl(MathCache *cache, double x)
 {
-    return cache->lookup(cbrt, x);
+    return cache->lookup(cbrt, x, MathCache::Cbrt);
 }
 
 double
@@ -1436,6 +1531,7 @@ static const JSFunctionSpec math_static_methods[] = {
     JS_FN("atan",           math_atan,            1, 0),
     JS_FN("atan2",          math_atan2,           2, 0),
     JS_FN("ceil",           math_ceil,            1, 0),
+    JS_FN("clz32",          math_clz32,           1, 0),
     JS_FN("cos",            math_cos,             1, 0),
     JS_FN("exp",            math_exp,             1, 0),
     JS_FN("floor",          math_floor,           1, 0),
@@ -1477,8 +1573,9 @@ js_InitMathClass(JSContext *cx, HandleObject obj)
     if (!Math)
         return nullptr;
 
-    if (!JS_DefineProperty(cx, obj, js_Math_str, OBJECT_TO_JSVAL(Math),
-                           JS_PropertyStub, JS_StrictPropertyStub, 0)) {
+    if (!JS_DefineProperty(cx, obj, js_Math_str, Math, 0,
+                           JS_PropertyStub, JS_StrictPropertyStub))
+    {
         return nullptr;
     }
 
