@@ -1,3 +1,5 @@
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* vim: set ts=8 sts=4 et sw=4 tw=99: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -5,6 +7,7 @@
 #include "nsAutoPtr.h"
 
 #include "jsapi.h"
+#include "jsfriendapi.h"
 #include "js/OldDebugAPI.h"
 
 #include "nsJSPrincipals.h"
@@ -28,7 +31,7 @@ ReadCachedScript(StartupCache* cache, nsACString &uri, JSContext *cx,
     if (NS_FAILED(rv))
         return rv; // don't warn since NOT_AVAILABLE is an ok error
 
-    scriptp.set(JS_DecodeScript(cx, buf, len, nsJSPrincipals::get(systemPrincipal), nullptr));
+    scriptp.set(JS_DecodeScript(cx, buf, len, nullptr));
     if (!scriptp)
         return NS_ERROR_OUT_OF_MEMORY;
     return NS_OK;
