@@ -47,15 +47,16 @@ nsTableCaptionFrame::GetType() const
 }
 
 /* virtual */ nscoord
-nsTableOuterFrame::GetBaseline() const
+nsTableOuterFrame::GetLogicalBaseline(WritingMode aWritingMode) const
 {
   nsIFrame* kid = mFrames.FirstChild();
   if (!kid) {
     NS_NOTREACHED("no inner table");
-    return nsContainerFrame::GetBaseline();
+    return nsContainerFrame::GetLogicalBaseline(aWritingMode);
   }
 
-  return kid->GetBaseline() + kid->GetPosition().y;
+  return kid->GetLogicalBaseline(aWritingMode) +
+         kid->BStart(aWritingMode, mRect.width);
 }
 
 /* virtual */ nsSize
@@ -124,15 +125,15 @@ nsTableCaptionFrame::AccessibleType()
 }
 #endif
 
-#ifdef DEBUG
-NS_IMETHODIMP
+#ifdef DEBUG_FRAME_DUMP
+nsresult
 nsTableCaptionFrame::GetFrameName(nsAString& aResult) const
 {
   return MakeFrameName(NS_LITERAL_STRING("Caption"), aResult);
 }
 #endif
 
-nsIFrame* 
+nsTableCaptionFrame* 
 NS_NewTableCaptionFrame(nsIPresShell* aPresShell, nsStyleContext* aContext)
 {
   return new (aPresShell) nsTableCaptionFrame(aContext);
@@ -188,82 +189,67 @@ nsTableOuterFrame::GetChildLists(nsTArray<ChildList>* aLists) const
   mCaptionFrames.AppendIfNonempty(aLists, kCaptionList);
 }
 
-NS_IMETHODIMP 
+void 
 nsTableOuterFrame::SetInitialChildList(ChildListID     aListID,
                                        nsFrameList&    aChildList)
 {
+  MOZ_ASSERT(kCaptionList == aListID || aListID == kPrincipalList,
+             "unexpected child list");
+  MOZ_ASSERT(GetChildList(aListID).IsEmpty(),
+             "already have child frames in SetInitialChildList");
+
   if (kCaptionList == aListID) {
     // the frame constructor already checked for table-caption display type
     mCaptionFrames.SetFrames(aChildList);
-  }
-  else {
-    NS_ASSERTION(aListID == kPrincipalList, "wrong childlist");
-    NS_ASSERTION(mFrames.IsEmpty(), "Frame leak!");
-    NS_ASSERTION(aChildList.FirstChild() &&
-                 nsGkAtoms::tableFrame == aChildList.FirstChild()->GetType(),
-                 "expected a table frame");
+  } else {
+    MOZ_ASSERT(aChildList.FirstChild() &&
+               aChildList.FirstChild() == aChildList.LastChild() &&
+               nsGkAtoms::tableFrame == aChildList.FirstChild()->GetType(),
+               "expected a single table frame");
     mFrames.SetFrames(aChildList);
   }
-
-  return NS_OK;
 }
 
-NS_IMETHODIMP
+void
 nsTableOuterFrame::AppendFrames(ChildListID     aListID,
                                 nsFrameList&    aFrameList)
 {
-  nsresult rv;
-
   // We only have two child frames: the inner table and a caption frame.
   // The inner frame is provided when we're initialized, and it cannot change
-  if (kCaptionList == aListID) {
-    NS_ASSERTION(aFrameList.IsEmpty() ||
-                 aFrameList.FirstChild()->GetType() == nsGkAtoms::tableCaptionFrame,
-                 "appending non-caption frame to captionList");
-    mCaptionFrames.AppendFrames(this, aFrameList);
-    rv = NS_OK;
+  MOZ_ASSERT(kCaptionList == aListID, "unexpected child list");
+  MOZ_ASSERT(aFrameList.IsEmpty() ||
+             aFrameList.FirstChild()->GetType() == nsGkAtoms::tableCaptionFrame,
+             "appending non-caption frame to captionList");
+  mCaptionFrames.AppendFrames(this, aFrameList);
 
-    // Reflow the new caption frame. It's already marked dirty, so
-    // just tell the pres shell.
-    PresContext()->PresShell()->
-      FrameNeedsReflow(this, nsIPresShell::eTreeChange,
-                       NS_FRAME_HAS_DIRTY_CHILDREN);
-  }
-  else {
-    NS_PRECONDITION(false, "unexpected child list");
-    rv = NS_ERROR_UNEXPECTED;
-  }
-
-  return rv;
+  // Reflow the new caption frame. It's already marked dirty, so
+  // just tell the pres shell.
+  PresContext()->PresShell()->
+    FrameNeedsReflow(this, nsIPresShell::eTreeChange,
+                     NS_FRAME_HAS_DIRTY_CHILDREN);
 }
 
-NS_IMETHODIMP
+void
 nsTableOuterFrame::InsertFrames(ChildListID     aListID,
                                 nsIFrame*       aPrevFrame,
                                 nsFrameList&    aFrameList)
 {
-  if (kCaptionList == aListID) {
-    NS_ASSERTION(!aPrevFrame || aPrevFrame->GetParent() == this,
-                 "inserting after sibling frame with different parent");
-    NS_ASSERTION(aFrameList.IsEmpty() ||
-                 aFrameList.FirstChild()->GetType() == nsGkAtoms::tableCaptionFrame,
-                 "inserting non-caption frame into captionList");
-    mCaptionFrames.InsertFrames(nullptr, aPrevFrame, aFrameList);
+  MOZ_ASSERT(kCaptionList == aListID, "unexpected child list");
+  MOZ_ASSERT(aFrameList.IsEmpty() ||
+             aFrameList.FirstChild()->GetType() == nsGkAtoms::tableCaptionFrame,
+             "inserting non-caption frame into captionList");
+  MOZ_ASSERT(!aPrevFrame || aPrevFrame->GetParent() == this,
+             "inserting after sibling frame with different parent");
+  mCaptionFrames.InsertFrames(nullptr, aPrevFrame, aFrameList);
 
-    // Reflow the new caption frame. It's already marked dirty, so
-    // just tell the pres shell.
-    PresContext()->PresShell()->
-      FrameNeedsReflow(this, nsIPresShell::eTreeChange,
-                       NS_FRAME_HAS_DIRTY_CHILDREN);
-    return NS_OK;
-  }
-  else {
-    NS_PRECONDITION(!aPrevFrame, "invalid previous frame");
-    return AppendFrames(aListID, aFrameList);
-  }
+  // Reflow the new caption frame. It's already marked dirty, so
+  // just tell the pres shell.
+  PresContext()->PresShell()->
+    FrameNeedsReflow(this, nsIPresShell::eTreeChange,
+                     NS_FRAME_HAS_DIRTY_CHILDREN);
 }
 
-NS_IMETHODIMP
+void
 nsTableOuterFrame::RemoveFrame(ChildListID     aListID,
                                nsIFrame*       aOldFrame)
 {
@@ -283,8 +269,6 @@ nsTableOuterFrame::RemoveFrame(ChildListID     aListID,
   PresContext()->PresShell()->
     FrameNeedsReflow(this, nsIPresShell::eTreeChange,
                      NS_FRAME_HAS_DIRTY_CHILDREN); // also means child removed
-
-  return NS_OK;
 }
 
 void
@@ -381,11 +365,11 @@ nsTableOuterFrame::GetChildMargin(nsPresContext*           aPresContext,
   // create and init the child reflow state
   // XXX We really shouldn't construct a reflow state to do this.
   nsHTMLReflowState childRS(aPresContext, aOuterRS, aChildFrame,
-                            nsSize(aAvailWidth, aOuterRS.availableHeight),
+                            nsSize(aAvailWidth, aOuterRS.AvailableHeight()),
                             -1, -1, nsHTMLReflowState::CALLER_WILL_INIT);
   InitChildReflowState(*aPresContext, childRS);
 
-  aMargin = childRS.mComputedMargin;
+  aMargin = childRS.ComputedPhysicalMargin();
 }
 
 static nsSize
@@ -491,19 +475,19 @@ ChildShrinkWrapWidth(nsRenderingContext *aRenderingContext,
   nsCSSOffsetState offsets(aChildFrame, aRenderingContext, aCBSize.width);
   nsSize size = aChildFrame->ComputeSize(aRenderingContext, aCBSize,
                   aAvailableWidth,
-                  nsSize(offsets.mComputedMargin.LeftRight(),
-                         offsets.mComputedMargin.TopBottom()),
-                  nsSize(offsets.mComputedBorderPadding.LeftRight() -
-                           offsets.mComputedPadding.LeftRight(),
-                         offsets.mComputedBorderPadding.TopBottom() -
-                           offsets.mComputedPadding.TopBottom()),
-                  nsSize(offsets.mComputedPadding.LeftRight(),
-                         offsets.mComputedPadding.TopBottom()),
+                  nsSize(offsets.ComputedPhysicalMargin().LeftRight(),
+                         offsets.ComputedPhysicalMargin().TopBottom()),
+                  nsSize(offsets.ComputedPhysicalBorderPadding().LeftRight() -
+                           offsets.ComputedPhysicalPadding().LeftRight(),
+                         offsets.ComputedPhysicalBorderPadding().TopBottom() -
+                           offsets.ComputedPhysicalPadding().TopBottom()),
+                  nsSize(offsets.ComputedPhysicalPadding().LeftRight(),
+                         offsets.ComputedPhysicalPadding().TopBottom()),
                   true);
   if (aMarginResult)
-    *aMarginResult = offsets.mComputedMargin.LeftRight();
-  return size.width + offsets.mComputedMargin.LeftRight() +
-                      offsets.mComputedBorderPadding.LeftRight();
+    *aMarginResult = offsets.ComputedPhysicalMargin().LeftRight();
+  return size.width + offsets.ComputedPhysicalMargin().LeftRight() +
+                      offsets.ComputedPhysicalBorderPadding().LeftRight();
 }
 
 /* virtual */ nsSize
@@ -802,14 +786,14 @@ nsTableOuterFrame::OuterBeginReflowChild(nsPresContext*           aPresContext,
                                          nscoord                  aAvailWidth)
 { 
   // work around pixel rounding errors, round down to ensure we don't exceed the avail height in
-  nscoord availHeight = aOuterRS.availableHeight;
+  nscoord availHeight = aOuterRS.AvailableHeight();
   if (NS_UNCONSTRAINEDSIZE != availHeight) {
     if (mCaptionFrames.FirstChild() == aChildFrame) {
       availHeight = NS_UNCONSTRAINEDSIZE;
     } else {
       nsMargin margin;
       GetChildMargin(aPresContext, aOuterRS, aChildFrame,
-                     aOuterRS.availableWidth, margin);
+                     aOuterRS.AvailableWidth(), margin);
     
       NS_ASSERTION(NS_UNCONSTRAINEDSIZE != margin.top, "No unconstrainedsize arithmetic, please");
       availHeight -= margin.top;
@@ -838,7 +822,7 @@ nsTableOuterFrame::OuterBeginReflowChild(nsPresContext*           aPresContext,
   }
 }
 
-nsresult
+void
 nsTableOuterFrame::OuterDoReflowChild(nsPresContext*             aPresContext,
                                       nsIFrame*                  aChildFrame,
                                       const nsHTMLReflowState&   aChildRS,
@@ -859,8 +843,8 @@ nsTableOuterFrame::OuterDoReflowChild(nsPresContext*             aPresContext,
     flags |= NS_FRAME_NO_DELETE_NEXT_IN_FLOW_CHILD;
   }
 
-  return ReflowChild(aChildFrame, aPresContext, aMetrics, aChildRS,
-                     childPt.x, childPt.y, flags, aStatus);
+  ReflowChild(aChildFrame, aPresContext, aMetrics, aChildRS,
+              childPt.x, childPt.y, flags, aStatus);
 }
 
 void 
@@ -870,7 +854,7 @@ nsTableOuterFrame::UpdateReflowMetrics(uint8_t              aCaptionSide,
                                        const nsMargin&      aCaptionMargin)
 {
   SetDesiredSize(aCaptionSide, aInnerMargin, aCaptionMargin,
-                 aMet.width, aMet.height);
+                 aMet.Width(), aMet.Height());
 
   aMet.SetOverflowAreasToDesiredBounds();
   ConsiderChildOverflow(aMet.mOverflowAreas, InnerTableFrame());
@@ -879,7 +863,8 @@ nsTableOuterFrame::UpdateReflowMetrics(uint8_t              aCaptionSide,
   }
 }
 
-NS_METHOD nsTableOuterFrame::Reflow(nsPresContext*           aPresContext,
+void
+nsTableOuterFrame::Reflow(nsPresContext*           aPresContext,
                                     nsHTMLReflowMetrics&     aDesiredSize,
                                     const nsHTMLReflowState& aOuterRS,
                                     nsReflowStatus&          aStatus)
@@ -887,17 +872,16 @@ NS_METHOD nsTableOuterFrame::Reflow(nsPresContext*           aPresContext,
   DO_GLOBAL_REFLOW_COUNT("nsTableOuterFrame");
   DISPLAY_REFLOW(aPresContext, this, aOuterRS, aDesiredSize, aStatus);
 
-  nsresult rv = NS_OK;
   uint8_t captionSide = GetCaptionSide();
 
   // Initialize out parameters
-  aDesiredSize.width = aDesiredSize.height = 0;
+  aDesiredSize.Width() = aDesiredSize.Height() = 0;
   aStatus = NS_FRAME_COMPLETE;
 
   if (!(GetStateBits() & NS_FRAME_FIRST_REFLOW)) {
     // Set up our kids.  They're already present, on an overflow list, 
     // or there are none so we'll create them now
-    MoveOverflowToChildList(aPresContext);
+    MoveOverflowToChildList();
   }
 
   // Use longs to get more-aligned space.
@@ -938,8 +922,8 @@ NS_METHOD nsTableOuterFrame::Reflow(nsPresContext*           aPresContext,
     OuterBeginReflowChild(aPresContext, mCaptionFrames.FirstChild(), aOuterRS,
                           captionRSSpace, aOuterRS.ComputedWidth());
     nscoord innerAvailWidth = aOuterRS.ComputedWidth() -
-      (captionRS->ComputedWidth() + captionRS->mComputedMargin.LeftRight() +
-       captionRS->mComputedBorderPadding.LeftRight());
+      (captionRS->ComputedWidth() + captionRS->ComputedPhysicalMargin().LeftRight() +
+       captionRS->ComputedPhysicalBorderPadding().LeftRight());
     OuterBeginReflowChild(aPresContext, InnerTableFrame(), aOuterRS,
                           innerRSSpace, innerAvailWidth);
 
@@ -961,7 +945,7 @@ NS_METHOD nsTableOuterFrame::Reflow(nsPresContext*           aPresContext,
     // neither are auto).  (We take advantage of that later when we call
     // GetCaptionOrigin, though.)
     nscoord innerBorderWidth = innerRS->ComputedWidth() +
-                               innerRS->mComputedBorderPadding.LeftRight();
+                               innerRS->ComputedPhysicalBorderPadding().LeftRight();
     OuterBeginReflowChild(aPresContext, mCaptionFrames.FirstChild(), aOuterRS,
                           captionRSSpace, innerBorderWidth);
   } else {
@@ -976,21 +960,20 @@ NS_METHOD nsTableOuterFrame::Reflow(nsPresContext*           aPresContext,
   }
 
   // First reflow the caption.
-  nsHTMLReflowMetrics captionMet;
+  nsHTMLReflowMetrics captionMet(captionRS->GetWritingMode());
   nsSize captionSize;
   nsMargin captionMargin;
   if (mCaptionFrames.NotEmpty()) {
     nsReflowStatus capStatus; // don't let the caption cause incomplete
-    rv = OuterDoReflowChild(aPresContext, mCaptionFrames.FirstChild(),
-                            *captionRS, captionMet, capStatus);
-    if (NS_FAILED(rv)) return rv;
-    captionSize.width = captionMet.width;
-    captionSize.height = captionMet.height;
-    captionMargin = captionRS->mComputedMargin;
+    OuterDoReflowChild(aPresContext, mCaptionFrames.FirstChild(),
+                       *captionRS, captionMet, capStatus);
+    captionSize.width = captionMet.Width();
+    captionSize.height = captionMet.Height();
+    captionMargin = captionRS->ComputedPhysicalMargin();
     // Now that we know the height of the caption, reduce the available height
     // for the table frame if we are height constrained and the caption is above
     // or below the inner table.
-    if (NS_UNCONSTRAINEDSIZE != aOuterRS.availableHeight) {
+    if (NS_UNCONSTRAINEDSIZE != aOuterRS.AvailableHeight()) {
       nscoord captionHeight = 0;
       switch (captionSide) {
         case NS_STYLE_CAPTION_SIDE_TOP:
@@ -1001,8 +984,8 @@ NS_METHOD nsTableOuterFrame::Reflow(nsPresContext*           aPresContext,
           break;
         }
       }
-      innerRS->availableHeight =
-        std::max(0, innerRS->availableHeight - captionHeight);
+      innerRS->AvailableHeight() =
+        std::max(0, innerRS->AvailableHeight() - captionHeight);
     }
   } else {
     captionSize.SizeTo(0,0);
@@ -1011,14 +994,13 @@ NS_METHOD nsTableOuterFrame::Reflow(nsPresContext*           aPresContext,
 
   // Then, now that we know how much to reduce the width of the inner
   // table to account for side captions, reflow the inner table.
-  nsHTMLReflowMetrics innerMet;
-  rv = OuterDoReflowChild(aPresContext, InnerTableFrame(), *innerRS,
-                          innerMet, aStatus);
-  if (NS_FAILED(rv)) return rv;
+  nsHTMLReflowMetrics innerMet(innerRS->GetWritingMode());
+  OuterDoReflowChild(aPresContext, InnerTableFrame(), *innerRS,
+                     innerMet, aStatus);
   nsSize innerSize;
-  innerSize.width = innerMet.width;
-  innerSize.height = innerMet.height;
-  nsMargin innerMargin = innerRS->mComputedMargin;
+  innerSize.width = innerMet.Width();
+  innerSize.height = innerMet.Height();
+  nsMargin innerMargin = innerRS->ComputedPhysicalMargin();
 
   nsSize   containSize = GetContainingBlockSize(aOuterRS);
 
@@ -1032,8 +1014,8 @@ NS_METHOD nsTableOuterFrame::Reflow(nsPresContext*           aPresContext,
     nsPoint captionOrigin;
     GetCaptionOrigin(captionSide, containSize, innerSize, 
                      innerMargin, captionSize, captionMargin, captionOrigin);
-    FinishReflowChild(mCaptionFrames.FirstChild(), aPresContext, captionRS,
-                      captionMet, captionOrigin.x, captionOrigin.y, 0);
+    FinishReflowChild(mCaptionFrames.FirstChild(), aPresContext, captionMet,
+                      captionRS, captionOrigin.x, captionOrigin.y, 0);
     captionRS->~nsHTMLReflowState();
   }
   // XXX If the height is constrained then we need to check whether
@@ -1042,7 +1024,7 @@ NS_METHOD nsTableOuterFrame::Reflow(nsPresContext*           aPresContext,
   nsPoint innerOrigin;
   GetInnerOrigin(captionSide, containSize, captionSize, 
                  captionMargin, innerSize, innerMargin, innerOrigin);
-  FinishReflowChild(InnerTableFrame(), aPresContext, innerRS, innerMet,
+  FinishReflowChild(InnerTableFrame(), aPresContext, innerMet, innerRS,
                     innerOrigin.x, innerOrigin.y, 0);
   innerRS->~nsHTMLReflowState();
 
@@ -1067,7 +1049,6 @@ NS_METHOD nsTableOuterFrame::Reflow(nsPresContext*           aPresContext,
   // Return our desired rect
 
   NS_FRAME_SET_TRUNCATION(aStatus, aOuterRS, aDesiredSize);
-  return rv;
 }
 
 nsIAtom*
@@ -1095,7 +1076,7 @@ nsTableOuterFrame::GetCellAt(uint32_t aRowIdx, uint32_t aColIdx) const
 }
 
 
-nsIFrame*
+nsTableOuterFrame*
 NS_NewTableOuterFrame(nsIPresShell* aPresShell, nsStyleContext* aContext)
 {
   return new (aPresShell) nsTableOuterFrame(aContext);
@@ -1103,8 +1084,8 @@ NS_NewTableOuterFrame(nsIPresShell* aPresShell, nsStyleContext* aContext)
 
 NS_IMPL_FRAMEARENA_HELPERS(nsTableOuterFrame)
 
-#ifdef DEBUG
-NS_IMETHODIMP
+#ifdef DEBUG_FRAME_DUMP
+nsresult
 nsTableOuterFrame::GetFrameName(nsAString& aResult) const
 {
   return MakeFrameName(NS_LITERAL_STRING("TableOuter"), aResult);
