@@ -378,7 +378,7 @@ nsresult TransportLayerDtls::InitInternal() {
 void TransportLayerDtls::WasInserted() {
   // Connect to the lower layers
   if (!Setup()) {
-    SetState(TS_ERROR);
+    TL_SET_STATE(TS_ERROR);
   }
 }
 
@@ -590,7 +590,7 @@ bool TransportLayerDtls::Setup() {
 void TransportLayerDtls::StateChange(TransportLayer *layer, State state) {
   if (state <= state_) {
     MOZ_MTLOG(ML_ERROR, "Lower layer state is going backwards from ours");
-    SetState(TS_ERROR);
+    TL_SET_STATE(TS_ERROR);
     return;
   }
 
@@ -602,33 +602,33 @@ void TransportLayerDtls::StateChange(TransportLayer *layer, State state) {
     case TS_INIT:
       MOZ_MTLOG(ML_ERROR,
                 LAYER_INFO << "State change of lower layer to INIT forbidden");
-      SetState(TS_ERROR);
+      TL_SET_STATE(TS_ERROR);
       break;
 
     case TS_CONNECTING:
-      MOZ_MTLOG(ML_ERROR, LAYER_INFO << "Lower lower is connecting.");
+      MOZ_MTLOG(ML_ERROR, LAYER_INFO << "Lower layer is connecting.");
       break;
 
     case TS_OPEN:
       MOZ_MTLOG(ML_ERROR,
-                LAYER_INFO << "Lower lower is now open; starting TLS");
+                LAYER_INFO << "Lower layer is now open; starting TLS");
       Handshake();
       break;
 
     case TS_CLOSED:
-      MOZ_MTLOG(ML_ERROR, LAYER_INFO << "Lower lower is now closed");
-      SetState(TS_CLOSED);
+      MOZ_MTLOG(ML_ERROR, LAYER_INFO << "Lower layer is now closed");
+      TL_SET_STATE(TS_CLOSED);
       break;
 
     case TS_ERROR:
-      MOZ_MTLOG(ML_ERROR, LAYER_INFO << "Lower lower experienced an error");
-      SetState(TS_ERROR);
+      MOZ_MTLOG(ML_ERROR, LAYER_INFO << "Lower layer experienced an error");
+      TL_SET_STATE(TS_ERROR);
       break;
   }
 }
 
 void TransportLayerDtls::Handshake() {
-  SetState(TS_CONNECTING);
+  TL_SET_STATE(TS_CONNECTING);
 
   // Clear the retransmit timer
   timer_->Cancel();
@@ -640,23 +640,23 @@ void TransportLayerDtls::Handshake() {
               LAYER_INFO << "****** SSL handshake completed ******");
     if (!cert_ok_) {
       MOZ_MTLOG(ML_ERROR, LAYER_INFO << "Certificate check never occurred");
-      SetState(TS_ERROR);
+      TL_SET_STATE(TS_ERROR);
       return;
     }
-    SetState(TS_OPEN);
+    TL_SET_STATE(TS_OPEN);
   } else {
     int32_t err = PR_GetError();
     switch(err) {
       case SSL_ERROR_RX_MALFORMED_HANDSHAKE:
         if (mode_ != DGRAM) {
           MOZ_MTLOG(ML_ERROR, LAYER_INFO << "Malformed TLS message");
-          SetState(TS_ERROR);
+          TL_SET_STATE(TS_ERROR);
         } else {
           MOZ_MTLOG(ML_ERROR, LAYER_INFO << "Malformed DTLS message; ignoring");
         }
         // Fall through
       case PR_WOULD_BLOCK_ERROR:
-        MOZ_MTLOG(ML_NOTICE, LAYER_INFO << "Would have blocked");
+        MOZ_MTLOG(ML_NOTICE, LAYER_INFO << "Handshake would have blocked");
         if (mode_ == DGRAM) {
           PRIntervalTime timeout;
           rv = DTLS_GetHandshakeTimeout(ssl_fd_, &timeout);
@@ -674,7 +674,7 @@ void TransportLayerDtls::Handshake() {
         break;
       default:
         MOZ_MTLOG(ML_ERROR, LAYER_INFO << "SSL handshake error "<< err);
-        SetState(TS_ERROR);
+        TL_SET_STATE(TS_ERROR);
         break;
     }
   }
@@ -709,16 +709,16 @@ void TransportLayerDtls::PacketReceived(TransportLayer* layer,
       MOZ_MTLOG(ML_DEBUG, LAYER_INFO << "Read " << rv << " bytes from NSS");
       SignalPacketReceived(this, buf, rv);
     } else if (rv == 0) {
-      SetState(TS_CLOSED);
+      TL_SET_STATE(TS_CLOSED);
     } else {
       int32_t err = PR_GetError();
 
       if (err == PR_WOULD_BLOCK_ERROR) {
         // This gets ignored
-        MOZ_MTLOG(ML_NOTICE, LAYER_INFO << "Would have blocked");
+        MOZ_MTLOG(ML_DEBUG, LAYER_INFO << "Receive would have blocked");
       } else {
         MOZ_MTLOG(ML_NOTICE, LAYER_INFO << "NSS Error " << err);
-        SetState(TS_ERROR);
+        TL_SET_STATE(TS_ERROR);
       }
     }
   }
@@ -742,7 +742,7 @@ TransportResult TransportLayerDtls::SendPacket(const unsigned char *data,
   }
 
   if (rv == 0) {
-    SetState(TS_CLOSED);
+    TL_SET_STATE(TS_CLOSED);
     return 0;
   }
 
@@ -750,12 +750,12 @@ TransportResult TransportLayerDtls::SendPacket(const unsigned char *data,
 
   if (err == PR_WOULD_BLOCK_ERROR) {
     // This gets ignored
-    MOZ_MTLOG(ML_NOTICE, LAYER_INFO << "Would have blocked");
+    MOZ_MTLOG(ML_DEBUG, LAYER_INFO << "Send would have blocked");
     return TE_WOULDBLOCK;
   }
 
   MOZ_MTLOG(ML_NOTICE, LAYER_INFO << "NSS Error " << err);
-  SetState(TS_ERROR);
+  TL_SET_STATE(TS_ERROR);
   return TE_ERROR;
 }
 
