@@ -13,15 +13,14 @@
 #include <assert.h>
 #include <dlfcn.h>
 
+#include "webrtc/modules/audio_device/android/audio_common.h"
+#include "webrtc/modules/audio_device/android/opensles_common.h"
 #include "webrtc/modules/audio_device/android/single_rw_fifo.h"
 #include "webrtc/modules/audio_device/audio_device_buffer.h"
 #include "webrtc/system_wrappers/interface/critical_section_wrapper.h"
 #include "webrtc/system_wrappers/interface/thread_wrapper.h"
 #include "webrtc/system_wrappers/interface/trace.h"
 
-using webrtc_opensl::kDefaultSampleRate;
-using webrtc_opensl::kNumChannels;
-
 #define VOID_RETURN
 #define OPENSL_RETURN_ON_FAILURE(op, ret_val)                    \
   do {                                                           \
@@ -46,8 +45,7 @@ enum {
 namespace webrtc {
 
 OpenSlesInput::OpenSlesInput(
-    const int32_t id,
-    webrtc_opensl::PlayoutDelayProvider* delay_provider)
+    const int32_t id, PlayoutDelayProvider* delay_provider)
     : id_(id),
       delay_provider_(delay_provider),
       initialized_(false),
@@ -73,6 +71,21 @@ OpenSlesInput::OpenSlesInput(
 OpenSlesInput::~OpenSlesInput() {
 }
 
+int32_t OpenSlesInput::SetAndroidAudioDeviceObjects(void* javaVM,
+                                                    void* env,
+                                                    void* context) {
+#if !defined(WEBRTC_GONK)
+  AudioManagerJni::SetAndroidAudioDeviceObjects(javaVM, env, context);
+#endif
+  return 0;
+}
+
+void OpenSlesInput::ClearAndroidAudioDeviceObjects() {
+#if !defined(WEBRTC_GONK)
+  AudioManagerJni::ClearAndroidAudioDeviceObjects();
+#endif
+}
+
 int32_t OpenSlesInput::Init() {
   assert(!initialized_);
 
@@ -104,7 +117,7 @@ int32_t OpenSlesInput::Init() {
 
   // Set up OpenSL engine.
   OPENSL_RETURN_ON_FAILURE(f_slCreateEngine(&sles_engine_, 1, kOption, 0,
-                                          NULL, NULL),
+                                            NULL, NULL),
                            -1);
   OPENSL_RETURN_ON_FAILURE((*sles_engine_)->Realize(sles_engine_,
                                                     SL_BOOLEAN_FALSE),
@@ -155,7 +168,6 @@ int32_t OpenSlesInput::RecordingIsAvailable(bool& available) {  // NOLINT
 
 int32_t OpenSlesInput::InitRecording() {
   assert(initialized_);
-  assert(!rec_initialized_);
   rec_initialized_ = true;
   return 0;
 }
@@ -194,6 +206,7 @@ int32_t OpenSlesInput::StartRecording() {
 int32_t OpenSlesInput::StopRecording() {
   StopCbThreads();
   DestroyAudioRecorder();
+  recording_ = false;
   return 0;
 }
 
