@@ -5,6 +5,9 @@
 #include "CacheLog.h"
 #include "CacheEntry.h"
 #include "CacheStorageService.h"
+#include "CacheObserver.h"
+#include "CacheFileUtils.h"
+#include "CacheIndex.h"
 
 #include "nsIInputStream.h"
 #include "nsIOutputStream.h"
@@ -14,6 +17,7 @@
 #include "nsICacheStorage.h"
 #include "nsISerializable.h"
 #include "nsIStreamTransportService.h"
+#include "nsISizeOf.h"
 
 #include "nsComponentManagerUtils.h"
 #include "nsServiceManagerUtils.h"
@@ -37,30 +41,44 @@ static uint32_t const ENTRY_NEEDS_REVALIDATION =
 static uint32_t const ENTRY_NOT_WANTED =
   nsICacheEntryOpenCallback::ENTRY_NOT_WANTED;
 
-NS_IMPL_ISUPPORTS1(CacheEntry::Handle, nsICacheEntry)
+NS_IMPL_ISUPPORTS(CacheEntryHandle, nsICacheEntry)
 
-// CacheEntry::Handle
+// CacheEntryHandle
 
-CacheEntry::Handle::Handle(CacheEntry* aEntry)
+CacheEntryHandle::CacheEntryHandle(CacheEntry* aEntry)
 : mEntry(aEntry)
 {
-  MOZ_COUNT_CTOR(CacheEntry::Handle);
+  MOZ_COUNT_CTOR(CacheEntryHandle);
 
-  LOG(("New CacheEntry::Handle %p for entry %p", this, aEntry));
+#ifdef DEBUG
+  if (!mEntry->HandlesCount()) {
+    // CacheEntry.mHandlesCount must go from zero to one only under
+    // the service lock. Can access CacheStorageService::Self() w/o a check
+    // since CacheEntry hrefs it.
+    CacheStorageService::Self()->Lock().AssertCurrentThreadOwns();
+  }
+#endif
+
+  mEntry->AddHandleRef();
+
+  LOG(("New CacheEntryHandle %p for entry %p", this, aEntry));
 }
 
-CacheEntry::Handle::~Handle()
+CacheEntryHandle::~CacheEntryHandle()
 {
-  mEntry->OnWriterClosed(this);
+  mEntry->ReleaseHandleRef();
+  mEntry->OnHandleClosed(this);
 
-  MOZ_COUNT_DTOR(CacheEntry::Handle);
+  MOZ_COUNT_DTOR(CacheEntryHandle);
 }
 
 // CacheEntry::Callback
 
-CacheEntry::Callback::Callback(nsICacheEntryOpenCallback *aCallback,
+CacheEntry::Callback::Callback(CacheEntry* aEntry,
+                               nsICacheEntryOpenCallback *aCallback,
                                bool aReadOnly, bool aCheckOnAnyThread)
-: mCallback(aCallback)
+: mEntry(aEntry)
+, mCallback(aCallback)
 , mTargetThread(do_GetCurrentThread())
 , mReadOnly(aReadOnly)
 , mCheckOnAnyThread(aCheckOnAnyThread)
@@ -68,10 +86,16 @@ CacheEntry::Callback::Callback(nsICacheEntryOpenCallback *aCallback,
 , mNotWanted(false)
 {
   MOZ_COUNT_CTOR(CacheEntry::Callback);
+
+  // The counter may go from zero to non-null only under the service lock
+  // but here we expect it to be already positive.
+  MOZ_ASSERT(mEntry->HandlesCount());
+  mEntry->AddHandleRef();
 }
 
 CacheEntry::Callback::Callback(CacheEntry::Callback const &aThat)
-: mCallback(aThat.mCallback)
+: mEntry(aThat.mEntry)
+, mCallback(aThat.mCallback)
 , mTargetThread(aThat.mTargetThread)
 , mReadOnly(aThat.mReadOnly)
 , mCheckOnAnyThread(aThat.mCheckOnAnyThread)
@@ -79,13 +103,34 @@ CacheEntry::Callback::Callback(CacheEntry::Callback const &aThat)
 , mNotWanted(aThat.mNotWanted)
 {
   MOZ_COUNT_CTOR(CacheEntry::Callback);
+
+  // The counter may go from zero to non-null only under the service lock
+  // but here we expect it to be already positive.
+  MOZ_ASSERT(mEntry->HandlesCount());
+  mEntry->AddHandleRef();
 }
 
 CacheEntry::Callback::~Callback()
 {
+  ProxyRelease(mCallback, mTargetThread);
+
+  mEntry->ReleaseHandleRef();
   MOZ_COUNT_DTOR(CacheEntry::Callback);
 }
 
+void CacheEntry::Callback::ExchangeEntry(CacheEntry* aEntry)
+{
+  if (mEntry == aEntry)
+    return;
+
+  // The counter may go from zero to non-null only under the service lock
+  // but here we expect it to be already positive.
+  MOZ_ASSERT(aEntry->HandlesCount());
+  aEntry->AddHandleRef();
+  mEntry->ReleaseHandleRef();
+  mEntry = aEntry;
+}
+
 nsresult CacheEntry::Callback::OnCheckThread(bool *aOnCheckThread) const
 {
   if (!mCheckOnAnyThread) {
@@ -105,10 +150,10 @@ nsresult CacheEntry::Callback::OnAvailThread(bool *aOnAvailThread) const
 
 // CacheEntry
 
-NS_IMPL_ISUPPORTS3(CacheEntry,
-                   nsICacheEntry,
-                   nsIRunnable,
-                   CacheFileListener)
+NS_IMPL_ISUPPORTS(CacheEntry,
+                  nsICacheEntry,
+                  nsIRunnable,
+                  CacheFileListener)
 
 CacheEntry::CacheEntry(const nsACString& aStorageID,
                        nsIURI* aURI,
@@ -130,7 +175,7 @@ CacheEntry::CacheEntry(const nsACString& aStorageID,
 , mRegistration(NEVERREGISTERED)
 , mWriter(nullptr)
 , mPredictedDataSize(0)
-, mDataSize(0)
+, mUseCount(0)
 , mReleaseThread(NS_GetCurrentThread())
 {
   MOZ_COUNT_CTOR(CacheEntry);
@@ -167,12 +212,12 @@ char const * CacheEntry::StateString(uint32_t aState)
 
 #endif
 
-nsresult CacheEntry::HashingKeyWithStorage(nsACString &aResult)
+nsresult CacheEntry::HashingKeyWithStorage(nsACString &aResult) const
 {
   return HashingKey(mStorageID, mEnhanceID, mURI, aResult);
 }
 
-nsresult CacheEntry::HashingKey(nsACString &aResult)
+nsresult CacheEntry::HashingKey(nsACString &aResult) const
 {
   return HashingKey(EmptyCString(), mEnhanceID, mURI, aResult);
 }
@@ -182,27 +227,34 @@ nsresult CacheEntry::HashingKey(nsCSubstring const& aStorageID,
                                 nsCSubstring const& aEnhanceID,
                                 nsIURI* aURI,
                                 nsACString &aResult)
+{
+  nsAutoCString spec;
+  nsresult rv = aURI->GetAsciiSpec(spec);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  return HashingKey(aStorageID, aEnhanceID, spec, aResult);
+}
+
+// static
+nsresult CacheEntry::HashingKey(nsCSubstring const& aStorageID,
+                                nsCSubstring const& aEnhanceID,
+                                nsCSubstring const& aURISpec,
+                                nsACString &aResult)
 {
   /**
    * This key is used to salt hash that is a base for disk file name.
    * Changing it will cause we will not be able to find files on disk.
    */
 
-  if (aStorageID.Length()) {
-    aResult.Append(aStorageID);
-    aResult.Append(':');
+  aResult.Append(aStorageID);
+
+  if (!aEnhanceID.IsEmpty()) {
+    CacheFileUtils::AppendTagWithValue(aResult, '~', aEnhanceID);
   }
 
-  if (aEnhanceID.Length()) {
-    aResult.Append(aEnhanceID);
-    aResult.Append(':');
-  }
-
-  nsAutoCString spec;
-  nsresult rv = aURI->GetAsciiSpec(spec);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  aResult.Append(spec);
+  // Appending directly
+  aResult.Append(':');
+  aResult.Append(aURISpec);
 
   return NS_OK;
 }
@@ -213,6 +265,7 @@ void CacheEntry::AsyncOpen(nsICacheEntryOpenCallback* aCallback, uint32_t aFlags
     this, StateString(mState), aFlags, aCallback));
 
   bool readonly = aFlags & nsICacheStorage::OPEN_READONLY;
+  bool bypassIfBusy = aFlags & nsICacheStorage::OPEN_BYPASS_IF_BUSY;
   bool truncate = aFlags & nsICacheStorage::OPEN_TRUNCATE;
   bool priority = aFlags & nsICacheStorage::OPEN_PRIORITY;
   bool multithread = aFlags & nsICacheStorage::CHECK_MULTITHREADED;
@@ -220,19 +273,37 @@ void CacheEntry::AsyncOpen(nsICacheEntryOpenCallback* aCallback, uint32_t aFlags
   MOZ_ASSERT(!readonly || !truncate, "Bad flags combination");
   MOZ_ASSERT(!(truncate && mState > LOADING), "Must not call truncate on already loaded entry");
 
-  Callback callback(aCallback, readonly, multithread);
+  Callback callback(this, aCallback, readonly, multithread);
 
+  if (!Open(callback, truncate, priority, bypassIfBusy)) {
+    // We get here when the callback wants to bypass cache when it's busy.
+    LOG(("  writing or revalidating, callback wants to bypass cache"));
+    callback.mNotWanted = true;
+    InvokeAvailableCallback(callback);
+  }
+}
+
+bool CacheEntry::Open(Callback & aCallback, bool aTruncate,
+                      bool aPriority, bool aBypassIfBusy)
+{
   mozilla::MutexAutoLock lock(mLock);
 
-  RememberCallback(callback);
+  // Check state under the lock
+  if (aBypassIfBusy && (mState == WRITING || mState == REVALIDATING)) {
+    return false;
+  }
+
+  RememberCallback(aCallback);
 
   // Load() opens the lock
-  if (Load(truncate, priority)) {
+  if (Load(aTruncate, aPriority)) {
     // Loading is in progress...
-    return;
+    return true;
   }
 
   InvokeCallbacks();
+
+  return true;
 }
 
 bool CacheEntry::Load(bool aTruncate, bool aPriority)
@@ -255,31 +326,61 @@ bool CacheEntry::Load(bool aTruncate, bool aPriority)
 
   MOZ_ASSERT(!mFile);
 
-  bool directLoad = aTruncate || !mUseDisk;
-  if (directLoad)
-    mFileStatus = NS_OK;
-  else
-    mLoadStart = TimeStamp::Now();
+  nsresult rv;
+
+  nsAutoCString fileKey;
+  rv = HashingKeyWithStorage(fileKey);
+
+  // Check the index under two conditions for two states and take appropriate action:
+  // 1. When this is a disk entry and not told to truncate, check there is a disk file.
+  //    If not, set the 'truncate' flag to true so that this entry will open instantly
+  //    as a new one.
+  // 2. When this is a memory-only entry, check there is a disk file.
+  //    If there is or could be, doom that file.
+  if ((!aTruncate || !mUseDisk) && NS_SUCCEEDED(rv)) {
+    // Check the index right now to know we have or have not the entry
+    // as soon as possible.
+    CacheIndex::EntryStatus status;
+    if (NS_SUCCEEDED(CacheIndex::HasEntry(fileKey, &status))) {
+      switch (status) {
+      case CacheIndex::DOES_NOT_EXIST:
+        LOG(("  entry doesn't exist according information from the index, truncating"));
+        aTruncate = true;
+        break;
+      case CacheIndex::EXISTS:
+      case CacheIndex::DO_NOT_KNOW:
+        if (!mUseDisk) {
+          LOG(("  entry open as memory-only, but there is (status=%d) a file, dooming it", status));
+          CacheFileIOManager::DoomFileByKey(fileKey, nullptr);
+        }
+        break;
+      }
+    }
+  }
 
   mFile = new CacheFile();
 
   BackgroundOp(Ops::REGISTER);
 
+  bool directLoad = aTruncate || !mUseDisk;
+  if (directLoad) {
+    mFileStatus = NS_OK;
+    // mLoadStart will be used to calculate telemetry of life-time of this entry.
+    // Low resulution is then enough.
+    mLoadStart = TimeStamp::NowLoRes();
+  } else {
+    mLoadStart = TimeStamp::Now();
+  }
+
   {
     mozilla::MutexAutoUnlock unlock(mLock);
 
-    nsresult rv;
-
-    nsAutoCString fileKey;
-    rv = HashingKeyWithStorage(fileKey);
-
     LOG(("  performing load, file=%p", mFile.get()));
     if (NS_SUCCEEDED(rv)) {
       rv = mFile->Init(fileKey,
                        aTruncate,
                        !mUseDisk,
                        aPriority,
-                       false /* key is not a hash */,
                        directLoad ? nullptr : this);
     }
 
@@ -332,9 +433,16 @@ NS_IMETHODIMP CacheEntry::OnFileReady(nsresult aResult, bool aIsNew)
 
   mFileStatus = aResult;
 
-  if (mState == READY)
+  if (mState == READY) {
     mHasData = true;
 
+    uint32_t frecency;
+    mFile->GetFrecency(&frecency);
+    // mFrecency is held in a double to increase computance precision.
+    // It is ok to persist frecency only as a uint32 with some math involved.
+    mFrecency = INT2FRECENCY(frecency);
+  }
+
   InvokeCallbacks();
   return NS_OK;
 }
@@ -350,7 +458,8 @@ NS_IMETHODIMP CacheEntry::OnFileDoomed(nsresult aResult)
   return NS_OK;
 }
 
-already_AddRefed<CacheEntry> CacheEntry::ReopenTruncated(nsICacheEntryOpenCallback* aCallback)
+already_AddRefed<CacheEntryHandle> CacheEntry::ReopenTruncated(bool aMemoryOnly,
+                                                               nsICacheEntryOpenCallback* aCallback)
 {
   LOG(("CacheEntry::ReopenTruncated [this=%p]", this));
 
@@ -359,6 +468,7 @@ already_AddRefed<CacheEntry> CacheEntry::ReopenTruncated(nsICacheEntryOpenCallba
   // Hold callbacks invocation, AddStorageEntry would invoke from doom prematurly
   mPreventCallbacks = true;
 
+  nsRefPtr<CacheEntryHandle> handle;
   nsRefPtr<CacheEntry> newEntry;
   {
     mozilla::MutexAutoUnlock unlock(mLock);
@@ -366,17 +476,17 @@ already_AddRefed<CacheEntry> CacheEntry::ReopenTruncated(nsICacheEntryOpenCallba
     // The following call dooms this entry (calls DoomAlreadyRemoved on us)
     nsresult rv = CacheStorageService::Self()->AddStorageEntry(
       GetStorageID(), GetURI(), GetEnhanceID(),
-      mUseDisk,
+      mUseDisk && !aMemoryOnly,
       true, // always create
       true, // truncate existing (this one)
-      getter_AddRefs(newEntry));
-
-    LOG(("  exchanged entry %p by entry %p, rv=0x%08x", this, newEntry.get(), rv));
+      getter_AddRefs(handle));
 
     if (NS_SUCCEEDED(rv)) {
+      newEntry = handle->Entry();
+      LOG(("  exchanged entry %p by entry %p, rv=0x%08x", this, newEntry.get(), rv));
       newEntry->AsyncOpen(aCallback, nsICacheStorage::OPEN_TRUNCATE);
-    }
-    else {
+    } else {
+      LOG(("  exchanged of entry %p failed, rv=0x%08x", this, rv));
       AsyncDoom(nullptr);
     }
   }
@@ -389,7 +499,14 @@ already_AddRefed<CacheEntry> CacheEntry::ReopenTruncated(nsICacheEntryOpenCallba
   newEntry->TransferCallbacks(*this);
   mCallbacks.Clear();
 
-  return newEntry.forget();
+  // Must return a new write handle, since the consumer is expected to
+  // write to this newly recreated entry.  The |handle| is only a common
+  // reference counter and doesn't revert entry state back when write
+  // fails and also doesn't update the entry frecency.  Not updating
+  // frecency causes entries to not be purged from our memory pools.
+  nsRefPtr<CacheEntryHandle> writeHandle =
+    newEntry->NewWriteHandle();
+  return writeHandle.forget();
 }
 
 void CacheEntry::TransferCallbacks(CacheEntry & aFromEntry)
@@ -404,16 +521,23 @@ void CacheEntry::TransferCallbacks(CacheEntry & aFromEntry)
   else
     mCallbacks.AppendElements(aFromEntry.mCallbacks);
 
-  if (mCallbacks.Length())
+  uint32_t callbacksLength = mCallbacks.Length();
+  if (callbacksLength) {
+    // Carry the entry reference (unfortunatelly, needs to be done manually...)
+    for (uint32_t i = 0; i < callbacksLength; ++i)
+      mCallbacks[i].ExchangeEntry(this);
+
     BackgroundOp(Ops::CALLBACKS, true);
+  }
 }
 
-void CacheEntry::RememberCallback(Callback const& aCallback)
+void CacheEntry::RememberCallback(Callback & aCallback)
 {
-  LOG(("CacheEntry::RememberCallback [this=%p, cb=%p]", this, aCallback.mCallback.get()));
-
   mLock.AssertCurrentThreadOwns();
 
+  LOG(("CacheEntry::RememberCallback [this=%p, cb=%p, state=%s]",
+    this, aCallback.mCallback.get(), StateString(mState)));
+
   mCallbacks.AppendElement(aCallback);
 }
 
@@ -514,20 +638,23 @@ bool CacheEntry::InvokeCallback(Callback & aCallback)
     // mRecheckAfterWrite flag already set means the callback has already passed
     // the onCacheEntryCheck call. Until the current write is not finished this
     // callback will be bypassed.
-    if (!aCallback.mReadOnly && !aCallback.mRecheckAfterWrite) {
-      if (mState == EMPTY) {
-        // Advance to writing state, we expect to invoke the callback and let
-        // it fill content of this entry.  Must set and check the state here
-        // to prevent more then one
-        mState = WRITING;
-        LOG(("  advancing to WRITING state"));
-      }
+    if (!aCallback.mRecheckAfterWrite) {
 
-      if (!aCallback.mCallback) {
-        // We can be given no callback only in case of recreate, it is ok
-        // to advance to WRITING state since the caller of recreate is expected
-        // to write this entry now.
-        return true;
+      if (!aCallback.mReadOnly) {
+        if (mState == EMPTY) {
+          // Advance to writing state, we expect to invoke the callback and let
+          // it fill content of this entry.  Must set and check the state here
+          // to prevent more then one
+          mState = WRITING;
+          LOG(("  advancing to WRITING state"));
+        }
+
+        if (!aCallback.mCallback) {
+          // We can be given no callback only in case of recreate, it is ok
+          // to advance to WRITING state since the caller of recreate is expected
+          // to write this entry now.
+          return true;
+        }
       }
 
       if (mState == READY) {
@@ -581,7 +708,7 @@ bool CacheEntry::InvokeCallback(Callback & aCallback)
       // If we don't have data and the callback wants a complete entry,
       // don't invoke now.
       bool bypass = !mHasData;
-      if (!bypass) {
+      if (!bypass && NS_SUCCEEDED(mFileStatus)) {
         int64_t _unused;
         bypass = !mFile->DataSize(&_unused);
       }
@@ -632,6 +759,11 @@ void CacheEntry::InvokeAvailableCallback(Callback const & aCallback)
     return;
   }
 
+  if (NS_SUCCEEDED(mFileStatus)) {
+    // Let the last-fetched and fetch-count properties be updated.
+    mFile->OnFetched();
+  }
+
   if (mIsDoomed || aCallback.mNotWanted) {
     LOG(("  doomed or not wanted, notifying OCEA with NS_ERROR_CACHE_KEY_NOT_FOUND"));
     aCallback.mCallback->OnCacheEntryAvailable(
@@ -646,8 +778,9 @@ void CacheEntry::InvokeAvailableCallback(Callback const & aCallback)
       BackgroundOp(Ops::FRECENCYUPDATE);
     }
 
+    nsRefPtr<CacheEntryHandle> handle = NewHandle();
     aCallback.mCallback->OnCacheEntryAvailable(
-      this, false, nullptr, NS_OK);
+      handle, false, nullptr, NS_OK);
     return;
   }
 
@@ -659,13 +792,13 @@ void CacheEntry::InvokeAvailableCallback(Callback const & aCallback)
   }
 
   // This is a new or potentially non-valid entry and needs to be fetched first.
-  // The Handle blocks other consumers until the channel
+  // The CacheEntryHandle blocks other consumers until the channel
   // either releases the entry or marks metadata as filled or whole entry valid,
   // i.e. until MetaDataReady() or SetValid() on the entry is called respectively.
 
   // Consumer will be responsible to fill or validate the entry metadata and data.
 
-  nsRefPtr<Handle> handle = NewWriteHandle();
+  nsRefPtr<CacheEntryHandle> handle = NewWriteHandle();
   rv = aCallback.mCallback->OnCacheEntryAvailable(
     handle, state == WRITING, nullptr, NS_OK);
 
@@ -673,24 +806,29 @@ void CacheEntry::InvokeAvailableCallback(Callback const & aCallback)
     LOG(("  writing/revalidating failed (0x%08x)", rv));
 
     // Consumer given a new entry failed to take care of the entry.
-    OnWriterClosed(handle);
+    OnHandleClosed(handle);
     return;
   }
 
   LOG(("  writing/revalidating"));
 }
 
-CacheEntry::Handle* CacheEntry::NewWriteHandle()
+CacheEntryHandle* CacheEntry::NewHandle()
+{
+  return new CacheEntryHandle(this);
+}
+
+CacheEntryHandle* CacheEntry::NewWriteHandle()
 {
   mozilla::MutexAutoLock lock(mLock);
 
   BackgroundOp(Ops::FRECENCYUPDATE);
-  return (mWriter = new Handle(this));
+  return (mWriter = NewHandle());
 }
 
-void CacheEntry::OnWriterClosed(Handle const* aHandle)
+void CacheEntry::OnHandleClosed(CacheEntryHandle const* aHandle)
 {
-  LOG(("CacheEntry::OnWriterClosed [this=%p, state=%s, handle=%p]", this, StateString(mState), aHandle));
+  LOG(("CacheEntry::OnHandleClosed [this=%p, state=%s, handle=%p]", this, StateString(mState), aHandle));
 
   nsCOMPtr<nsIOutputStream> outputStream;
 
@@ -698,7 +836,7 @@ void CacheEntry::OnWriterClosed(Handle const* aHandle)
     mozilla::MutexAutoLock lock(mLock);
 
     if (mWriter != aHandle) {
-      LOG(("  not the current writer"));
+      LOG(("  not the writer"));
       return;
     }
 
@@ -721,6 +859,21 @@ void CacheEntry::OnWriterClosed(Handle const* aHandle)
       mState = READY;
     }
 
+    if (mState == READY && !mHasData) {
+      // We may get to this state when following steps happen:
+      // 1. a new entry is given to a consumer
+      // 2. the consumer calls MetaDataReady(), we transit to READY
+      // 3. abandons the entry w/o opening the output stream, mHasData left false
+      //
+      // In this case any following consumer will get a ready entry (with metadata)
+      // but in state like the entry data write was still happening (was in progress)
+      // and will indefinitely wait for the entry data or even the entry itself when
+      // RECHECK_AFTER_WRITE is returned from onCacheEntryCheck.
+      LOG(("  we are in READY state, pretend we have data regardless it"
+            " has actully been never touched"));
+      mHasData = true;
+    }
+
     InvokeCallbacks();
   }
 
@@ -739,28 +892,22 @@ void CacheEntry::OnOutputClosed()
   InvokeCallbacks();
 }
 
-bool CacheEntry::UsingDisk() const
+bool CacheEntry::IsReferenced() const
 {
   CacheStorageService::Self()->Lock().AssertCurrentThreadOwns();
 
-  return mUseDisk;
+  // Increasing this counter from 0 to non-null and this check both happen only
+  // under the service lock.
+  return mHandlesCount > 0;
 }
 
-bool CacheEntry::SetUsingDisk(bool aUsingDisk)
+bool CacheEntry::IsFileDoomed()
 {
-  // Called by the service when this entry is reopen to reflect
-  // demanded storage target.
-
-  if (mState >= READY) {
-    // Don't modify after this entry has been filled.
-    return false;
+  if (NS_SUCCEEDED(mFileStatus)) {
+    return mFile->IsDoomed();
   }
 
-  CacheStorageService::Self()->Lock().AssertCurrentThreadOwns();
-
-  bool changed = mUseDisk != aUsingDisk;
-  mUseDisk = aUsingDisk;
-  return changed;
+  return false;
 }
 
 uint32_t CacheEntry::GetMetadataMemoryConsumption()
@@ -776,7 +923,7 @@ uint32_t CacheEntry::GetMetadataMemoryConsumption()
 
 // nsICacheEntry
 
-NS_IMETHODIMP CacheEntry::GetPersistToDisk(bool *aPersistToDisk)
+NS_IMETHODIMP CacheEntry::GetPersistent(bool *aPersistToDisk)
 {
   // No need to sync when only reading.
   // When consumer needs to be consistent with state of the memory storage entries
@@ -784,28 +931,6 @@ NS_IMETHODIMP CacheEntry::GetPersistToDisk(bool *aPersistToDisk)
   *aPersistToDisk = mUseDisk;
   return NS_OK;
 }
-NS_IMETHODIMP CacheEntry::SetPersistToDisk(bool aPersistToDisk)
-{
-  LOG(("CacheEntry::SetPersistToDisk [this=%p, persist=%d]", this, aPersistToDisk));
-
-  if (mState >= READY) {
-    LOG(("  failed, called after filling the entry"));
-    return NS_ERROR_NOT_AVAILABLE;
-  }
-
-  if (mUseDisk == aPersistToDisk)
-    return NS_OK;
-
-  mozilla::MutexAutoLock lock(CacheStorageService::Self()->Lock());
-
-  mUseDisk = aPersistToDisk;
-  CacheStorageService::Self()->RecordMemoryOnlyEntry(
-    this, !aPersistToDisk, false /* don't overwrite */);
-
-  // File persistence is setup just before we open output stream on it.
-
-  return NS_OK;
-}
 
 NS_IMETHODIMP CacheEntry::GetKey(nsACString & aKey)
 {
@@ -894,7 +1019,7 @@ NS_IMETHODIMP CacheEntry::OpenOutputStream(int64_t offset, nsIOutputStream * *_r
 
   MOZ_ASSERT(mState > EMPTY);
 
-  if (mOutputStream) {
+  if (mOutputStream && !mIsDoomed) {
     LOG(("  giving phantom output stream"));
     mOutputStream.forget(_retval);
   }
@@ -966,6 +1091,14 @@ NS_IMETHODIMP CacheEntry::GetPredictedDataSize(int64_t *aPredictedDataSize)
 NS_IMETHODIMP CacheEntry::SetPredictedDataSize(int64_t aPredictedDataSize)
 {
   mPredictedDataSize = aPredictedDataSize;
+
+  if (CacheObserver::EntryIsTooBig(mPredictedDataSize, mUseDisk)) {
+    LOG(("CacheEntry::SetPredictedDataSize [this=%p] too big, dooming", this));
+    AsyncDoom(nullptr);
+
+    return NS_ERROR_FILE_TOO_BIG;
+  }
+
   return NS_OK;
 }
 
@@ -981,16 +1114,15 @@ NS_IMETHODIMP CacheEntry::GetSecurityInfo(nsISupports * *aSecurityInfo)
 
   NS_ENSURE_SUCCESS(mFileStatus, NS_ERROR_NOT_AVAILABLE);
 
-  char const* info;
+  nsXPIDLCString info;
   nsCOMPtr<nsISupports> secInfo;
   nsresult rv;
 
-  rv = mFile->GetElement("security-info", &info);
+  rv = mFile->GetElement("security-info", getter_Copies(info));
   NS_ENSURE_SUCCESS(rv, rv);
 
   if (info) {
-    rv = NS_DeserializeObject(nsDependentCString(info),
-                              getter_AddRefs(secInfo));
+    rv = NS_DeserializeObject(info, getter_AddRefs(secInfo));
     NS_ENSURE_SUCCESS(rv, rv);
   }
 
@@ -1061,11 +1193,14 @@ NS_IMETHODIMP CacheEntry::AsyncDoom(nsICacheEntryDoomCallback *aCallback)
 
     mIsDoomed = true;
     mDoomCallback = aCallback;
-    BackgroundOp(Ops::DOOM);
   }
 
-  // Immediately remove the entry from the storage hash table
-  CacheStorageService::Self()->RemoveEntry(this);
+  // This immediately removes the entry from the master hashtable and also
+  // immediately dooms the file.  This way we make sure that any consumer
+  // after this point asking for the same entry won't get
+  //   a) this entry
+  //   b) a new entry with the same file
+  PurgeAndDoom();
 
   return NS_OK;
 }
@@ -1074,15 +1209,7 @@ NS_IMETHODIMP CacheEntry::GetMetaDataElement(const char * aKey, char * *aRetval)
 {
   NS_ENSURE_SUCCESS(mFileStatus, NS_ERROR_NOT_AVAILABLE);
 
-  const char *value;
-  nsresult rv = mFile->GetElement(aKey, &value);
-  NS_ENSURE_SUCCESS(rv, rv);
-
-  if (!value)
-    return NS_ERROR_NOT_AVAILABLE;
-
-  *aRetval = NS_strdup(value);
-  return NS_OK;
+  return mFile->GetElement(aKey, aRetval);
 }
 
 NS_IMETHODIMP CacheEntry::SetMetaDataElement(const char * aKey, const char * aValue)
@@ -1092,6 +1219,13 @@ NS_IMETHODIMP CacheEntry::SetMetaDataElement(const char * aKey, const char * aVa
   return mFile->SetElement(aKey, aValue);
 }
 
+NS_IMETHODIMP CacheEntry::VisitMetaData(nsICacheEntryMetaDataVisitor *aVisitor)
+{
+  NS_ENSURE_SUCCESS(mFileStatus, NS_ERROR_NOT_AVAILABLE);
+
+  return mFile->VisitMetaData(aVisitor);
+}
+
 NS_IMETHODIMP CacheEntry::MetaDataReady()
 {
   mozilla::MutexAutoLock lock(mLock);
@@ -1135,15 +1269,15 @@ NS_IMETHODIMP CacheEntry::SetValid()
   return NS_OK;
 }
 
-NS_IMETHODIMP CacheEntry::Recreate(nsICacheEntry **_retval)
+NS_IMETHODIMP CacheEntry::Recreate(bool aMemoryOnly,
+                                   nsICacheEntry **_retval)
 {
   LOG(("CacheEntry::Recreate [this=%p, state=%s]", this, StateString(mState)));
 
   mozilla::MutexAutoLock lock(mLock);
 
-  nsRefPtr<CacheEntry> newEntry = ReopenTruncated(nullptr);
-  if (newEntry) {
-    nsRefPtr<Handle> handle = newEntry->NewWriteHandle();
+  nsRefPtr<CacheEntryHandle> handle = ReopenTruncated(aMemoryOnly, nullptr);
+  if (handle) {
     handle.forget(_retval);
     return NS_OK;
   }
@@ -1152,13 +1286,6 @@ NS_IMETHODIMP CacheEntry::Recreate(nsICacheEntry **_retval)
   return NS_OK;
 }
 
-NS_IMETHODIMP CacheEntry::SetDataSize(uint32_t size)
-{
-  // ?
-  mDataSize = size;
-  return NS_OK;
-}
-
 NS_IMETHODIMP CacheEntry::GetDataSize(int64_t *aDataSize)
 {
   LOG(("CacheEntry::GetDataSize [this=%p]", this));
@@ -1209,17 +1336,6 @@ NS_IMETHODIMP CacheEntry::Close()
   return NS_OK;
 }
 
-NS_IMETHODIMP CacheEntry::GetStoragePolicy(nsCacheStoragePolicy *aStoragePolicy)
-{
-  // NOT IMPLEMENTED ACTUALLY
-  return NS_OK;
-}
-NS_IMETHODIMP CacheEntry::SetStoragePolicy(nsCacheStoragePolicy aStoragePolicy)
-{
-  // NOT IMPLEMENTED ACTUALLY
-  return NS_OK;
-}
-
 // nsIRunnable
 
 NS_IMETHODIMP CacheEntry::Run()
@@ -1298,12 +1414,25 @@ bool CacheEntry::Purge(uint32_t aWhat)
     return false;
   }
 
+  if (NS_SUCCEEDED(mFileStatus) && mFile->IsWriteInProgress()) {
+    // The file is used when there are open streams or chunks/metadata still waiting for
+    // write.  In this case, this entry cannot be purged, otherwise reopenned entry
+    // would may not even find the data on disk - CacheFile is not shared and cannot be
+    // left orphan when its job is not done, hence keep the whole entry.
+    LOG(("  file still under use"));
+    return false;
+  }
+
   switch (aWhat) {
   case PURGE_WHOLE_ONLY_DISK_BACKED:
   case PURGE_WHOLE:
     {
+      if (!CacheStorageService::Self()->RemoveEntry(this, true)) {
+        LOG(("  not purging, still referenced"));
+        return false;
+      }
+
       CacheStorageService::Self()->UnregisterEntry(this);
-      CacheStorageService::Self()->RemoveEntry(this);
 
       // Entry removed it self from control arrays, return true
       return true;
@@ -1328,8 +1457,6 @@ void CacheEntry::PurgeAndDoom()
 {
   LOG(("CacheEntry::PurgeAndDoom [this=%p]", this));
 
-  MOZ_ASSERT(CacheStorageService::IsOnManagementThread());
-
   CacheStorageService::Self()->RemoveEntry(this);
   DoomAlreadyRemoved();
 }
@@ -1338,36 +1465,45 @@ void CacheEntry::DoomAlreadyRemoved()
 {
   LOG(("CacheEntry::DoomAlreadyRemoved [this=%p]", this));
 
+  mozilla::MutexAutoLock lock(mLock);
+
   mIsDoomed = true;
 
-  if (!CacheStorageService::IsOnManagementThread()) {
-    mozilla::MutexAutoLock lock(mLock);
+  // This schedules dooming of the file, dooming is ensured to happen
+  // sooner than demand to open the same file made after this point
+  // so that we don't get this file for any newer opened entry(s).
+  DoomFile();
 
-    BackgroundOp(Ops::DOOM);
-    return;
-  }
+  // Must force post here since may be indirectly called from
+  // InvokeCallbacks of this entry and we don't want reentrancy here.
+  BackgroundOp(Ops::CALLBACKS, true);
+  // Process immediately when on the management thread.
+  BackgroundOp(Ops::UNREGISTER);
+}
 
-  CacheStorageService::Self()->UnregisterEntry(this);
-
-  {
-    mozilla::MutexAutoLock lock(mLock);
-
-    if (mCallbacks.Length()) {
-      // Must force post here since may be indirectly called from
-      // InvokeCallbacks of this entry and we don't want reentrancy here.
-      BackgroundOp(Ops::CALLBACKS, true);
-    }
-  }
+void CacheEntry::DoomFile()
+{
+  nsresult rv = NS_ERROR_NOT_AVAILABLE;
 
   if (NS_SUCCEEDED(mFileStatus)) {
-    nsresult rv = mFile->Doom(mDoomCallback ? this : nullptr);
+    // Always calls the callback asynchronously.
+    rv = mFile->Doom(mDoomCallback ? this : nullptr);
     if (NS_SUCCEEDED(rv)) {
       LOG(("  file doomed"));
       return;
     }
+    
+    if (NS_ERROR_FILE_NOT_FOUND == rv) {
+      // File is set to be just memory-only, notify the callbacks
+      // and pretend dooming has succeeded.  From point of view of
+      // the entry it actually did - the data is gone and cannot be
+      // reused.
+      rv = NS_OK;
+    }
   }
 
-  OnFileDoomed(NS_OK);
+  // Always posts to the main thread.
+  OnFileDoomed(rv);
 }
 
 void CacheEntry::BackgroundOp(uint32_t aOperations, bool aForceAsync)
@@ -1382,53 +1518,73 @@ void CacheEntry::BackgroundOp(uint32_t aOperations, bool aForceAsync)
     return;
   }
 
-  mozilla::MutexAutoUnlock unlock(mLock);
+  {
+    mozilla::MutexAutoUnlock unlock(mLock);
 
-  MOZ_ASSERT(CacheStorageService::IsOnManagementThread());
+    MOZ_ASSERT(CacheStorageService::IsOnManagementThread());
 
-  if (aOperations & Ops::FRECENCYUPDATE) {
-    #ifndef M_LN2
-    #define M_LN2 0.69314718055994530942
-    #endif
+    if (aOperations & Ops::FRECENCYUPDATE) {
+      ++mUseCount;
 
-    // Half-life is 90 days.
-    static double const half_life = 90.0 * (24 * 60 * 60);
-    // Must convert from seconds to milliseconds since PR_Now() gives usecs.
-    static double const decay = (M_LN2 / half_life) / static_cast<double>(PR_USEC_PER_SEC);
+      #ifndef M_LN2
+      #define M_LN2 0.69314718055994530942
+      #endif
 
-    double now_decay = static_cast<double>(PR_Now()) * decay;
+      // Half-life is dynamic, in seconds.
+      static double half_life = CacheObserver::HalfLifeSeconds();
+      // Must convert from seconds to milliseconds since PR_Now() gives usecs.
+      static double const decay = (M_LN2 / half_life) / static_cast<double>(PR_USEC_PER_SEC);
 
-    if (mFrecency == 0) {
-      mFrecency = now_decay;
+      double now_decay = static_cast<double>(PR_Now()) * decay;
+
+      if (mFrecency == 0) {
+        mFrecency = now_decay;
+      }
+      else {
+        // TODO: when C++11 enabled, use std::log1p(n) which is equal to log(n + 1) but
+        // more precise.
+        mFrecency = log(exp(mFrecency - now_decay) + 1) + now_decay;
+      }
+      LOG(("CacheEntry FRECENCYUPDATE [this=%p, frecency=%1.10f]", this, mFrecency));
+
+      // Because CacheFile::Set*() are not thread-safe to use (uses WeakReference that
+      // is not thread-safe) we must post to the main thread...
+      nsRefPtr<nsRunnableMethod<CacheEntry> > event =
+        NS_NewRunnableMethod(this, &CacheEntry::StoreFrecency);
+      NS_DispatchToMainThread(event);
     }
-    else {
-      // TODO: when C++11 enabled, use std::log1p(n) which is equal to log(n + 1) but
-      // more precise.
-      mFrecency = log(exp(mFrecency - now_decay) + 1) + now_decay;
+
+    if (aOperations & Ops::REGISTER) {
+      LOG(("CacheEntry REGISTER [this=%p]", this));
+
+      CacheStorageService::Self()->RegisterEntry(this);
     }
-    LOG(("CacheEntry FRECENCYUPDATE [this=%p, frecency=%1.10f]", this, mFrecency));
-  }
 
-  if (aOperations & Ops::REGISTER) {
-    LOG(("CacheEntry REGISTER [this=%p]", this));
+    if (aOperations & Ops::UNREGISTER) {
+      LOG(("CacheEntry UNREGISTER [this=%p]", this));
 
-    CacheStorageService::Self()->RegisterEntry(this);
-  }
-
-  if (aOperations & Ops::DOOM) {
-    LOG(("CacheEntry DOOM [this=%p]", this));
-
-    DoomAlreadyRemoved();
-  }
+      CacheStorageService::Self()->UnregisterEntry(this);
+    }
+  } // unlock
 
   if (aOperations & Ops::CALLBACKS) {
     LOG(("CacheEntry CALLBACKS (invoke) [this=%p]", this));
 
-    mozilla::MutexAutoLock lock(mLock);
     InvokeCallbacks();
   }
 }
 
+void CacheEntry::StoreFrecency()
+{
+  // No need for thread safety over mFrecency, it will be rewriten
+  // correctly on following invocation if broken by concurrency.
+  MOZ_ASSERT(NS_IsMainThread());
+
+  if (NS_SUCCEEDED(mFileStatus)) {
+    mFile->SetFrecency(FRECENCY2INT(mFrecency));
+  }
+}
+
 // CacheOutputCloseListener
 
 CacheOutputCloseListener::CacheOutputCloseListener(CacheEntry* aEntry)
@@ -1456,5 +1612,40 @@ NS_IMETHODIMP CacheOutputCloseListener::Run()
   return NS_OK;
 }
 
+// Memory reporting
+
+size_t CacheEntry::SizeOfExcludingThis(mozilla::MallocSizeOf mallocSizeOf) const
+{
+  size_t n = 0;
+  nsCOMPtr<nsISizeOf> sizeOf;
+
+  n += mCallbacks.SizeOfExcludingThis(mallocSizeOf);
+  if (mFile) {
+    n += mFile->SizeOfIncludingThis(mallocSizeOf);
+  }
+
+  sizeOf = do_QueryInterface(mURI);
+  if (sizeOf) {
+    n += sizeOf->SizeOfIncludingThis(mallocSizeOf);
+  }
+
+  n += mEnhanceID.SizeOfExcludingThisIfUnshared(mallocSizeOf);
+  n += mStorageID.SizeOfExcludingThisIfUnshared(mallocSizeOf);
+
+  // mDoomCallback is an arbitrary class that is probably reported elsewhere.
+  // mOutputStream is reported in mFile.
+  // mWriter is one of many handles we create, but (intentionally) not keep
+  // any reference to, so those unfortunatelly cannot be reported.  Handles are
+  // small, though.
+  // mSecurityInfo doesn't impl nsISizeOf.
+
+  return n;
+}
+
+size_t CacheEntry::SizeOfIncludingThis(mozilla::MallocSizeOf mallocSizeOf) const
+{
+  return mallocSizeOf(this) + SizeOfExcludingThis(mallocSizeOf);
+}
+
 } // net
 } // mozilla
