@@ -98,6 +98,8 @@ class DataChannelConnection: public nsITimerCallback
                              , public sigslot::has_slots<>
 #endif
 {
+  virtual ~DataChannelConnection();
+
 public:
   NS_DECL_THREADSAFE_ISUPPORTS
   NS_DECL_NSITIMERCALLBACK
@@ -105,20 +107,14 @@ public:
   class DataConnectionListener : public SupportsWeakPtr<DataConnectionListener>
   {
   public:
+    MOZ_DECLARE_REFCOUNTED_TYPENAME(DataChannelConnection::DataConnectionListener)
     virtual ~DataConnectionListener() {}
 
-    // Called when a the connection is open
-    virtual void NotifyConnection() = 0;
-
-    // Called when a the connection is lost/closed
-    virtual void NotifyClosedConnection() = 0;
-
     // Called when a new DataChannel has been opened by the other side.
     virtual void NotifyDataChannel(already_AddRefed<DataChannel> channel) = 0;
   };
 
   DataChannelConnection(DataConnectionListener *listener);
-  virtual ~DataChannelConnection();
 
   bool Init(unsigned short aPort, uint16_t aNumStreams, bool aUsingDtls);
   void Destroy(); // So we can spawn refs tied to runnables in shutdown
@@ -191,6 +187,8 @@ public:
 
   void ReadBlob(already_AddRefed<DataChannelConnection> aThis, uint16_t aStream, nsIInputStream* aBlob);
 
+  void GetStreamIds(std::vector<uint16_t>* aStreamList);
+
 protected:
   friend class DataChannelOnMessageAvailable;
   // Avoid cycles with PeerConnectionImpl
@@ -202,7 +200,7 @@ private:
 
 #ifdef SCTP_DTLS_SUPPORTED
   static void DTLSConnectThread(void *data);
-  int SendPacket(const unsigned char* data, size_t len, bool release);
+  int SendPacket(unsigned char data[], size_t len, bool release);
   void SctpDtlsInput(TransportFlow *flow, const unsigned char *data, size_t len);
   static int SctpDtlsOutput(void *addr, void *buffer, size_t length, uint8_t tos, uint8_t set_df);
 #endif
@@ -222,7 +220,7 @@ private:
 
   void DeliverQueuedData(uint16_t stream);
 
-  already_AddRefed<DataChannel> OpenFinish(already_AddRefed<DataChannel> channel) NS_WARN_UNUSED_RESULT;
+  already_AddRefed<DataChannel> OpenFinish(already_AddRefed<DataChannel>&& aChannel);
 
   void StartDefer();
   bool SendDeferredMessages();
@@ -334,7 +332,10 @@ public:
       NS_ASSERTION(mConnection,"NULL connection");
     }
 
+private:
   ~DataChannel();
+
+public:
   void Destroy(); // when we disconnect from the connection after stream RESET
 
   NS_INLINE_DECL_THREADSAFE_REFCOUNTING(DataChannel)
@@ -477,11 +478,9 @@ public:
 
   // for ON_CONNECTION/ON_DISCONNECTED
   DataChannelOnMessageAvailable(int32_t     aType,
-                                DataChannelConnection *aConnection,
-                                bool aResult = true)
+                                DataChannelConnection *aConnection)
     : mType(aType),
-      mConnection(aConnection),
-      mResult(aResult) {}
+      mConnection(aConnection) {}
 
   NS_IMETHOD Run()
   {
@@ -530,14 +529,7 @@ public:
             // important to give it an already_AddRefed pointer!
             mConnection->mListener->NotifyDataChannel(mChannel.forget());
             break;
-          case ON_CONNECTION:
-            if (mResult) {
-              mConnection->mListener->NotifyConnection();
-            }
-            // FIX - on mResult false (failure) we should do something.  Needs spec work here
-            break;
-          case ON_DISCONNECTED:
-            mConnection->mListener->NotifyClosedConnection();
+          default:
             break;
         }
         break;
@@ -557,7 +549,6 @@ private:
   nsRefPtr<DataChannelConnection>   mConnection;
   nsCString                         mData;
   int32_t                           mLen;
-  bool                              mResult;
 };
 
 }
