@@ -1,4 +1,4 @@
-/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* -*- indent-tabs-mode: nil; js-indent-level: 2 -*- */
 Components.utils.import("resource://gre/modules/NetUtil.jsm");
 
 var gIoService = Components.classes["@mozilla.org/network/io-service;1"]
@@ -8,6 +8,8 @@ var gIoService = Components.classes["@mozilla.org/network/io-service;1"]
 // Run by: cd objdir;  make -C netwerk/test/ xpcshell-tests    
 // or: cd objdir; make SOLO_FILE="test_URIs.js" -C netwerk/test/ check-one
 
+// See also test_URIs2.js.
+
 // Relevant RFCs: 1738, 1808, 2396, 3986 (newer than the code)
 // http://greenbytes.de/tech/webdav/rfc3986.html#rfc.section.5.4
 // http://greenbytes.de/tech/tc/uris/
@@ -276,295 +278,9 @@ var gTests = [
     path:    "/a/b/c/d",
     ref:     "",
     nsIURL:  true, nsINestedURI: false },
-  // relative URL testcases from http://greenbytes.de/tech/webdav/rfc3986.html#rfc.section.5.4
-  { spec:    "http://a/b/c/d;p?q",
-    relativeURI: "g:h",
-    scheme:  "g",
-    prePath: "g:",
-    path:    "h",
-    ref:     "",
-    nsIURL:  false, nsINestedURI: false },
-  { spec:    "http://a/b/c/d;p?q",
-    relativeURI: "g",
-    scheme:  "http",
-    prePath: "http://a",
-    path:    "/b/c/g",
-    ref:     "",
-    nsIURL:  true, nsINestedURI: false },
-  { spec:    "http://a/b/c/d;p?q",
-    relativeURI: "./g",
-    scheme:  "http",
-    prePath: "http://a",
-    path:    "/b/c/g",
-    ref:     "",
-    nsIURL:  true, nsINestedURI: false },
-  { spec:    "http://a/b/c/d;p?q",
-    relativeURI: "g/",
-    scheme:  "http",
-    prePath: "http://a",
-    path:    "/b/c/g/",
-    ref:     "",
-    nsIURL:  true, nsINestedURI: false },
-  { spec:    "http://a/b/c/d;p?q",
-    relativeURI: "/g",
-    scheme:  "http",
-    prePath: "http://a",
-    path:    "/g",
-    ref:     "",
-    nsIURL:  true, nsINestedURI: false },
-  { spec:    "http://a/b/c/d;p?q",
-    relativeURI: "?y",
-    scheme:  "http",
-    prePath: "http://a",
-    path:    "/b/c/d;p?y",
-    ref:     "",// fix
-    nsIURL:  true, nsINestedURI: false },
-  { spec:    "http://a/b/c/d;p?q",
-    relativeURI: "g?y",
-    scheme:  "http",
-    prePath: "http://a",
-    path:    "/b/c/g?y",
-    ref:     "",// fix
-    specIgnoringRef: "http://a/b/c/g?y",
-    hasRef:  false,
-    nsIURL:  true, nsINestedURI: false },
-  { spec:    "http://a/b/c/d;p?q",
-    relativeURI: "#s",
-    scheme:  "http",
-    prePath: "http://a",
-    path:    "/b/c/d;p?q#s",
-    ref:     "s",// fix
-    specIgnoringRef: "http://a/b/c/d;p?q",
-    hasRef:  true,
-    nsIURL:  true, nsINestedURI: false },
-  { spec:    "http://a/b/c/d;p?q",
-    relativeURI: "g#s",
-    scheme:  "http",
-    prePath: "http://a",
-    path:    "/b/c/g#s",
-    ref:     "s",
-    nsIURL:  true, nsINestedURI: false },
-  { spec:    "http://a/b/c/d;p?q",
-    relativeURI: "g?y#s",
-    scheme:  "http",
-    prePath: "http://a",
-    path:    "/b/c/g?y#s",
-    ref:     "s",
-    nsIURL:  true, nsINestedURI: false },
-  /*
-    Bug xxxxxx - we return a path of b/c/;x
-  { spec:    "http://a/b/c/d;p?q",
-    relativeURI: ";x",
-    scheme:  "http",
-    prePath: "http://a",
-    path:    "/b/c/d;x",
-    ref:     "",
-    nsIURL:  true, nsINestedURI: false },
-  */
-  { spec:    "http://a/b/c/d;p?q",
-    relativeURI: "g;x",
-    scheme:  "http",
-    prePath: "http://a",
-    path:    "/b/c/g;x",
-    ref:     "",
-    nsIURL:  true, nsINestedURI: false },
-  { spec:    "http://a/b/c/d;p?q",
-    relativeURI: "g;x?y#s",
-    scheme:  "http",
-    prePath: "http://a",
-    path:    "/b/c/g;x?y#s",
-    ref:     "s",
-    nsIURL:  true, nsINestedURI: false },
-  /*
-    Can't easily specify a relative URI of "" to the test code
-  { spec:    "http://a/b/c/d;p?q",
-    relativeURI: "",
-    scheme:  "http",
-    prePath: "http://a",
-    path:    "/b/c/d",
-    ref:     "",
-    nsIURL:  true, nsINestedURI: false },
-  */
-  { spec:    "http://a/b/c/d;p?q",
-    relativeURI: ".",
-    scheme:  "http",
-    prePath: "http://a",
-    path:    "/b/c/",
-    ref:     "",
-    nsIURL:  true, nsINestedURI: false },
-  { spec:    "http://a/b/c/d;p?q",
-    relativeURI: "./",
-    scheme:  "http",
-    prePath: "http://a",
-    path:    "/b/c/",
-    ref:     "",
-    nsIURL:  true, nsINestedURI: false },
-  { spec:    "http://a/b/c/d;p?q",
-    relativeURI: "..",
-    scheme:  "http",
-    prePath: "http://a",
-    path:    "/b/",
-    ref:     "",
-    nsIURL:  true, nsINestedURI: false },
-  { spec:    "http://a/b/c/d;p?q",
-    relativeURI: "../",
-    scheme:  "http",
-    prePath: "http://a",
-    path:    "/b/",
-    ref:     "",
-    nsIURL:  true, nsINestedURI: false },
-  { spec:    "http://a/b/c/d;p?q",
-    relativeURI: "../g",
-    scheme:  "http",
-    prePath: "http://a",
-    path:    "/b/g",
-    ref:     "",
-    nsIURL:  true, nsINestedURI: false },
-  { spec:    "http://a/b/c/d;p?q",
-    relativeURI: "../..",
-    scheme:  "http",
-    prePath: "http://a",
-    path:    "/",
-    ref:     "",
-    nsIURL:  true, nsINestedURI: false },
-  { spec:    "http://a/b/c/d;p?q",
-    relativeURI: "../../",
-    scheme:  "http",
-    prePath: "http://a",
-    path:    "/",
-    ref:     "",
-    nsIURL:  true, nsINestedURI: false },
-  { spec:    "http://a/b/c/d;p?q",
-    relativeURI: "../../g",
-    scheme:  "http",
-    prePath: "http://a",
-    path:    "/g",
-    ref:     "",
-    nsIURL:  true, nsINestedURI: false },
 
-  // abnormal examples
-  { spec:    "http://a/b/c/d;p?q",
-    relativeURI: "../../../g",
-    scheme:  "http",
-    prePath: "http://a",
-    path:    "/g",
-    ref:     "",
-    nsIURL:  true, nsINestedURI: false },
-  { spec:    "http://a/b/c/d;p?q",
-    relativeURI: "../../../../g",
-    scheme:  "http",
-    prePath: "http://a",
-    path:    "/g",
-    ref:     "",
-    nsIURL:  true, nsINestedURI: false },
-
-  // coalesce 
-  { spec:    "http://a/b/c/d;p?q",
-    relativeURI: "/./g",
-    scheme:  "http",
-    prePath: "http://a",
-    path:    "/g",
-    ref:     "",
-    nsIURL:  true, nsINestedURI: false },
-  { spec:    "http://a/b/c/d;p?q",
-    relativeURI: "/../g",
-    scheme:  "http",
-    prePath: "http://a",
-    path:    "/g",
-    ref:     "",
-    nsIURL:  true, nsINestedURI: false },
-  { spec:    "http://a/b/c/d;p?q",
-    relativeURI: "g.",
-    scheme:  "http",
-    prePath: "http://a",
-    path:    "/b/c/g.",
-    ref:     "",
-    nsIURL:  true, nsINestedURI: false },
-  { spec:    "http://a/b/c/d;p?q",
-    relativeURI: ".g",
-    scheme:  "http",
-    prePath: "http://a",
-    path:    "/b/c/.g",
-    ref:     "",
-    nsIURL:  true, nsINestedURI: false },
-  { spec:    "http://a/b/c/d;p?q",
-    relativeURI: "g..",
-    scheme:  "http",
-    prePath: "http://a",
-    path:    "/b/c/g..",
-    ref:     "",
-    nsIURL:  true, nsINestedURI: false },
-  { spec:    "http://a/b/c/d;p?q",
-    relativeURI: "..g",
-    scheme:  "http",
-    prePath: "http://a",
-    path:    "/b/c/..g",
-    ref:     "",
-    nsIURL:  true, nsINestedURI: false },
-  { spec:    "http://a/b/c/d;p?q",
-    relativeURI: ".",
-    scheme:  "http",
-    prePath: "http://a",
-    path:    "/b/c/",
-    ref:     "",
-    nsIURL:  true, nsINestedURI: false },
-  { spec:    "http://a/b/c/d;p?q",
-    relativeURI: "./../g",
-    scheme:  "http",
-    prePath: "http://a",
-    path:    "/b/g",
-    ref:     "",
-    nsIURL:  true, nsINestedURI: false },
-  { spec:    "http://a/b/c/d;p?q",
-    relativeURI: "./g/.",
-    scheme:  "http",
-    prePath: "http://a",
-    path:    "/b/c/g/",
-    ref:     "",
-    nsIURL:  true, nsINestedURI: false },
-  { spec:    "http://a/b/c/d;p?q",
-    relativeURI: "g/./h",
-    scheme:  "http",
-    prePath: "http://a",
-    path:    "/b/c/g/h",
-    ref:     "",
-    nsIURL:  true, nsINestedURI: false },
-  { spec:    "http://a/b/c/d;p?q",
-    relativeURI: "g/../h",
-    scheme:  "http",
-    prePath: "http://a",
-    path:    "/b/c/h",
-    ref:     "",// fix
-    nsIURL:  true, nsINestedURI: false },
-  { spec:    "http://a/b/c/d;p?q",
-    relativeURI: "g;x=1/./y",
-    scheme:  "http",
-    prePath: "http://a",
-    path:    "/b/c/g;x=1/y",
-    ref:     "",
-    nsIURL:  true, nsINestedURI: false },
-  { spec:    "http://a/b/c/d;p?q",
-    relativeURI: "g;x=1/../y",
-    scheme:  "http",
-    prePath: "http://a",
-    path:    "/b/c/y",
-    ref:     "",
-    nsIURL:  true, nsINestedURI: false },
-  // protocol-relative http://tools.ietf.org/html/rfc3986#section-4.2
-  { spec:    "http://www2.example.com/",
-    relativeURI: "//www3.example2.com/bar",
-    scheme:  "http",
-    prePath: "http://www3.example2.com",
-    path:    "/bar",
-    ref:     "",
-    nsIURL:  true, nsINestedURI: false },
-  { spec:    "https://www2.example.com/",
-    relativeURI: "//www3.example2.com/bar",
-    scheme:  "https",
-    prePath: "https://www3.example2.com",
-    path:    "/bar",
-    ref:     "",
-    nsIURL:  true, nsINestedURI: false },
+    // Adding more? Consider adding to test_URIs2.js instead, so that neither
+    // test runs for *too* long, risking timeouts on slow platforms.
 ];
 
 var gHashSuffixes = [
