@@ -259,9 +259,9 @@ NS_IMETHODIMP_(void)
 nsParser::SetCommand(const char* aCommand)
 {
   mCommandStr.Assign(aCommand);
-  if (mCommandStr.Equals("view-source")) {
+  if (mCommandStr.EqualsLiteral("view-source")) {
     mCommand = eViewSource;
-  } else if (mCommandStr.Equals("view-fragment")) {
+  } else if (mCommandStr.EqualsLiteral("view-fragment")) {
     mCommand = eViewFragment;
   } else {
     mCommand = eViewNormal;
@@ -354,13 +354,13 @@ static int32_t
 ParsePS(const nsString& aBuffer, int32_t aIndex)
 {
   for (;;) {
-    PRUnichar ch = aBuffer.CharAt(aIndex);
-    if ((ch == PRUnichar(' ')) || (ch == PRUnichar('\t')) ||
-        (ch == PRUnichar('\n')) || (ch == PRUnichar('\r'))) {
+    char16_t ch = aBuffer.CharAt(aIndex);
+    if ((ch == char16_t(' ')) || (ch == char16_t('\t')) ||
+        (ch == char16_t('\n')) || (ch == char16_t('\r'))) {
       ++aIndex;
-    } else if (ch == PRUnichar('-')) {
+    } else if (ch == char16_t('-')) {
       int32_t tmpIndex;
-      if (aBuffer.CharAt(aIndex+1) == PRUnichar('-') &&
+      if (aBuffer.CharAt(aIndex+1) == char16_t('-') &&
           kNotFound != (tmpIndex=aBuffer.Find("--",false,aIndex+2,-1))) {
         aIndex = tmpIndex + 2;
       } else {
@@ -393,8 +393,8 @@ ParseDocTypeDecl(const nsString &aBuffer,
   do {
     theIndex = aBuffer.FindChar('<', theIndex);
     if (theIndex == kNotFound) break;
-    PRUnichar nextChar = aBuffer.CharAt(theIndex+1);
-    if (nextChar == PRUnichar('!')) {
+    char16_t nextChar = aBuffer.CharAt(theIndex+1);
+    if (nextChar == char16_t('!')) {
       int32_t tmpIndex = theIndex + 2;
       if (kNotFound !=
           (theIndex=aBuffer.Find("DOCTYPE", true, tmpIndex, 0))) {
@@ -404,7 +404,7 @@ ParseDocTypeDecl(const nsString &aBuffer,
       }
       theIndex = ParsePS(aBuffer, tmpIndex);
       theIndex = aBuffer.FindChar('>', theIndex);
-    } else if (nextChar == PRUnichar('?')) {
+    } else if (nextChar == char16_t('?')) {
       theIndex = aBuffer.FindChar('>', theIndex);
     } else {
       break;
@@ -431,8 +431,8 @@ ParseDocTypeDecl(const nsString &aBuffer,
     // Now find the beginning and end of the public identifier
     // and the system identifier (if present).
 
-    PRUnichar lit = aBuffer.CharAt(theIndex);
-    if ((lit != PRUnichar('\"')) && (lit != PRUnichar('\'')))
+    char16_t lit = aBuffer.CharAt(theIndex);
+    if ((lit != char16_t('\"')) && (lit != char16_t('\'')))
       return false;
 
     // Start is the first character, excluding the quote, and End is
@@ -443,15 +443,15 @@ ParseDocTypeDecl(const nsString &aBuffer,
     if (kNotFound == PublicIDEnd)
       return false;
     theIndex = ParsePS(aBuffer, PublicIDEnd + 1);
-    PRUnichar next = aBuffer.CharAt(theIndex);
-    if (next == PRUnichar('>')) {
+    char16_t next = aBuffer.CharAt(theIndex);
+    if (next == char16_t('>')) {
       // There was a public identifier, but no system
       // identifier,
       // so do nothing.
       // This is needed to avoid the else at the end, and it's
       // also the most common case.
-    } else if ((next == PRUnichar('\"')) ||
-               (next == PRUnichar('\''))) {
+    } else if ((next == char16_t('\"')) ||
+               (next == char16_t('\''))) {
       // We found a system identifier.
       *aResultFlags |= PARSE_DTD_HAVE_SYSTEM_ID;
       int32_t SystemIDStart = theIndex + 1;
@@ -460,7 +460,7 @@ ParseDocTypeDecl(const nsString &aBuffer,
         return false;
       aSystemID =
         Substring(aBuffer, SystemIDStart, SystemIDEnd - SystemIDStart);
-    } else if (next == PRUnichar('[')) {
+    } else if (next == char16_t('[')) {
       // We found an internal subset.
       *aResultFlags |= PARSE_DTD_HAVE_INTERNAL_SUBSET;
     } else {
@@ -480,8 +480,8 @@ ParseDocTypeDecl(const nsString &aBuffer,
       *aResultFlags |= PARSE_DTD_HAVE_SYSTEM_ID;
 
       theIndex = ParsePS(aBuffer, tmpIndex+6);
-      PRUnichar next = aBuffer.CharAt(theIndex);
-      if (next != PRUnichar('\"') && next != PRUnichar('\''))
+      char16_t next = aBuffer.CharAt(theIndex);
+      if (next != char16_t('\"') && next != char16_t('\''))
         return false;
 
       int32_t SystemIDStart = theIndex + 1;
@@ -494,10 +494,10 @@ ParseDocTypeDecl(const nsString &aBuffer,
       theIndex = ParsePS(aBuffer, SystemIDEnd + 1);
     }
 
-    PRUnichar nextChar = aBuffer.CharAt(theIndex);
-    if (nextChar == PRUnichar('['))
+    char16_t nextChar = aBuffer.CharAt(theIndex);
+    if (nextChar == char16_t('['))
       *aResultFlags |= PARSE_DTD_HAVE_INTERNAL_SUBSET;
-    else if (nextChar != PRUnichar('>'))
+    else if (nextChar != char16_t('>'))
       return false;
   }
   return true;
@@ -662,7 +662,7 @@ DetermineHTMLParseMode(const nsString& aBuffer,
 
       // Special hack for IBM's custom DOCTYPE.
       if (!(resultFlags & PARSE_DTD_HAVE_INTERNAL_SUBSET) &&
-          sysIDUCS2 == NS_LITERAL_STRING(
+          sysIDUCS2.EqualsLiteral(
                "http://www.ibm.com/data/dtd/v11/ibmxhtml1-transitional.dtd")) {
         aParseMode = eDTDMode_quirks;
         aDocType = eHTML_Quirks;
@@ -850,7 +850,7 @@ nsParser::WillBuildModel(nsString& aFilename)
 
   if (eDTDMode_unknown == mParserContext->mDTDMode ||
       eDTDMode_autodetect == mParserContext->mDTDMode) {
-    PRUnichar buf[1025];
+    char16_t buf[1025];
     nsFixedString theBuffer(buf, 1024, 0);
 
     // Grab 1024 characters, starting at the first non-whitespace
@@ -1328,15 +1328,15 @@ nsParser::ParseFragment(const nsAString& aSourceBuffer,
   mFlags &= ~NS_PARSER_FLAG_OBSERVERS_ENABLED;
 
   for (theIndex = 0; theIndex < theCount; theIndex++) {
-    theContext.AppendLiteral("<");
+    theContext.Append('<');
     theContext.Append(aTagStack[theCount - theIndex - 1]);
-    theContext.AppendLiteral(">");
+    theContext.Append('>');
   }
 
   if (theCount == 0) {
     // Ensure that the buffer is not empty. Because none of the DTDs care
     // about leading whitespace, this doesn't change the result.
-    theContext.AssignLiteral(" ");
+    theContext.Assign(' ');
   }
 
   // First, parse the context to build up the DTD's tag stack. Note that we
@@ -1385,14 +1385,14 @@ nsParser::ParseFragment(const nsAString& aSourceBuffer,
 
         nsString& thisTag = aTagStack[theIndex];
         // was there an xmlns=?
-        int32_t endOfTag = thisTag.FindChar(PRUnichar(' '));
+        int32_t endOfTag = thisTag.FindChar(char16_t(' '));
         if (endOfTag == -1) {
           endContext.Append(thisTag);
         } else {
           endContext.Append(Substring(thisTag,0,endOfTag));
         }
 
-        endContext.AppendLiteral(">");
+        endContext.Append('>');
       }
 
       result = Parse(endContext,
