@@ -9,13 +9,14 @@ import logging
 import os
 import traceback
 import sys
+import time
 
 from mach.mixin.logging import LoggingMixin
 
 import mozpack.path as mozpath
 import manifestparser
-
-from mozpack.files import FileFinder
+import reftest
+import mozinfo
 
 from .data import (
     ConfigFileSubstitution,
@@ -25,17 +26,22 @@ from .data import (
     GeneratedEventWebIDLFile,
     GeneratedInclude,
     GeneratedWebIDLFile,
+    ExampleWebIDLInterface,
     HeaderFileSubstitution,
     HostProgram,
     HostSimpleProgram,
     InstallationTarget,
     IPDLFile,
+    JARManifest,
+    JavaScriptModules,
     LibraryDefinition,
     LocalInclude,
+    PerSourceFlag,
     PreprocessedTestWebIDLFile,
     PreprocessedWebIDLFile,
     Program,
     ReaderSummary,
+    Resources,
     SandboxWrapped,
     SimpleProgram,
     TestWebIDLFile,
@@ -50,6 +56,8 @@ from .reader import (
     SandboxValidationError,
 )
 
+from .gyp_reader import GypSandbox
+
 
 class TreeMetadataEmitter(LoggingMixin):
     """Converts the executed mozbuild files into data structures.
@@ -64,12 +72,16 @@ class TreeMetadataEmitter(LoggingMixin):
 
         self.config = config
 
-        # TODO add mozinfo into config or somewhere else.
-        mozinfo_path = os.path.join(config.topobjdir, 'mozinfo.json')
-        if os.path.exists(mozinfo_path):
-            self.mozinfo = json.load(open(mozinfo_path, 'rt'))
-        else:
-            self.mozinfo = {}
+        mozinfo.find_and_update_from_json(config.topobjdir)
+
+        # Python 2.6 doesn't allow unicode keys to be used for keyword
+        # arguments. This gross hack works around the problem until we
+        # rid ourselves of 2.6.
+        self.info = {}
+        for k, v in mozinfo.info.items():
+            if isinstance(k, unicode):
+                k = k.encode('ascii')
+            self.info[k] = v
 
         self._libs = {}
         self._final_libs = []
@@ -81,26 +93,44 @@ class TreeMetadataEmitter(LoggingMixin):
         typically fed into this function.
         """
         file_count = 0
-        execution_time = 0.0
+        sandbox_execution_time = 0.0
+        emitter_time = 0.0
         sandboxes = {}
 
+        def emit_objs(objs):
+            for o in objs:
+                yield o
+                if not o._ack:
+                    raise Exception('Unhandled object of type %s' % type(o))
+
         for out in output:
-            if isinstance(out, MozbuildSandbox):
+            if isinstance(out, (MozbuildSandbox, GypSandbox)):
                 # Keep all sandboxes around, we will need them later.
                 sandboxes[out['OBJDIR']] = out
 
-                for o in self.emit_from_sandbox(out):
-                    yield o
-                    if not o._ack:
-                        raise Exception('Unhandled object of type %s' % type(o))
+                start = time.time()
+                # We need to expand the generator for the timings to work.
+                objs = list(self.emit_from_sandbox(out))
+                emitter_time += time.time() - start
+
+                for o in emit_objs(objs): yield o
 
                 # Update the stats.
                 file_count += len(out.all_paths)
-                execution_time += out.execution_time
+                sandbox_execution_time += out.execution_time
 
             else:
-                raise Exception('Unhandled output type: %s' % out)
+                raise Exception('Unhandled output type: %s' % type(out))
 
+        start = time.time()
+        objs = list(self._emit_libs_derived(sandboxes))
+        emitter_time += time.time() - start
+
+        for o in emit_objs(objs): yield o
+
+        yield ReaderSummary(file_count, sandbox_execution_time, emitter_time)
+
+    def _emit_libs_derived(self, sandboxes):
         for objdir, libname, final_lib in self._final_libs:
             if final_lib not in self._libs:
                 raise Exception('FINAL_LIBRARY in %s (%s) does not match any '
@@ -136,8 +166,6 @@ class TreeMetadataEmitter(LoggingMixin):
                         yield passthru
                 yield libdef
 
-        yield ReaderSummary(file_count, execution_time)
-
     def emit_from_sandbox(self, sandbox):
         """Convert a MozbuildSandbox to tree metadata objects.
 
@@ -163,7 +191,11 @@ class TreeMetadataEmitter(LoggingMixin):
 
         if sandbox['XPIDL_SOURCES'] and not xpidl_module:
             raise SandboxValidationError('XPIDL_MODULE must be defined if '
-                'XPIDL_SOURCES is defined.')
+                'XPIDL_SOURCES is defined.', sandbox)
+
+        if xpidl_module and not sandbox['XPIDL_SOURCES']:
+            raise SandboxValidationError('XPIDL_MODULE cannot be defined '
+                'unless there are XPIDL_SOURCES', sandbox)
 
         if sandbox['XPIDL_SOURCES'] and sandbox['NO_DIST_INSTALL']:
             self.log(logging.WARN, 'mozbuild_warning', dict(
@@ -176,51 +208,65 @@ class TreeMetadataEmitter(LoggingMixin):
 
         for symbol in ('SOURCES', 'HOST_SOURCES', 'UNIFIED_SOURCES'):
             for src in (sandbox[symbol] or []):
-                if not os.path.exists(os.path.join(sandbox['SRCDIR'], src)):
+                if not os.path.exists(mozpath.join(sandbox['SRCDIR'], src)):
                     raise SandboxValidationError('Reference to a file that '
-                        'doesn\'t exist in %s (%s) in %s'
-                        % (symbol, src, sandbox['RELATIVEDIR']))
-
-        if sandbox.get('LIBXUL_LIBRARY') and sandbox.get('FORCE_STATIC_LIB'):
-            raise SandboxValidationError('LIBXUL_LIBRARY implies FORCE_STATIC_LIB')
+                        'doesn\'t exist in %s (%s)'
+                        % (symbol, src), sandbox)
 
         # Proxy some variables as-is until we have richer classes to represent
         # them. We should aim to keep this set small because it violates the
         # desired abstraction of the build definition away from makefiles.
         passthru = VariablePassthru(sandbox)
-        varmap = dict(
-            # Makefile.in : moz.build
-            ANDROID_GENERATED_RESFILES='ANDROID_GENERATED_RESFILES',
-            ANDROID_RES_DIRS='ANDROID_RES_DIRS',
-            CPP_UNIT_TESTS='CPP_UNIT_TESTS',
-            EXPORT_LIBRARY='EXPORT_LIBRARY',
-            EXTRA_COMPONENTS='EXTRA_COMPONENTS',
-            EXTRA_JS_MODULES='EXTRA_JS_MODULES',
-            EXTRA_PP_COMPONENTS='EXTRA_PP_COMPONENTS',
-            EXTRA_PP_JS_MODULES='EXTRA_PP_JS_MODULES',
-            FAIL_ON_WARNINGS='FAIL_ON_WARNINGS',
-            FILES_PER_UNIFIED_FILE='FILES_PER_UNIFIED_FILE',
-            FORCE_SHARED_LIB='FORCE_SHARED_LIB',
-            FORCE_STATIC_LIB='FORCE_STATIC_LIB',
-            GENERATED_FILES='GENERATED_FILES',
-            HOST_LIBRARY_NAME='HOST_LIBRARY_NAME',
-            IS_COMPONENT='IS_COMPONENT',
-            JS_MODULES_PATH='JS_MODULES_PATH',
-            LIBS='LIBS',
-            LIBXUL_LIBRARY='LIBXUL_LIBRARY',
-            MSVC_ENABLE_PGO='MSVC_ENABLE_PGO',
-            NO_DIST_INSTALL='NO_DIST_INSTALL',
-            OS_LIBS='OS_LIBS',
-            SDK_LIBRARY='SDK_LIBRARY',
-        )
-        for mak, moz in varmap.items():
-            if sandbox[moz]:
-                passthru.variables[mak] = sandbox[moz]
+        varlist = [
+            'ANDROID_GENERATED_RESFILES',
+            'ANDROID_RES_DIRS',
+            'CPP_UNIT_TESTS',
+            'DISABLE_STL_WRAPPING',
+            'EXTRA_ASSEMBLER_FLAGS',
+            'EXTRA_COMPILE_FLAGS',
+            'EXTRA_COMPONENTS',
+            'EXTRA_DSO_LDOPTS',
+            'EXTRA_JS_MODULES',
+            'EXTRA_PP_COMPONENTS',
+            'EXTRA_PP_JS_MODULES',
+            'FAIL_ON_WARNINGS',
+            'FILES_PER_UNIFIED_FILE',
+            'FORCE_SHARED_LIB',
+            'FORCE_STATIC_LIB',
+            'USE_STATIC_LIBS',
+            'GENERATED_FILES',
+            'HOST_LIBRARY_NAME',
+            'IS_COMPONENT',
+            'IS_GYP_DIR',
+            'JS_MODULES_PATH',
+            'LIBS',
+            'MSVC_ENABLE_PGO',
+            'NO_DIST_INSTALL',
+            'OS_LIBS',
+            'RCFILE',
+            'RESFILE',
+            'RCINCLUDE',
+            'DEFFILE',
+            'SDK_LIBRARY',
+            'WIN32_EXE_LDFLAGS',
+            'LD_VERSION_SCRIPT',
+        ]
+        for v in varlist:
+            if v in sandbox and sandbox[v]:
+                passthru.variables[v] = sandbox[v]
+
+        for v in ['CFLAGS', 'CXXFLAGS', 'CMFLAGS', 'CMMFLAGS', 'LDFLAGS']:
+            if v in sandbox and sandbox[v]:
+                passthru.variables['MOZBUILD_' + v] = sandbox[v]
 
         # NO_VISIBILITY_FLAGS is slightly different
         if sandbox['NO_VISIBILITY_FLAGS']:
             passthru.variables['VISIBILITY_FLAGS'] = ''
 
+        if sandbox['DELAYLOAD_DLLS']:
+            passthru.variables['DELAYLOAD_LDFLAGS'] = [('-DELAYLOAD:%s' % dll) for dll in sandbox['DELAYLOAD_DLLS']]
+            passthru.variables['USE_DELAYIMP'] = True
+
         varmap = dict(
             SOURCES={
                 '.s': 'ASFILES',
@@ -230,6 +276,7 @@ class TreeMetadataEmitter(LoggingMixin):
                 '.mm': 'CMMSRCS',
                 '.cc': 'CPPSRCS',
                 '.cpp': 'CPPSRCS',
+                '.cxx': 'CPPSRCS',
                 '.S': 'SSRCS',
             },
             HOST_SOURCES={
@@ -237,21 +284,24 @@ class TreeMetadataEmitter(LoggingMixin):
                 '.mm': 'HOST_CMMSRCS',
                 '.cc': 'HOST_CPPSRCS',
                 '.cpp': 'HOST_CPPSRCS',
+                '.cxx': 'HOST_CPPSRCS',
             },
             UNIFIED_SOURCES={
                 '.c': 'UNIFIED_CSRCS',
                 '.mm': 'UNIFIED_CMMSRCS',
                 '.cc': 'UNIFIED_CPPSRCS',
                 '.cpp': 'UNIFIED_CPPSRCS',
+                '.cxx': 'UNIFIED_CPPSRCS',
             }
         )
         varmap.update(dict(('GENERATED_%s' % k, v) for k, v in varmap.items()
                            if k in ('SOURCES', 'UNIFIED_SOURCES')))
         for variable, mapping in varmap.items():
             for f in sandbox[variable]:
-                ext = os.path.splitext(f)[1]
+                ext = mozpath.splitext(f)[1]
                 if ext not in mapping:
-                    raise SandboxValidationError('%s has an unknown file type in %s' % (f, sandbox['RELATIVEDIR']))
+                    raise SandboxValidationError(
+                        '%s has an unknown file type.' % f, sandbox)
                 l = passthru.variables.setdefault(mapping[ext], [])
                 l.append(f)
                 if variable.startswith('GENERATED_'):
@@ -263,11 +313,17 @@ class TreeMetadataEmitter(LoggingMixin):
         no_pgo_sources = [f for f in sources if sources[f].no_pgo]
         if no_pgo:
             if no_pgo_sources:
-                raise SandboxValidationError('NO_PGO and SOURCES[...].no_pgo cannot be set at the same time')
+                raise SandboxValidationError('NO_PGO and SOURCES[...].no_pgo '
+                    'cannot be set at the same time', sandbox)
             passthru.variables['NO_PROFILE_GUIDED_OPTIMIZE'] = no_pgo
         if no_pgo_sources:
             passthru.variables['NO_PROFILE_GUIDED_OPTIMIZE'] = no_pgo_sources
 
+        sources_with_flags = [f for f in sources if sources[f].flags]
+        for f in sources_with_flags:
+            ext = mozpath.splitext(f)[1]
+            yield PerSourceFlag(sandbox, f, sources[f].flags)
+
         exports = sandbox.get('EXPORTS')
         if exports:
             yield Exports(sandbox, exports,
@@ -277,6 +333,10 @@ class TreeMetadataEmitter(LoggingMixin):
         if defines:
             yield Defines(sandbox, defines)
 
+        resources = sandbox.get('RESOURCE_FILES')
+        if resources:
+            yield Resources(sandbox, resources, defines)
+
         program = sandbox.get('PROGRAM')
         if program:
             yield Program(sandbox, program, sandbox['CONFIG']['BIN_SUFFIX'])
@@ -291,6 +351,10 @@ class TreeMetadataEmitter(LoggingMixin):
         for program in sandbox['HOST_SIMPLE_PROGRAMS']:
             yield HostSimpleProgram(sandbox, program, sandbox['CONFIG']['HOST_BIN_SUFFIX'])
 
+        test_js_modules = sandbox.get('TESTING_JS_MODULES')
+        if test_js_modules:
+            yield JavaScriptModules(sandbox, test_js_modules, 'testing')
+
         simple_lists = [
             ('GENERATED_EVENTS_WEBIDL_FILES', GeneratedEventWebIDLFile),
             ('GENERATED_WEBIDL_FILES', GeneratedWebIDLFile),
@@ -301,6 +365,7 @@ class TreeMetadataEmitter(LoggingMixin):
             ('PREPROCESSED_WEBIDL_FILES', PreprocessedWebIDLFile),
             ('TEST_WEBIDL_FILES', TestWebIDLFile),
             ('WEBIDL_FILES', WebIDLFile),
+            ('WEBIDL_EXAMPLE_INTERFACES', ExampleWebIDLInterface),
         ]
         for sandbox_var, klass in simple_lists:
             for name in sandbox.get(sandbox_var, []):
@@ -320,11 +385,21 @@ class TreeMetadataEmitter(LoggingMixin):
                 LibraryDefinition(sandbox, libname)
 
         if final_lib:
-            if sandbox.get('FORCE_STATIC_LIB'):
-                raise SandboxValidationError('FINAL_LIBRARY implies FORCE_STATIC_LIB')
+            if isinstance(sandbox, MozbuildSandbox) and \
+                    sandbox.get('FORCE_STATIC_LIB'):
+                raise SandboxValidationError(
+                    'FINAL_LIBRARY implies FORCE_STATIC_LIB', sandbox)
             self._final_libs.append((sandbox['OBJDIR'], libname, final_lib))
             passthru.variables['FORCE_STATIC_LIB'] = True
 
+        soname = sandbox.get('SONAME')
+        if soname:
+            if not sandbox.get('FORCE_SHARED_LIB'):
+                raise SandboxValidationError(
+                    'SONAME applicable only for shared libraries', sandbox)
+            else:
+                passthru.variables['SONAME'] = soname
+
         # While there are multiple test manifests, the behavior is very similar
         # across them. We enforce this by having common handling of all
         # manifests and outputting a single class type with the differences
@@ -345,14 +420,14 @@ class TreeMetadataEmitter(LoggingMixin):
         # harness can yet deal with test filtering. Once all harnesses can do
         # this, this feature can be dropped.
         test_manifests = dict(
-            A11Y=('a11y', 'testing/mochitest/a11y', True),
-            BROWSER_CHROME=('browser-chrome', 'testing/mochitest/browser', True),
-            METRO_CHROME=('metro-chrome', 'testing/mochitest/metro', True),
-            MOCHITEST=('mochitest', 'testing/mochitest/tests', True),
-            MOCHITEST_CHROME=('chrome', 'testing/mochitest/chrome', True),
-            MOCHITEST_WEBAPPRT_CHROME=('webapprt-chrome', 'testing/mochitest/webapprtChrome', True),
-            WEBRTC_SIGNALLING_TEST=('steeplechase', 'steeplechase', True),
-            XPCSHELL_TESTS=('xpcshell', 'xpcshell', False),
+            A11Y=('a11y', 'testing/mochitest', 'a11y', True),
+            BROWSER_CHROME=('browser-chrome', 'testing/mochitest', 'browser', True),
+            METRO_CHROME=('metro-chrome', 'testing/mochitest', 'metro', True),
+            MOCHITEST=('mochitest', 'testing/mochitest', 'tests', True),
+            MOCHITEST_CHROME=('chrome', 'testing/mochitest', 'chrome', True),
+            MOCHITEST_WEBAPPRT_CHROME=('webapprt-chrome', 'testing/mochitest', 'webapprtChrome', True),
+            WEBRTC_SIGNALLING_TEST=('steeplechase', 'steeplechase', '.', True),
+            XPCSHELL_TESTS=('xpcshell', 'xpcshell', '.', False),
         )
 
         for prefix, info in test_manifests.items():
@@ -360,9 +435,35 @@ class TreeMetadataEmitter(LoggingMixin):
                 for obj in self._process_test_manifest(sandbox, info, path):
                     yield obj
 
+        for flavor in ('crashtest', 'reftest'):
+            for path in sandbox.get('%s_MANIFESTS' % flavor.upper(), []):
+                for obj in self._process_reftest_manifest(sandbox, flavor, path):
+                    yield obj
+
+        jar_manifests = sandbox.get('JAR_MANIFESTS', [])
+        if len(jar_manifests) > 1:
+            raise SandboxValidationError('While JAR_MANIFESTS is a list, '
+                'it is currently limited to one value.', sandbox)
+
+        for path in jar_manifests:
+            yield JARManifest(sandbox, mozpath.join(sandbox['SRCDIR'], path))
+
+        # Temporary test to look for jar.mn files that creep in without using
+        # the new declaration. Before, we didn't require jar.mn files to
+        # declared anywhere (they were discovered). This will detect people
+        # relying on the old behavior.
+        if os.path.exists(os.path.join(sandbox['SRCDIR'], 'jar.mn')):
+            if 'jar.mn' not in jar_manifests:
+                raise SandboxValidationError('A jar.mn exists but it '
+                    'is not referenced in the moz.build file. '
+                    'Please define JAR_MANIFESTS.', sandbox)
+
         for name, jar in sandbox.get('JAVA_JAR_TARGETS', {}).items():
             yield SandboxWrapped(sandbox, jar)
 
+        for name, data in sandbox.get('ANDROID_ECLIPSE_PROJECT_TARGETS', {}).items():
+            yield SandboxWrapped(sandbox, data)
+
         if passthru.variables:
             yield passthru
 
@@ -371,41 +472,56 @@ class TreeMetadataEmitter(LoggingMixin):
             path = path[1:]
 
         sub = cls(sandbox)
-        sub.input_path = os.path.join(sandbox['SRCDIR'], '%s.in' % path)
-        sub.output_path = os.path.join(sandbox['OBJDIR'], path)
+        sub.input_path = mozpath.join(sandbox['SRCDIR'], '%s.in' % path)
+        sub.output_path = mozpath.join(sandbox['OBJDIR'], path)
         sub.relpath = path
 
         return sub
 
     def _process_test_manifest(self, sandbox, info, manifest_path):
-        flavor, install_prefix, filter_inactive = info
+        flavor, install_root, install_subdir, filter_inactive = info
 
-        manifest_path = os.path.normpath(manifest_path)
+        manifest_path = mozpath.normpath(manifest_path)
         path = mozpath.normpath(mozpath.join(sandbox['SRCDIR'], manifest_path))
         manifest_dir = mozpath.dirname(path)
         manifest_reldir = mozpath.dirname(mozpath.relpath(path,
             sandbox['TOPSRCDIR']))
+        install_prefix = mozpath.join(install_root, install_subdir)
 
         try:
             m = manifestparser.TestManifest(manifests=[path], strict=True)
-
-            if not m.tests:
+            defaults = m.manifest_defaults[os.path.normpath(path)]
+            if not m.tests and not 'support-files' in defaults:
                 raise SandboxValidationError('Empty test manifest: %s'
-                    % path)
+                    % path, sandbox)
 
             obj = TestManifest(sandbox, path, m, flavor=flavor,
                 install_prefix=install_prefix,
                 relpath=mozpath.join(manifest_reldir, mozpath.basename(path)),
-                dupe_manifest='dupe-manifest' in m.tests[0])
+                dupe_manifest='dupe-manifest' in defaults)
 
             filtered = m.tests
 
             if filter_inactive:
-                filtered = m.active_tests(disabled=False, **self.mozinfo)
+                # We return tests that don't exist because we want manifests
+                # defining tests that don't exist to result in error.
+                filtered = m.active_tests(exists=False, disabled=True,
+                    **self.info)
+
+                missing = [t['name'] for t in filtered if not os.path.exists(t['path'])]
+                if missing:
+                    raise SandboxValidationError('Test manifest (%s) lists '
+                        'test that does not exist: %s' % (
+                        path, ', '.join(missing)), sandbox)
 
             out_dir = mozpath.join(install_prefix, manifest_reldir)
+            if 'install-to-subdir' in defaults:
+                # This is terrible, but what are you going to do?
+                out_dir = mozpath.join(out_dir, defaults['install-to-subdir'])
+                obj.manifest_obj_relpath = mozpath.join(manifest_reldir,
+                                                        defaults['install-to-subdir'],
+                                                        mozpath.basename(path))
 
-            finder = FileFinder(base=manifest_dir, find_executables=False)
 
             # "head" and "tail" lists.
             # All manifests support support-files.
@@ -416,13 +532,7 @@ class TreeMetadataEmitter(LoggingMixin):
             extras = (('head', set()),
                       ('tail', set()),
                       ('support-files', set()))
-
-            for test in filtered:
-                obj.tests.append(test)
-
-                obj.installs[mozpath.normpath(test['path'])] = \
-                    mozpath.join(out_dir, test['relpath'])
-
+            def process_support_files(test):
                 for thing, seen in extras:
                     value = test.get(thing, '')
                     if value in seen:
@@ -431,49 +541,77 @@ class TreeMetadataEmitter(LoggingMixin):
                     for pattern in value.split():
                         # We only support globbing on support-files because
                         # the harness doesn't support * for head and tail.
-                        #
-                        # While we could feed everything through the finder, we
-                        # don't because we want explicitly listed files that
-                        # no longer exist to raise an error. The finder is also
-                        # slower than simple lookup.
                         if '*' in pattern and thing == 'support-files':
-                            paths = [f[0] for f in finder.find(pattern)]
-                            if not paths:
-                                raise SandboxValidationError('%s support-files '
-                                    'wildcard in %s returns no results.' % (
-                                    pattern, path))
-
-                            for f in paths:
-                                full = mozpath.normpath(mozpath.join(manifest_dir, f))
-                                obj.installs[full] = mozpath.join(out_dir, f)
-
+                            obj.pattern_installs.append(
+                                (manifest_dir, pattern, out_dir))
+                        # "absolute" paths identify files that are to be
+                        # placed in the install_root directory (no globs)
+                        elif pattern[0] == '/':
+                            full = mozpath.normpath(mozpath.join(manifest_dir,
+                                mozpath.basename(pattern)))
+                            obj.installs[full] = (mozpath.join(install_root,
+                                pattern[1:]), False)
                         else:
                             full = mozpath.normpath(mozpath.join(manifest_dir,
                                 pattern))
-                            # Only install paths in our directory. This
-                            # rule is somewhat arbitrary and could be lifted.
+
+                            dest_path = mozpath.join(out_dir, pattern)
+
+                            # If the path resolves to a different directory
+                            # tree, we take special behavior depending on the
+                            # entry type.
                             if not full.startswith(manifest_dir):
-                                continue
+                                # If it's a support file, we install the file
+                                # into the current destination directory.
+                                # This implementation makes installing things
+                                # with custom prefixes impossible. If this is
+                                # needed, we can add support for that via a
+                                # special syntax later.
+                                if thing == 'support-files':
+                                    dest_path = mozpath.join(out_dir,
+                                        os.path.basename(pattern))
+                                # If it's not a support file, we ignore it.
+                                # This preserves old behavior so things like
+                                # head files doesn't get installed multiple
+                                # times.
+                                else:
+                                    continue
 
-                            obj.installs[full] = mozpath.join(out_dir, pattern)
+                            obj.installs[full] = (mozpath.normpath(dest_path),
+                                False)
 
-            # We also copy the manifest into the output directory.
-            out_path = mozpath.join(out_dir, os.path.basename(manifest_path))
-            obj.installs[path] = out_path
+            for test in filtered:
+                obj.tests.append(test)
+
+                obj.installs[mozpath.normpath(test['path'])] = \
+                    (mozpath.join(out_dir, test['relpath']), True)
+
+                process_support_files(test)
+
+            if not filtered:
+                # If there are no tests, look for support-files under DEFAULT.
+                process_support_files(defaults)
+
+            # We also copy manifests into the output directory,
+            # including manifests from [include:foo] directives.
+            for mpath in m.manifests():
+                mpath = mozpath.normpath(mpath)
+                out_path = mozpath.join(out_dir, mozpath.basename(mpath))
+                obj.installs[mpath] = (out_path, False)
 
             # Some manifests reference files that are auto generated as
             # part of the build or shouldn't be installed for some
             # reason. Here, we prune those files from the install set.
             # FUTURE we should be able to detect autogenerated files from
             # other build metadata. Once we do that, we can get rid of this.
-            for f in m.tests[0].get('generated-files', '').split():
+            for f in defaults.get('generated-files', '').split():
                 # We re-raise otherwise the stack trace isn't informative.
                 try:
                     del obj.installs[mozpath.join(manifest_dir, f)]
                 except KeyError:
                     raise SandboxValidationError('Error processing test '
                         'manifest %s: entry in generated-files not present '
-                        'elsewhere in manifest: %s' % (path, f))
+                        'elsewhere in manifest: %s' % (path, f), sandbox)
 
                 obj.external_installs.add(mozpath.join(out_dir, f))
 
@@ -481,7 +619,40 @@ class TreeMetadataEmitter(LoggingMixin):
         except (AssertionError, Exception):
             raise SandboxValidationError('Error processing test '
                 'manifest file %s: %s' % (path,
-                    '\n'.join(traceback.format_exception(*sys.exc_info()))))
+                    '\n'.join(traceback.format_exception(*sys.exc_info()))),
+                sandbox)
+
+    def _process_reftest_manifest(self, sandbox, flavor, manifest_path):
+        manifest_path = mozpath.normpath(manifest_path)
+        manifest_full_path = mozpath.normpath(mozpath.join(
+            sandbox['SRCDIR'], manifest_path))
+        manifest_reldir = mozpath.dirname(mozpath.relpath(manifest_full_path,
+            sandbox['TOPSRCDIR']))
+
+        manifest = reftest.ReftestManifest()
+        manifest.load(manifest_full_path)
+
+        # reftest manifests don't come from manifest parser. But they are
+        # similar enough that we can use the same emitted objects. Note
+        # that we don't perform any installs for reftests.
+        obj = TestManifest(sandbox, manifest_full_path, manifest,
+                flavor=flavor, install_prefix='%s/' % flavor,
+                relpath=mozpath.join(manifest_reldir,
+                    mozpath.basename(manifest_path)))
+
+        for test in sorted(manifest.files):
+            obj.tests.append({
+                'path': test,
+                'here': mozpath.dirname(test),
+                'manifest': manifest_full_path,
+                'name': mozpath.basename(test),
+                'head': '',
+                'tail': '',
+                'support-files': '',
+                'subsuite': '',
+            })
+
+        yield obj
 
     def _emit_directory_traversal_from_sandbox(self, sandbox):
         o = DirectoryTraversal(sandbox)
@@ -490,8 +661,6 @@ class TreeMetadataEmitter(LoggingMixin):
         o.tool_dirs = sandbox.get('TOOL_DIRS', [])
         o.test_dirs = sandbox.get('TEST_DIRS', [])
         o.test_tool_dirs = sandbox.get('TEST_TOOL_DIRS', [])
-        o.external_make_dirs = sandbox.get('EXTERNAL_MAKE_DIRS', [])
-        o.parallel_external_make_dirs = sandbox.get('PARALLEL_EXTERNAL_MAKE_DIRS', [])
         o.is_tool_dir = sandbox.get('IS_TOOL_DIR', False)
         o.affected_tiers = sandbox.get_affected_tiers()
 
