@@ -10,6 +10,8 @@ import operator
 import os
 import sys
 
+import mozpack.path as mozpath
+
 from mach.decorators import (
     CommandArgument,
     CommandProvider,
@@ -49,6 +51,23 @@ Preferences.
 ===================
 '''.strip()
 
+EXCESSIVE_SWAP_MESSAGE = '''
+===================
+PERFORMANCE WARNING
+
+Your machine experienced a lot of swap activity during the build. This is
+possibly a sign that your machine doesn't have enough physical memory or
+not enough available memory to perform the build. It's also possible some
+other system activity during the build is to blame.
+
+If you feel this message is not appropriate for your machine configuration,
+please file a Core :: Build Config bug at
+https://bugzilla.mozilla.org/enter_bug.cgi?product=Core&component=Build%20Config
+and tell us about your machine and build configuration so we can adjust the
+warning heuristic.
+===================
+'''
+
 
 class TerminalLoggingHandler(logging.Handler):
     """Custom logging handler that works with terminal window dressing.
@@ -136,42 +155,6 @@ class BuildProgressFooter(object):
             else:
                 parts.extend([tier, ' '])
 
-        parts.extend([('bold', 'SUBTIER'), ':', ' '])
-        for subtier, active, finished in tiers.current_subtier_status():
-            if active:
-                parts.extend([('underline_yellow', subtier), ' '])
-            elif finished:
-                parts.extend([('green', subtier), ' '])
-            else:
-                parts.extend([subtier, ' '])
-
-        if tiers.active_dirs:
-            parts.extend([('bold', 'DIRECTORIES'), ': '])
-            have_dirs = False
-
-            for subtier, all_dirs, active_dirs, complete in tiers.current_dirs_status():
-                if len(all_dirs) < 2:
-                    continue
-
-                have_dirs = True
-
-                parts.extend([
-                    '%02d' % (complete + 1),
-                    '/',
-                    '%02d' % len(all_dirs),
-                    ' ',
-                    '(',
-                ])
-                if active_dirs:
-                    commas = [', '] * (len(active_dirs) - 1)
-                    magenta_dirs = [('magenta', d) for d in active_dirs]
-                    parts.extend(i.next() for i in itertools.cycle((iter(magenta_dirs),
-                                                                    iter(commas))))
-                parts.append(')')
-
-            if not have_dirs:
-                parts = parts[0:-2]
-
         # We don't want to write more characters than the current width of the
         # terminal otherwise wrapping may result in weird behavior. We can't
         # simply truncate the line at terminal width characters because a)
@@ -286,15 +269,13 @@ class Build(MachCommandBase):
     @CommandArgument('--jobs', '-j', default='0', metavar='jobs', type=int,
         help='Number of concurrent jobs to run. Default is the number of CPUs.')
     @CommandArgument('what', default=None, nargs='*', help=BUILD_WHAT_HELP)
-    @CommandArgument('-p', '--pymake', action='store_true',
-        help='Force using pymake over GNU make.')
     @CommandArgument('-X', '--disable-extra-make-dependencies',
                      default=False, action='store_true',
                      help='Do not add extra make dependencies.')
     @CommandArgument('-v', '--verbose', action='store_true',
         help='Verbose output for what commands the build is running.')
-    def build(self, what=None, pymake=False,
-        disable_extra_make_dependencies=None, jobs=0, verbose=False):
+    def build(self, what=None, disable_extra_make_dependencies=None, jobs=0,
+        verbose=False):
         import which
         from mozbuild.controller.building import BuildMonitor
         from mozbuild.util import resolve_target_to_make
@@ -362,8 +343,8 @@ class Build(MachCommandBase):
                 # comprehensive history lesson.
                 self._run_make(directory=self.topobjdir,
                     target='backend.RecursiveMakeBackend',
-                    force_pymake=pymake, line_handler=output.on_line,
-                    log=False, print_directory=False)
+                    line_handler=output.on_line, log=False,
+                    print_directory=False)
 
                 # Build target pairs.
                 for make_dir, make_target in target_pairs:
@@ -374,8 +355,7 @@ class Build(MachCommandBase):
                     status = self._run_make(directory=make_dir, target=make_target,
                         line_handler=output.on_line, log=False, print_directory=False,
                         ensure_exit_code=False, num_jobs=jobs, silent=not verbose,
-                        append_env={b'NO_BUILDSTATUS_MESSAGES': b'1'},
-                        force_pymake=pymake)
+                        append_env={b'NO_BUILDSTATUS_MESSAGES': b'1'})
 
                     if status != 0:
                         break
@@ -384,7 +364,16 @@ class Build(MachCommandBase):
                 status = self._run_make(srcdir=True, filename='client.mk',
                     line_handler=output.on_line, log=False, print_directory=False,
                     allow_parallel=False, ensure_exit_code=False, num_jobs=jobs,
-                    silent=not verbose, force_pymake=pymake)
+                    silent=not verbose)
+
+                make_extra = self.mozconfig['make_extra'] or []
+                make_extra = dict(m.split('=', 1) for m in make_extra)
+
+                moz_automation = os.getenv('MOZ_AUTOMATION') or make_extra.get('export MOZ_AUTOMATION', None)
+                if moz_automation and status == 0:
+                    status = self._run_make(target='automation/build',
+                        line_handler=output.on_line, log=False, print_directory=False,
+                        ensure_exit_code=False, num_jobs=jobs, silent=not verbose)
 
                 self.log(logging.WARNING, 'warning_summary',
                     {'count': len(monitor.warnings_database)},
@@ -424,21 +413,29 @@ class Build(MachCommandBase):
             print('Your build was successful!')
 
         if monitor.have_resource_usage:
+            excessive, swap_in, swap_out = monitor.have_excessive_swapping()
+            # if excessive:
+            #    print(EXCESSIVE_SWAP_MESSAGE)
+
             print('To view resource usage of the build, run |mach '
                 'resource-usage|.')
 
         # Only for full builds because incremental builders likely don't
         # need to be burdened with this.
         if not what:
-            # Fennec doesn't have useful output from just building. We should
-            # arguably make the build action useful for Fennec. Another day...
-            if self.substs['MOZ_BUILD_APP'] != 'mobile/android':
-                app_path = self.get_binary_path('app')
-                print('To take your build for a test drive, run: %s' % app_path)
-            app = self.substs['MOZ_BUILD_APP']
-            if app in ('browser', 'mobile/android'):
-                print('For more information on what to do now, see '
-                    'https://developer.mozilla.org/docs/Developer_Guide/So_You_Just_Built_Firefox')
+            try:
+                # Fennec doesn't have useful output from just building. We should
+                # arguably make the build action useful for Fennec. Another day...
+                if self.substs['MOZ_BUILD_APP'] != 'mobile/android':
+                    print('To take your build for a test drive, run: |mach run|')
+                app = self.substs['MOZ_BUILD_APP']
+                if app in ('browser', 'mobile/android'):
+                    print('For more information on what to do now, see '
+                        'https://developer.mozilla.org/docs/Developer_Guide/So_You_Just_Built_Firefox')
+            except Exception:
+                # Ignore Exceptions in case we can't find config.status (such
+                # as when doing OSX Universal builds)
+                pass
 
         return status
 
@@ -505,13 +502,24 @@ class Build(MachCommandBase):
 
     @Command('build-backend', category='build',
         description='Generate a backend used to build the tree.')
-    def build_backend(self):
-        # When we support multiple build backends (Tup, Visual Studio, etc),
-        # this command will be expanded to support choosing what to generate.
+    @CommandArgument('-d', '--diff', action='store_true',
+        help='Show a diff of changes.')
+    # It would be nice to filter the choices below based on
+    # conditions, but that is for another day.
+    @CommandArgument('-b', '--backend',
+        choices=['RecursiveMake', 'AndroidEclipse', 'CppEclipse', 'VisualStudio'],
+        default='RecursiveMake',
+        help='Which backend to build (default: RecursiveMake).')
+    def build_backend(self, backend='RecursiveMake', diff=False):
         python = self.virtualenv_manager.python_path
         config_status = os.path.join(self.topobjdir, 'config.status')
-        return self._run_command_in_objdir(args=[python, config_status],
-            pass_thru=True, ensure_exit_code=False)
+
+        args = [python, config_status, '--backend=%s' % backend]
+        if diff:
+            args.append('--diff')
+
+        return self._run_command_in_objdir(args=args, pass_thru=True,
+            ensure_exit_code=False)
 
 
 @CommandProvider
@@ -729,7 +737,9 @@ class RunProgram(MachCommandBase):
         help='Do not pass the -no-remote argument by default.')
     @CommandArgument('+background', '+b', action='store_true',
         help='Do not pass the -foreground argument by default on Mac')
-    def run(self, params, remote, background):
+    @CommandArgument('+profile', '+P', action='store_true',
+        help='Specifiy thr profile to use')
+    def run(self, params, remote, background, profile):
         try:
             args = [self.get_binary_path('app')]
         except Exception as e:
@@ -741,6 +751,12 @@ class RunProgram(MachCommandBase):
             args.append('-no-remote')
         if not background and sys.platform == 'darwin':
             args.append('-foreground')
+        if '-profile' not in params and '-P' not in params:
+            path = os.path.join(self.topobjdir, 'tmp', 'scratch_user')
+            if not os.path.isdir(path):
+                os.makedirs(path)
+            args.append('-profile')
+            args.append(path)
         if params:
             args.extend(params)
         return self.run_process(args=args, ensure_exit_code=False,
@@ -758,8 +774,12 @@ class DebugProgram(MachCommandBase):
         help='Do not pass the -no-remote argument by default')
     @CommandArgument('+background', '+b', action='store_true',
         help='Do not pass the -foreground argument by default on Mac')
-    @CommandArgument('+gdbparams', default=None, metavar='params', type=str,
-        help='Command-line arguments to pass to GDB itself; split as the Bourne shell would.')
+    @CommandArgument('+debugger', default=None, type=str,
+        help='Name of debugger to launch')
+    @CommandArgument('+debugparams', default=None, metavar='params', type=str,
+        help='Command-line arguments to pass to GDB or LLDB itself; split as the Bourne shell would.')
+    @CommandArgument('+profile', '+P', action='store_true',
+        help='Specifiy thr profile to use')
     # Bug 933807 introduced JS_DISABLE_SLOW_SCRIPT_SIGNALS to avoid clever
     # segfaults induced by the slow-script-detecting logic for Ion/Odin JITted
     # code.  If we don't pass this, the user will need to periodically type
@@ -767,37 +787,71 @@ class DebugProgram(MachCommandBase):
     # automatic resuming; see the bug.
     @CommandArgument('+slowscript', action='store_true',
         help='Do not set the JS_DISABLE_SLOW_SCRIPT_SIGNALS env variable; when not set, recoverable but misleading SIGSEGV instances may occur in Ion/Odin JIT code')
-    def debug(self, params, remote, background, gdbparams, slowscript):
+    def debug(self, params, remote, background, profile, debugger, debugparams, slowscript):
         import which
-        try:
-            debugger = which.which('gdb')
-        except Exception as e:
-            print("You don't have gdb in your PATH")
-            print(e)
-            return 1
+        if debugger:
+            try:
+                debugger = which.which(debugger)
+            except Exception as e:
+                print("You don't have %s in your PATH" % (debugger))
+                print(e)
+                return 1
+        else:
+            try:
+                debugger = which.which('gdb')
+            except Exception:
+                try:
+                    debugger = which.which('lldb')
+                except Exception as e:
+                    print("You don't have gdb or lldb in your PATH")
+                    print(e)
+                    return 1
         args = [debugger]
-        extra_env = {}
-        if gdbparams:
+        extra_env = { 'MOZ_CRASHREPORTER_DISABLE' : '1' }
+        if debugparams:
             import pymake.process
-            (argv, badchar) = pymake.process.clinetoargv(gdbparams, os.getcwd())
+            argv, badchar = pymake.process.clinetoargv(debugparams, os.getcwd())
             if badchar:
-                print("The +gdbparams you passed require a real shell to parse them.")
+                print("The +debugparams you passed require a real shell to parse them.")
                 print("(We can't handle the %r character.)" % (badchar,))
                 return 1
             args.extend(argv)
+
+        binpath = None
+
         try:
-            args.extend(['--args', self.get_binary_path('app')])
+            binpath = self.get_binary_path('app')
         except Exception as e:
             print("It looks like your program isn't built.",
                 "You can run |mach build| to build it.")
             print(e)
             return 1
+
+        # args added to separate the debugger and process arguments.
+        args_separator = {
+            'gdb': '--args',
+            'ddd': '--args',
+            'cgdb': '--args',
+            'lldb': '--'
+        }
+
+        debugger_name = os.path.basename(debugger)
+        if debugger_name in args_separator:
+            args.append(args_separator[debugger_name])
+        args.append(binpath)
+
         if not remote:
             args.append('-no-remote')
         if not background and sys.platform == 'darwin':
             args.append('-foreground')
         if params:
             args.extend(params)
+        if '-profile' not in params and '-P' not in params:
+            path = os.path.join(self.topobjdir, 'tmp', 'scratch_user')
+            if not os.path.isdir(path):
+                os.makedirs(path)
+            args.append('-profile')
+            args.append(path)
         if not slowscript:
             extra_env['JS_DISABLE_SLOW_SCRIPT_SIGNALS'] = '1'
         return self.run_process(args=args, append_env=extra_env,
@@ -852,11 +906,15 @@ class Makefiles(MachCommandBase):
             return True
 
         for path in self._makefile_ins():
-            statements = [s for s in pymake.parser.parsefile(path)
-                if is_statement_relevant(s)]
+            relpath = os.path.relpath(path, self.topsrcdir)
+            try:
+                statements = [s for s in pymake.parser.parsefile(path)
+                    if is_statement_relevant(s)]
 
-            if not statements:
-                print(os.path.relpath(path, self.topsrcdir))
+                if not statements:
+                    print(relpath)
+            except pymake.parser.SyntaxError:
+                print('Warning: Could not parse %s' % relpath, file=sys.stderr)
 
     def _makefile_ins(self):
         for root, dirs, files in os.walk(self.topsrcdir):
@@ -868,144 +926,128 @@ class Makefiles(MachCommandBase):
 class MachDebug(MachCommandBase):
     @Command('environment', category='build-dev',
         description='Show info about the mach and build environment.')
+    @CommandArgument('--format', default='pretty',
+        choices=['pretty', 'client.mk', 'configure', 'json'],
+        help='Print data in the given format.')
+    @CommandArgument('--output', '-o', type=str,
+        help='Output to the given file.')
     @CommandArgument('--verbose', '-v', action='store_true',
         help='Print verbose output.')
-    def environment(self, verbose=False):
+    def environment(self, format, output=None, verbose=False):
+        func = getattr(self, '_environment_%s' % format.replace('.', '_'))
+
+        if output:
+            # We want to preserve mtimes if the output file already exists
+            # and the content hasn't changed.
+            from mozbuild.util import FileAvoidWrite
+            with FileAvoidWrite(output) as out:
+                return func(out, verbose)
+        return func(sys.stdout, verbose)
+
+    def _environment_pretty(self, out, verbose):
+        state_dir = self._mach_context.state_dir
         import platform
-        print('platform:\n\t%s' % platform.platform())
-        print('python version:\n\t%s' % sys.version)
-        print('python prefix:\n\t%s' % sys.prefix)
-        print('mach cwd:\n\t%s' % self._mach_context.cwd)
-        print('os cwd:\n\t%s' % os.getcwd())
-        print('mach directory:\n\t%s' % self._mach_context.topdir)
-        print('state directory:\n\t%s' % self._mach_context.state_dir)
+        print('platform:\n\t%s' % platform.platform(), file=out)
+        print('python version:\n\t%s' % sys.version, file=out)
+        print('python prefix:\n\t%s' % sys.prefix, file=out)
+        print('mach cwd:\n\t%s' % self._mach_context.cwd, file=out)
+        print('os cwd:\n\t%s' % os.getcwd(), file=out)
+        print('mach directory:\n\t%s' % self._mach_context.topdir, file=out)
+        print('state directory:\n\t%s' % state_dir, file=out)
 
-        try:
-            mb = MozbuildObject.from_environment(cwd=self._mach_context.cwd)
-        except ObjdirMismatchException as e:
-            print('Ambiguous object directory detected. We detected that '
-                'both %s and %s could be object directories. This is '
-                'typically caused by having a mozconfig pointing to a '
-                'different object directory from the current working '
-                'directory. To solve this problem, ensure you do not have a '
-                'default mozconfig in searched paths.' % (e.objdir1,
-                    e.objdir2))
-            return 1
+        print('object directory:\n\t%s' % self.topobjdir, file=out)
 
-        mozconfig = None
+        if self.mozconfig['path']:
+            print('mozconfig path:\n\t%s' % self.mozconfig['path'], file=out)
+            if self.mozconfig['configure_args']:
+                print('mozconfig configure args:', file=out)
+                for arg in self.mozconfig['configure_args']:
+                    print('\t%s' % arg, file=out)
 
-        try:
-            mozconfig = mb.mozconfig
-            print('mozconfig path:\n\t%s' % mozconfig['path'])
-        except MozconfigFindException as e:
-            print('Unable to find mozconfig: %s' % e.message)
-            return 1
+            if self.mozconfig['make_extra']:
+                print('mozconfig extra make args:', file=out)
+                for arg in self.mozconfig['make_extra']:
+                    print('\t%s' % arg, file=out)
 
-        except MozconfigLoadException as e:
-            print('Error loading mozconfig: %s' % e.path)
-            print(e.message)
-
-            if e.output:
-                print('mozconfig evaluation output:')
-                for line in e.output:
-                    print(line)
-
-            return 1
-
-        print('object directory:\n\t%s' % mb.topobjdir)
-
-        if mozconfig:
-            print('mozconfig configure args:')
-            if mozconfig['configure_args']:
-                for arg in mozconfig['configure_args']:
-                    print('\t%s' % arg)
-
-            print('mozconfig extra make args:')
-            if mozconfig['make_extra']:
-                for arg in mozconfig['make_extra']:
-                    print('\t%s' % arg)
-
-            print('mozconfig make flags:')
-            if mozconfig['make_flags']:
-                for arg in mozconfig['make_flags']:
-                    print('\t%s' % arg)
+            if self.mozconfig['make_flags']:
+                print('mozconfig make flags:', file=out)
+                for arg in self.mozconfig['make_flags']:
+                    print('\t%s' % arg, file=out)
 
         config = None
 
         try:
-            config = mb.config_environment
+            config = self.config_environment
 
         except Exception:
             pass
 
         if config:
-            print('config topsrcdir:\n\t%s' % config.topsrcdir)
-            print('config topobjdir:\n\t%s' % config.topobjdir)
+            print('config topsrcdir:\n\t%s' % config.topsrcdir, file=out)
+            print('config topobjdir:\n\t%s' % config.topobjdir, file=out)
 
             if verbose:
-                print('config substitutions:')
+                print('config substitutions:', file=out)
                 for k in sorted(config.substs):
-                    print('\t%s: %s' % (k, config.substs[k]))
+                    print('\t%s: %s' % (k, config.substs[k]), file=out)
 
-                print('config defines:')
+                print('config defines:', file=out)
                 for k in sorted(config.defines):
-                    print('\t%s' % k)
+                    print('\t%s' % k, file=out)
 
+    def _environment_client_mk(self, out, verbose):
+        if self.mozconfig['make_extra']:
+            for arg in self.mozconfig['make_extra']:
+                print(arg, file=out)
+        objdir = mozpath.normsep(self.topobjdir)
+        print('MOZ_OBJDIR=%s' % objdir, file=out)
+        if 'MOZ_CURRENT_PROJECT' in os.environ:
+            objdir = mozpath.join(objdir, os.environ['MOZ_CURRENT_PROJECT'])
+        print('OBJDIR=%s' % objdir, file=out)
+        if self.mozconfig['path']:
+            print('FOUND_MOZCONFIG=%s' % mozpath.normsep(self.mozconfig['path']),
+                file=out)
 
-@CommandProvider
-class Documentation(MachCommandBase):
-    """Helps manage in-tree documentation."""
+    def _environment_configure(self, out, verbose):
+        if self.mozconfig['path']:
+            # Replace ' with '"'"', so that shell quoting e.g.
+            # a'b becomes 'a'"'"'b'.
+            quote = lambda s: s.replace("'", """'"'"'""")
+            if self.mozconfig['configure_args'] and \
+                    'COMM_BUILD' not in os.environ:
+                print('echo Adding configure options from %s' %
+                    mozpath.normsep(self.mozconfig['path']), file=out)
+                for arg in self.mozconfig['configure_args']:
+                    quoted_arg = quote(arg)
+                    print("echo '  %s'" % quoted_arg, file=out)
+                    print("""set -- "$@" '%s'""" % quoted_arg, file=out)
+            for key, value in self.mozconfig['env']['added'].items():
+                print("export %s='%s'" % (key, quote(value)), file=out)
+            for key, (old, value) in self.mozconfig['env']['modified'].items():
+                print("export %s='%s'" % (key, quote(value)), file=out)
+            for key, value in self.mozconfig['vars']['added'].items():
+                print("%s='%s'" % (key, quote(value)), file=out)
+            for key, (old, value) in self.mozconfig['vars']['modified'].items():
+                print("%s='%s'" % (key, quote(value)), file=out)
+            for key in self.mozconfig['env']['removed'].keys() + \
+                    self.mozconfig['vars']['removed'].keys():
+                print("unset %s" % key, file=out)
 
-    @Command('build-docs', category='build-dev',
-        description='Generate documentation for the tree.')
-    @CommandArgument('--format', default='html',
-        help='Documentation format to write.')
-    @CommandArgument('--api-docs', action='store_true',
-        help='If specified, we will generate api docs templates for in-tree '
-            'Python. This will likely create and/or modify files in '
-            'build/docs. It is meant to be run by build maintainers when new '
-            'Python modules are added to the tree.')
-    @CommandArgument('outdir', default='<DEFAULT>', nargs='?',
-        help='Where to write output.')
-    def build_docs(self, format=None, api_docs=False, outdir=None):
-        self._activate_virtualenv()
-
-        self.virtualenv_manager.install_pip_package('mdn-sphinx-theme==0.3')
-
-        if api_docs:
-            import sphinx.apidoc
-            outdir = os.path.join(self.topsrcdir, 'build', 'docs', 'python')
-
-            base_args = [sys.argv[0], '--no-toc', '-o', outdir]
-
-            packages = [
-                ('python/codegen',),
-                ('python/mozbuild/mozbuild', 'test'),
-                ('python/mozbuild/mozpack', 'test'),
-                ('python/mozversioncontrol/mozversioncontrol',),
-            ]
-
-            for package in packages:
-                extra_args = [os.path.join(self.topsrcdir, package[0])]
-                extra_args.extend(package[1:])
-
-                sphinx.apidoc.main(base_args + extra_args)
-
-            return 0
-
-        import sphinx
-        if outdir == '<DEFAULT>':
-            outdir = os.path.join(self.topobjdir, 'docs', format)
-
-        args = [
-            sys.argv[0],
-            '-b', format,
-            os.path.join(self.topsrcdir, 'build', 'docs'),
-            outdir,
-        ]
-
-        result = sphinx.main(args)
-
-        print('Docs written to %s.' % outdir)
-
-        return result
+    def _environment_json(self, out, verbose):
+        import json
+        class EnvironmentEncoder(json.JSONEncoder):
+            def default(self, obj):
+                if isinstance(obj, MozbuildObject):
+                    result = {
+                        'topsrcdir': obj.topsrcdir,
+                        'topobjdir': obj.topobjdir,
+                        'mozconfig': obj.mozconfig,
+                    }
+                    if verbose:
+                        result['substs'] = obj.substs
+                        result['defines'] = obj.defines
+                    return result
+                elif isinstance(obj, set):
+                    return list(obj)
+                return json.JSONEncoder.default(self, obj)
+        json.dump(self, cls=EnvironmentEncoder, sort_keys=True, fp=out)
