@@ -8,7 +8,7 @@
 import os
 import re
 import json
-
+import mozbuild.mozconfig as mozconfig
 
 def build_dict(config, env=os.environ):
     """
@@ -27,10 +27,9 @@ def build_dict(config, env=os.environ):
     d = {}
     d['topsrcdir'] = config.topsrcdir
 
-    if 'MOZCONFIG' in env:
-        mozconfig = env["MOZCONFIG"]
-        mozconfig = os.path.join(config.topsrcdir, mozconfig)
-        d['mozconfig'] = os.path.normpath(mozconfig)
+    the_mozconfig = mozconfig.MozconfigLoader(config.topsrcdir).find_mozconfig(env)
+    if the_mozconfig:
+        d['mozconfig'] = the_mozconfig
 
     # os
     o = substs["OS_TARGET"]
@@ -51,6 +50,10 @@ def build_dict(config, env=os.environ):
     if 'MOZ_APP_NAME' in substs:
         d["appname"] = substs["MOZ_APP_NAME"]
 
+    # Build app name
+    if 'MOZ_BUILD_APP' in substs:
+        d["buildapp"] = substs["MOZ_BUILD_APP"]
+
     # processor
     p = substs["TARGET_CPU"]
     # for universal mac builds, put in a special value
@@ -75,11 +78,11 @@ def build_dict(config, env=os.environ):
     d['debug'] = substs.get('MOZ_DEBUG') == '1'
     d['crashreporter'] = bool(substs.get('MOZ_CRASHREPORTER'))
     d['datareporting'] = bool(substs.get('MOZ_DATA_REPORTING'))
+    d['healthreport'] = substs.get('MOZ_SERVICES_HEALTHREPORT') == '1'
     d['asan'] = substs.get('MOZ_ASAN') == '1'
     d['tests_enabled'] = substs.get('ENABLE_TESTS') == "1"
     d['bin_suffix'] = substs.get('BIN_SUFFIX', '')
 
-    d['ogg'] = bool(substs.get('MOZ_OGG'))
     d['webm'] = bool(substs.get('MOZ_WEBM'))
     d['wave'] = bool(substs.get('MOZ_WAVE'))
 
