@@ -89,8 +89,10 @@ public:
               bool aTruthValue);
     Assertion(nsIRDFResource* aSource);     // PLDHashTable assertion variant
 
+private:
     ~Assertion();
 
+public:
     void AddRef() {
         if (mRefCnt == UINT16_MAX) {
             NS_WARNING("refcount overflow, leaking Assertion");
@@ -391,6 +393,8 @@ private:
     Assertion*      mNextAssertion;
     nsCOMPtr<nsISupportsArray> mHashArcs;
 
+    virtual ~InMemoryAssertionEnumeratorImpl();
+
 public:
     InMemoryAssertionEnumeratorImpl(InMemoryDataSource* aDataSource,
                                     nsIRDFResource* aSource,
@@ -398,8 +402,6 @@ public:
                                     nsIRDFNode* aTarget,
                                     bool aTruthValue);
 
-    virtual ~InMemoryAssertionEnumeratorImpl();
-
     // nsISupports interface
     NS_DECL_ISUPPORTS
 
@@ -469,7 +471,7 @@ InMemoryAssertionEnumeratorImpl::~InMemoryAssertionEnumeratorImpl()
 
 NS_IMPL_ADDREF(InMemoryAssertionEnumeratorImpl)
 NS_IMPL_RELEASE(InMemoryAssertionEnumeratorImpl)
-NS_IMPL_QUERY_INTERFACE1(InMemoryAssertionEnumeratorImpl, nsISimpleEnumerator)
+NS_IMPL_QUERY_INTERFACE(InMemoryAssertionEnumeratorImpl, nsISimpleEnumerator)
 
 NS_IMETHODIMP
 InMemoryAssertionEnumeratorImpl::HasMoreElements(bool* aResult)
@@ -563,13 +565,13 @@ private:
     ArcEnumerator(PLDHashTable* aTable, PLDHashEntryHdr* aHdr,
                        uint32_t aNumber, void* aArg);
 
+    virtual ~InMemoryArcsEnumeratorImpl();
+
 public:
     InMemoryArcsEnumeratorImpl(InMemoryDataSource* aDataSource,
                                nsIRDFResource* aSource,
                                nsIRDFNode* aTarget);
 
-    virtual ~InMemoryArcsEnumeratorImpl();
-
     // nsISupports interface
     NS_DECL_ISUPPORTS
 
@@ -637,7 +639,7 @@ InMemoryArcsEnumeratorImpl::~InMemoryArcsEnumeratorImpl()
 
 NS_IMPL_ADDREF(InMemoryArcsEnumeratorImpl)
 NS_IMPL_RELEASE(InMemoryArcsEnumeratorImpl)
-NS_IMPL_QUERY_INTERFACE1(InMemoryArcsEnumeratorImpl, nsISimpleEnumerator)
+NS_IMPL_QUERY_INTERFACE(InMemoryArcsEnumeratorImpl, nsISimpleEnumerator)
 
 NS_IMETHODIMP
 InMemoryArcsEnumeratorImpl::HasMoreElements(bool* aResult)
@@ -780,22 +782,17 @@ InMemoryDataSource::InMemoryDataSource(nsISupports* aOuter)
 nsresult
 InMemoryDataSource::Init()
 {
-    if (!PL_DHashTableInit(&mForwardArcs,
-                           PL_DHashGetStubOps(),
-                           nullptr,
-                           sizeof(Entry),
-                           PL_DHASH_MIN_SIZE)) {
-        mForwardArcs.ops = nullptr;
-        return NS_ERROR_OUT_OF_MEMORY;
-    }
-    if (!PL_DHashTableInit(&mReverseArcs,
-                           PL_DHashGetStubOps(),
-                           nullptr,
-                           sizeof(Entry),
-                           PL_DHASH_MIN_SIZE)) {
-        mReverseArcs.ops = nullptr;
-        return NS_ERROR_OUT_OF_MEMORY;
-    }
+    PL_DHashTableInit(&mForwardArcs,
+                      PL_DHashGetStubOps(),
+                      nullptr,
+                      sizeof(Entry),
+                      PL_DHASH_MIN_SIZE);
+
+    PL_DHashTableInit(&mReverseArcs,
+                      PL_DHashGetStubOps(),
+                      nullptr,
+                      sizeof(Entry),
+                      PL_DHASH_MIN_SIZE);
 
 #ifdef PR_LOGGING
     if (! gLog)
@@ -1322,7 +1319,7 @@ InMemoryDataSource::LockedUnassert(nsIRDFResource* aSource,
             else {
                 // If this second-level hash empties out, clean it up.
                 if (!root->u.hash.mPropertyHash->entryCount) {
-                    delete root;
+                    root->Release();
                     SetForwardArcs(aSource, nullptr);
                 }
             }
@@ -1741,7 +1738,7 @@ InMemoryDataSource::EnsureFastContainment(nsIRDFResource* aSource)
     // Add the datasource's owning reference.
     hashAssertion->AddRef();
 
-    register Assertion *first = GetForwardArcs(aSource);
+    Assertion *first = GetForwardArcs(aSource);
     SetForwardArcs(aSource, hashAssertion);
 
     // mutate references of existing forward assertions into this hash
@@ -1926,7 +1923,7 @@ InMemoryDataSource::SweepForwardArcsEntries(PLDHashTable* aTable,
 
         // If the sub-hash is now empty, clean it up.
         if (!as->u.hash.mPropertyHash->entryCount) {
-            delete as;
+            as->Release();
             result = PL_DHASH_REMOVE;
         }
 
