@@ -17,7 +17,7 @@ Cu.import("resource://gre/modules/services/datareporting/policy.jsm");
 Cu.import("resource://gre/modules/Services.jsm");
 Cu.import("resource://gre/modules/Task.jsm");
 Cu.import("resource://testing-common/httpd.js");
-Cu.import("resource://testing-common/services-common/bagheeraserver.js");
+Cu.import("resource://testing-common/services/common/bagheeraserver.js");
 Cu.import("resource://testing-common/services/metrics/mocks.jsm");
 Cu.import("resource://testing-common/services/healthreport/utils.jsm");
 Cu.import("resource://testing-common/AppData.jsm");
@@ -103,13 +103,14 @@ add_task(function test_init() {
 
 add_task(function test_constructor() {
   let reporter = yield getReporter("constructor");
-
   try {
     do_check_eq(reporter.lastPingDate.getTime(), 0);
     do_check_null(reporter.lastSubmitID);
     do_check_eq(typeof(reporter._state), "object");
     do_check_eq(reporter._state.lastPingDate.getTime(), 0);
     do_check_eq(reporter._state.remoteIDs.length, 0);
+    do_check_eq(reporter._state.clientIDVersion, 1);
+    do_check_neq(reporter._state.clientID, null);
 
     let failed = false;
     try {
@@ -122,7 +123,7 @@ add_task(function test_constructor() {
       failed = false;
     }
   } finally {
-    reporter._shutdown();
+    yield reporter._shutdown();
   }
 });
 
@@ -132,7 +133,7 @@ add_task(function test_shutdown_normal() {
   // We can't send "quit-application" notification because the xpcshell runner
   // will shut down!
   reporter._initiateShutdown();
-  reporter._waitForShutdown();
+  yield reporter._promiseShutdown;
 });
 
 add_task(function test_shutdown_storage_in_progress() {
@@ -145,7 +146,7 @@ add_task(function test_shutdown_storage_in_progress() {
 
   reporter.init();
 
-  reporter._waitForShutdown();
+  yield reporter._promiseShutdown;
   do_check_eq(reporter.providerManagerShutdownCount, 0);
   do_check_eq(reporter.storageCloseCount, 1);
 });
@@ -164,7 +165,7 @@ add_task(function test_shutdown_provider_manager_in_progress() {
   reporter.init();
 
   // This will hang if shutdown logic is busted.
-  reporter._waitForShutdown();
+  yield reporter._promiseShutdown;
   do_check_eq(reporter.providerManagerShutdownCount, 1);
   do_check_eq(reporter.storageCloseCount, 1);
 });
@@ -174,15 +175,21 @@ add_task(function test_shutdown_when_provider_manager_errors() {
   let reporter = yield getHealthReporter("shutdown_when_provider_manager_errors",
                                        DUMMY_URI, true);
 
+  let error = new Error("Fake error during provider manager initialization.");
   reporter.onInitializeProviderManagerFinished = function () {
     print("Throwing fake error.");
-    throw new Error("Fake error during provider manager initialization.");
+    throw error;
   };
 
-  reporter.init();
+  try {
+    yield reporter.init();
+    do_throw("The error was not reported by init()");
+  } catch (ex if ex == error) {
+    do_print("The error was reported by init()");
+  }
 
   // This will hang if shutdown logic is busted.
-  reporter._waitForShutdown();
+  yield reporter._promiseShutdown;
   do_check_eq(reporter.providerManagerShutdownCount, 1);
   do_check_eq(reporter.storageCloseCount, 1);
 });
@@ -219,7 +226,7 @@ add_task(function test_pull_only_providers() {
     let values = yield reporter._storage.getMeasurementValues(mID);
     do_check_true(values.singular.size > 0);
   } finally {
-    reporter._shutdown();
+    yield reporter._shutdown();
   }
 });
 
@@ -250,7 +257,7 @@ add_task(function test_collect_daily() {
     yield reporter.collectMeasurements();
     do_check_eq(provider.collectDailyCount, 3);
   } finally {
-    reporter._shutdown();
+    yield reporter._shutdown();
   }
 });
 
@@ -274,7 +281,7 @@ add_task(function test_remove_old_lastpayload() {
       do_check_false(yield OS.File.exists(path));
     }
     yield reporter._state.save();
-    reporter._shutdown();
+    yield reporter._shutdown();
 
     let o = yield CommonUtils.readJSON(reporter._state._filename);
     do_check_true(o.removedOutdatedLastpayload);
@@ -286,13 +293,16 @@ add_task(function test_remove_old_lastpayload() {
       do_check_true(yield OS.File.exists(path));
     }
   } finally {
-    reporter._shutdown();
+    yield reporter._shutdown();
   }
 });
 
 add_task(function test_json_payload_simple() {
   let reporter = yield getReporter("json_payload_simple");
 
+  let clientID = reporter._state.clientID;
+  do_check_neq(clientID, null);
+
   try {
     let now = new Date();
     let payload = yield reporter.getJSONPayload();
@@ -301,6 +311,9 @@ add_task(function test_json_payload_simple() {
 
     do_check_eq(original.version, 2);
     do_check_eq(original.thisPingDate, reporter._formatDate(now));
+    do_check_eq(original.clientID, clientID);
+    do_check_eq(original.clientIDVersion, reporter._state.clientIDVersion);
+    do_check_eq(original.clientIDVersion, 1);
     do_check_eq(Object.keys(original.data.last).length, 0);
     do_check_eq(Object.keys(original.data.days).length, 0);
     do_check_false("notInitialized" in original);
@@ -310,6 +323,7 @@ add_task(function test_json_payload_simple() {
 
     original = JSON.parse(yield reporter.getJSONPayload());
     do_check_eq(original.lastPingDate, reporter._formatDate(reporter.lastPingDate));
+    do_check_eq(original.clientID, clientID);
 
     // This could fail if we cross UTC day boundaries at the exact instance the
     // test is executed. Let's tempt fate.
@@ -318,7 +332,7 @@ add_task(function test_json_payload_simple() {
     payload = yield reporter.getJSONPayload(true);
     do_check_eq(typeof payload, "object");
   } finally {
-    reporter._shutdown();
+    yield reporter._shutdown();
   }
 });
 
@@ -337,7 +351,7 @@ add_task(function test_json_payload_dummy_provider() {
     do_check_true(name in o.data.last);
     do_check_eq(o.data.last[name]._v, 1);
   } finally {
-    reporter._shutdown();
+    yield reporter._shutdown();
   }
 });
 
@@ -355,7 +369,7 @@ add_task(function test_collect_and_obtain_json_payload() {
     payload = yield reporter.collectAndObtainJSONPayload(true);
     do_check_eq(typeof payload, "object");
   } finally {
-    reporter._shutdown();
+    yield reporter._shutdown();
   }
 });
 
@@ -411,7 +425,7 @@ add_task(function test_constant_only_providers_in_json_payload() {
 
     yield deferred.promise;
   } finally {
-    reporter._shutdown();
+    yield reporter._shutdown();
   }
 });
 
@@ -446,7 +460,7 @@ add_task(function test_json_payload_multiple_days() {
     let today = reporter._formatDate(now);
     do_check_true(today in o.data.days);
   } finally {
-    reporter._shutdown();
+    yield reporter._shutdown();
   }
 });
 
@@ -523,7 +537,7 @@ add_task(function test_json_payload_newer_version_overwrites() {
     do_check_eq(o.data.days[day]["MultiMeasurementProvider.DummyMeasurement"]._v, highestVersion);
 
   } finally {
-    reporter._shutdown();
+    yield reporter._shutdown();
   }
 });
 
@@ -548,7 +562,7 @@ add_task(function test_idle_daily() {
     values = yield m.getValues();
     do_check_eq(values.days.size, 180);
   } finally {
-    reporter._shutdown();
+    yield reporter._shutdown();
   }
 });
 
@@ -636,15 +650,17 @@ add_task(function test_data_submission_success() {
 
     let d = reporter.lastPingDate;
     let id = reporter.lastSubmitID;
+    let clientID = reporter._state.clientID;
 
-    reporter._shutdown();
+    yield reporter._shutdown();
 
     // Ensure reloading state works.
     reporter = yield getReporter("data_submission_success");
     do_check_eq(reporter.lastSubmitID, id);
     do_check_eq(reporter.lastPingDate.getTime(), d.getTime());
+    do_check_eq(reporter._state.clientID, clientID);
 
-    reporter._shutdown();
+    yield reporter._shutdown();
   } finally {
     yield shutdownServer(server);
   }
@@ -703,6 +719,9 @@ add_task(function test_request_remote_data_deletion() {
     do_check_neq(id, null);
     do_check_true(server.hasDocument(reporter.serverNamespace, id));
 
+    let clientID = reporter._state.clientID;
+    do_check_neq(clientID, null);
+
     defineNow(policy, policy._futureDate(10 * 1000));
 
     let promise = reporter.requestDeleteRemoteData();
@@ -711,6 +730,16 @@ add_task(function test_request_remote_data_deletion() {
     do_check_null(reporter.lastSubmitID);
     do_check_false(reporter.haveRemoteData());
     do_check_false(server.hasDocument(reporter.serverNamespace, id));
+
+    // Client ID should be updated.
+    do_check_neq(reporter._state.clientID, null);
+    do_check_neq(reporter._state.clientID, clientID);
+    do_check_eq(reporter._state.clientIDVersion, 1);
+
+    // And it should be persisted to disk.
+    let o = yield CommonUtils.readJSON(reporter._state._filename);
+    do_check_eq(o.clientID, reporter._state.clientID);
+    do_check_eq(o.clientIDVersion, 1);
   } finally {
     reporter._shutdown();
     yield shutdownServer(server);
@@ -782,7 +811,7 @@ add_task(function test_policy_accept_reject() {
     do_check_false(policy.dataSubmissionPolicyAccepted);
     do_check_false(reporter.willUploadData);
   } finally {
-    reporter._shutdown();
+    yield reporter._shutdown();
     yield shutdownServer(server);
   }
 });
@@ -805,7 +834,7 @@ add_task(function test_error_message_scrubbing() {
     reporter._recordError("Foo " + uri.spec);
     do_check_eq(reporter._errors[0], "Foo <AppDataURI>");
   } finally {
-    reporter._shutdown();
+    yield reporter._shutdown();
   }
 });
 
@@ -831,7 +860,7 @@ add_task(function test_basic_appinfo() {
     do_check_eq(payload["version"], 2);
     verify(payload["geckoAppInfo"]);
   } finally {
-    reporter._shutdown();
+    yield reporter._shutdown();
   }
 });
 
@@ -852,13 +881,13 @@ add_task(function test_collect_when_upload_disabled() {
     // We would ideally ensure the timer fires and does the right thing.
     // However, testing the update timer manager is quite involved.
   } finally {
-    reporter._shutdown();
+    yield reporter._shutdown();
   }
 });
 
 add_task(function test_failure_if_not_initialized() {
   let reporter = yield getReporter("failure_if_not_initialized");
-  reporter._shutdown();
+  yield reporter._shutdown();
 
   let error = false;
   try {
@@ -886,13 +915,15 @@ add_task(function test_failure_if_not_initialized() {
 });
 
 add_task(function test_upload_on_init_failure() {
+  let MESSAGE = "Fake error during provider manager initialization." + Math.random();
+
   let server = new BagheeraServer();
   server.start();
   let reporter = yield getHealthReporter("upload_on_init_failure", server.serverURI, true);
   server.createNamespace(reporter.serverNamespace);
 
   reporter.onInitializeProviderManagerFinished = function () {
-    throw new Error("Fake error during provider manager initialization.");
+    throw new Error(MESSAGE);
   };
 
   let deferred = Promise.defer();
@@ -930,9 +961,9 @@ add_task(function test_upload_on_init_failure() {
   do_check_eq(doc.notInitialized, 1);
   do_check_true("errors" in doc);
   do_check_eq(doc.errors.length, 1);
-  do_check_true(doc.errors[0].contains("Fake error during provider manager initialization"));
+  do_check_true(doc.errors[0].contains(MESSAGE));
 
-  reporter._shutdown();
+  yield reporter._shutdown();
   yield shutdownServer(server);
 });
 
@@ -958,7 +989,7 @@ add_task(function test_state_prefs_conversion_simple() {
     do_check_false(prefs.isSet("lastSubmitID"));
     do_check_false(prefs.isSet("lastPingTime"));
   } finally {
-    reporter._shutdown();
+    yield reporter._shutdown();
   }
 });
 
@@ -980,7 +1011,7 @@ add_task(function test_state_no_json_object() {
     do_check_eq(o.lastPingTime, 0);
     do_check_eq(o.remoteIDs.length, 0);
   } finally {
-    reporter._shutdown();
+    yield reporter._shutdown();
   }
 });
 
@@ -1001,7 +1032,7 @@ add_task(function test_state_future_version() {
     do_check_eq(o.lastPingTime, 2412);
     do_check_eq(o.remoteIDs.length, 1);
   } finally {
-    reporter._shutdown();
+    yield reporter._shutdown();
   }
 });
 
@@ -1020,7 +1051,7 @@ add_task(function test_state_invalid_json() {
     do_check_eq(reporter.lastPingDate.getTime(), 0);
     do_check_null(reporter.lastSubmitID);
   } finally {
-    reporter._shutdown();
+    yield reporter._shutdown();
   }
 });
 
@@ -1061,7 +1092,7 @@ add_task(function test_state_multiple_remote_ids() {
     do_check_eq(o.lastPingTime, reporter.lastPingDate.getTime());
   } finally {
     yield shutdownServer(server);
-    reporter._shutdown();
+    yield reporter._shutdown();
   }
 });
 
@@ -1092,7 +1123,138 @@ add_task(function test_state_downgrade_upgrade() {
     let o = yield CommonUtils.readJSON(reporter._state._filename);
     do_check_eq(o.remoteIDs.length, 3);
     do_check_eq(o.lastPingTime, now.getTime() + 1000);
+  } finally {
+    yield reporter._shutdown();
+  }
+});
+
+// Missing client ID in state should be created on state load.
+add_task(function* test_state_create_client_id() {
+  let reporter = getHealthReporter("state_create_client_id");
+
+  yield CommonUtils.writeJSON({
+    v: 1,
+    remoteIDs: ["id1", "id2"],
+    lastPingTime: Date.now(),
+    removeOutdatedLastPayload: true,
+  }, reporter._state._filename);
+
+  try {
+    yield reporter.init();
+
+    do_check_eq(reporter.lastSubmitID, "id1");
+    do_check_neq(reporter._state.clientID, null);
+    do_check_eq(reporter._state.clientID.length, 36);
+    do_check_eq(reporter._state.clientIDVersion, 1);
+
+    let clientID = reporter._state.clientID;
+
+    // The client ID should be persisted as soon as it is created.
+    reporter._shutdown();
+
+    reporter = getHealthReporter("state_create_client_id");
+    yield reporter.init();
+    do_check_eq(reporter._state.clientID, clientID);
   } finally {
     reporter._shutdown();
   }
 });
+
+// Invalid stored client ID is reset automatically.
+add_task(function* test_empty_client_id() {
+  let reporter = getHealthReporter("state_empty_client_id");
+
+  yield CommonUtils.writeJSON({
+    v: 1,
+    clientID: "",
+    remoteIDs: ["id1", "id2"],
+    lastPingTime: Date.now(),
+    removeOutdatedLastPayload: true,
+  }, reporter._state._filename);
+
+  try {
+    yield reporter.init();
+
+    do_check_neq(reporter._state.clientID, null);
+    do_check_eq(reporter._state.clientID.length, 36);
+  } finally {
+    reporter._shutdown();
+  }
+});
+
+add_task(function* test_nonstring_client_id() {
+  let reporter = getHealthReporter("state_nonstring_client_id");
+
+  yield CommonUtils.writeJSON({
+    v: 1,
+    clientID: 42,
+    remoteIDs: ["id1", "id2"],
+    lastPingTime: Date.now(),
+    remoteOutdatedLastPayload: true,
+  }, reporter._state._filename);
+
+  try {
+    yield reporter.init();
+
+    do_check_neq(reporter._state.clientID, null);
+    do_check_eq(reporter._state.clientID.length, 36);
+  } finally {
+    reporter._shutdown();
+  }
+});
+
+function UnicodeMeasurement() {
+  Metrics.Measurement.call(this);
+}
+
+UnicodeMeasurement.prototype = {
+  __proto__: Metrics.Measurement.prototype,
+
+  name: "unicode",
+  version: 1,
+
+  fields: {
+    "last-text": {type: Metrics.Storage.FIELD_LAST_TEXT},
+  },
+};
+
+function UnicodeProvider() {
+  Metrics.Provider.call(this);
+}
+
+UnicodeProvider.prototype = {
+  __proto__: Metrics.Provider.prototype,
+
+  name: "unicode",
+
+  measurementTypes: [UnicodeMeasurement],
+
+  collectConstantData: function () {
+    return this.enqueueStorageOperation(() => {
+      let m = this.getMeasurement("unicode", 1);
+      return m.setLastText("last-text", "ᄃᄄᄅ");
+    });
+  },
+};
+
+// Check for proper handling of Unicode in payload.
+add_task(function* test_unicode_payload() {
+  let [reporter, server] = yield getReporterAndServer("unicode_payload");
+  try {
+    yield reporter._providerManager.registerProviderFromType(UnicodeProvider);
+
+    let deferred = Promise.defer();
+    let request = new DataSubmissionRequest(deferred, new Date());
+    reporter.requestDataUpload(request);
+    yield deferred.promise;
+    Assert.equal(request.state, request.SUBMISSION_SUCCESS);
+    Assert.ok(server.hasDocument(reporter.serverNamespace, reporter.lastSubmitID));
+
+    let p = server.getDocument(reporter.serverNamespace, reporter.lastSubmitID);
+    let v = p.data.last['unicode.unicode']['last-text'];
+    Assert.equal(v, "ᄃᄄᄅ");
+  } finally {
+    yield shutdownServer(server);
+    reporter._shutdown();
+  }
+});
