@@ -47,9 +47,11 @@ public:
   nsTArray<nsString> mValues;
   nsAutoString mSearchString;
   uint16_t mSearchResult;
+private:
+  ~nsFileResult() {}
 };
 
-NS_IMPL_ISUPPORTS1(nsFileResult, nsIAutoCompleteResult)
+NS_IMPL_ISUPPORTS(nsFileResult, nsIAutoCompleteResult)
 
 nsFileResult::nsFileResult(const nsAString& aSearchString,
                            const nsAString& aSearchParam):
@@ -172,6 +174,11 @@ NS_IMETHODIMP nsFileResult::GetImageAt(int32_t index, nsAString & aImage)
   aImage.Truncate();
   return NS_OK;
 }
+NS_IMETHODIMP nsFileResult::GetFinalCompleteValueAt(int32_t index,
+                                                    nsAString & aValue)
+{
+  return GetValueAt(index, aValue);
+}
 
 NS_IMETHODIMP nsFileResult::RemoveValueAt(int32_t rowIndex, bool removeFromDb)
 {
@@ -180,12 +187,13 @@ NS_IMETHODIMP nsFileResult::RemoveValueAt(int32_t rowIndex, bool removeFromDb)
 
 class nsFileComplete MOZ_FINAL : public nsIAutoCompleteSearch
 {
+  ~nsFileComplete() {}
 public:
   NS_DECL_ISUPPORTS
   NS_DECL_NSIAUTOCOMPLETESEARCH
 };
 
-NS_IMPL_ISUPPORTS1(nsFileComplete, nsIAutoCompleteSearch)
+NS_IMPL_ISUPPORTS(nsFileComplete, nsIAutoCompleteSearch)
 
 NS_IMETHODIMP
 nsFileComplete::StartSearch(const nsAString& aSearchString,
@@ -239,7 +247,7 @@ protected:
   int16_t mSortType;
   int32_t mTotalRows;
 
-  nsTArray<PRUnichar*> mCurrentFilters;
+  nsTArray<char16_t*> mCurrentFilters;
 
   bool mShowHiddenFiles;
   bool mDirectoryFilter;
@@ -300,7 +308,7 @@ nsFileView::Init()
 
 // nsISupports implementation
 
-NS_IMPL_ISUPPORTS2(nsFileView, nsITreeView, nsIFileView)
+NS_IMPL_ISUPPORTS(nsFileView, nsITreeView, nsIFileView)
 
 // nsIFileView implementation
 
@@ -485,7 +493,7 @@ nsFileView::SetFilter(const nsAString& aFilterString)
     while (iter != end && (*iter != ';' && *iter != ' '))
       ++iter;
 
-    PRUnichar* filter = ToNewUnicode(Substring(start, iter));
+    char16_t* filter = ToNewUnicode(Substring(start, iter));
     if (!filter)
       return NS_ERROR_OUT_OF_MEMORY;
 
@@ -721,7 +729,7 @@ nsFileView::GetCellText(int32_t aRow, nsITreeColumn* aCol,
     return NS_OK;
   }
 
-  const PRUnichar* colID;
+  const char16_t* colID;
   aCol->GetIdConst(&colID);
   if (NS_LITERAL_STRING("FilenameColumn").Equals(colID)) {
     curFile->GetLeafName(aCellText);
@@ -809,19 +817,19 @@ nsFileView::SetCellText(int32_t aRow, nsITreeColumn* aCol,
 }
 
 NS_IMETHODIMP
-nsFileView::PerformAction(const PRUnichar* aAction)
+nsFileView::PerformAction(const char16_t* aAction)
 {
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFileView::PerformActionOnRow(const PRUnichar* aAction, int32_t aRow)
+nsFileView::PerformActionOnRow(const char16_t* aAction, int32_t aRow)
 {
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsFileView::PerformActionOnCell(const PRUnichar* aAction, int32_t aRow,
+nsFileView::PerformActionOnCell(const char16_t* aAction, int32_t aRow,
                                 nsITreeColumn* aCol)
 {
   return NS_OK;
@@ -854,7 +862,7 @@ nsFileView::FilterFiles()
       for (uint32_t j = 0; j < filterCount; ++j) {
         bool matched = false;
         if (!nsCRT::strcmp(mCurrentFilters.ElementAt(j),
-                           NS_LITERAL_STRING("..apps").get()))
+                           MOZ_UTF16("..apps")))
         {
           file->IsExecutable(&matched);
         } else
