@@ -286,6 +286,16 @@ public:
      */
     eIntID_MacLionTheme,
 
+   /*
+    * A Boolean value to determine whether the Mac OS X Yosemite-specific theming
+    * should be used.
+    *
+    * The value of this metric is not used on non-Mac platforms. These
+    * platforms should return NS_ERROR_NOT_IMPLEMENTED when queried for this
+    * metric.
+    */
+   eIntID_MacYosemiteTheme,
+
     /*
      * eIntID_AlertNotificationOrigin indicates from which corner of the
      * screen alerts slide in, and from which direction (horizontal/vertical).
@@ -548,7 +558,7 @@ public:
    * GetPasswordCharacter() returns a unicode character which should be used
    * for a masked character in password editor.  E.g., '*'.
    */
-  static PRUnichar GetPasswordCharacter();
+  static char16_t GetPasswordCharacter();
 
   /**
    * If the latest character in password field shouldn't be hidden by the
